/***************
 * NAME:            IconDrop - version 1.01
 * DESCRIPTION:     Functions for saving icons
 * AUTHOR:          (c) Mike Riddle - initial version 12-17-23                     
 ***************/


/* CHANGELOG 12-14-21 */

/************************************************************************************************************/

/* IconDrop - Release 12-20-23 (First Version) */
/* IconDrop - 12-06-23 - version 1.01 - Split IconDrop from IconPress using a simple gadtools user interface for png icons */
/* IconDrop - 12-06-23 - version 1.01 - Added second button for "Icon Info" for IconDrop to show new icon information */
/* IconDrop - 12-17-23 - version 1.01 - Based current icondrop on iconpress_11-16-23_os35.c from testing folder */
/* IconDrop - 12-18-23 - version 1.01 - Disabled Save_Amiga_Icon function in GID_SAV2 for ILBM images to save Glow Icons */
/* IconDrop - 12-18-23 - version 1.01 - Need to have Single Image Support for Icon Drop to save Single Image PNG Icons */

/* IconDrop - 12-26-23 - version 1.02 - Need to use Display_Blank method instead of EraseRectangle because of coords issue */
/* IconDrop - 12-27-23 - version 1.02 - Cleaned up source code by removing the working changelog and program notes */
/* IconDrop - 12-28-23 - version 1.02 - Added new functions Adjust_Alpha & Display_Blank to erase previous image */

/************************************************************************************************************/

 
/* RELEASES */

/* IconDrop - 12-20-23 - Version - 1.01 Release - x86 - workbench & commandline */
/* IconDrop - 12-29-23 - Version - 1.02 Release - x86 - workbench & commandline */

 
//IconClone Errors_02-18-23 
//def icons don't work in destination but they work in source display area (fixed)
//icon with bad filename that doesn't exist causes a crash instead of not displaying (fixed)
//icon with bad filename with extra .info causes a crash instead of not displaying (fixed)
//Supress message or WritePixelArrayAlpha in Display_Icon (fixed)
//Catch bad filename at command line & for drag-n-drop (fixed)
//Now we test at command line & for drag-n-drop if file exists. (added)
//Condtitional statements for commandline exchange & open gui use if (error == FALSE). (added)
//In-Rectangel call for Display_Icon also uses if (error == FALSE). (added)

//IconClone Errors_02-18-23
//1. For commandline can't open icons with dot in filename such as those using def icons.
//This happens because RemoveFileExtension interferes with the filename but using add null
//char causes the other icon filenames to not display correctly because of filename issues.
//2. Opening from commandline on Magellan with mismatched icon styles second image displays
//wrong when the first icon image is an OS3.5 Gow Icon & the second image is a png icon.
//When the first icon image is png & the second image an OS3.5 Glow Icon the png displays twice.
//But this only happens at the commandline. Using drag-n-drop is still worning correctly.


/* REFERENCES */

// http://krashan.ppa.pl/articles/amigaicons/ (HTML - Amiga Icon File Format) (no longer active)
// http://www.evillabs.net/index.php/Amiga_Icon_Formats (Text - Dirk Stoecker)
// ilbmtoicon - https://github.com/AmigaPorts/ilbmtoicon (source code only) (includes infoinfo.c)
// all Amiga icons formats are supported except oldicons/newIcons



#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <dos/dos.h>
#include <exec/exec.h>

#include <proto/wb.h>
#include <proto/icon.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <proto/asl.h>
#include <proto/icon.h>
#include <proto/graphics.h>
#include <proto/workbench.h>
#include <proto/intuition.h>
#include <proto/datatypes.h>
#include <proto/gadtools.h>
#include <proto/iffparse.h>
#include <libraries/iffparse.h>


#include <workbench/icon.h>
#include <workbench/workbench.h>
#include <workbench/startup.h>
#include <intuition/intuition.h>
#include <datatypes/datatypes.h>
#include <datatypes/pictureclass.h>
#include <datatypes/datatypesclass.h>
#include <proto/cybergraphics.h>
#include <cybergraphx/cybergraphics.h>

 
#define GID_SAV1  4714 
#define GID_SAV2  4715
//#define GID_REQ1  4716 
//#define GID_REQ2  4717 
//#define GID_EXCH  4718

#define MAX_IMAGE_WIDTH 80
#define MAX_IMAGE_HEIGHT 80
#define MAXBUFFERSIZE 100000
#define MAXICONSIZE 256
#define MAXDEPTH 8
#define MAXCOLORS 256

#define OK 0
#define NOT_OK 1

#define BOOL_YES 0x2A2A2A2A
#define BOOL_NO  0x00000000

#define ID_FORM MAKE_ID('F','O','R','M')
#define ID_ICON MAKE_ID('I','C','O','N')
#define ID_FACE MAKE_ID('F','A','C','E')
#define ID_IMAG MAKE_ID('I','M','A','G')
#define ID_ARGB MAKE_ID('A','R','G','B')

/* Values for ICONCTRLA_GetImageDataFormat */
#define ICONCTRLA_SetImageDataFormat        (ICONA_BASE + 0x67) /*103*/
#define ICONCTRLA_GetImageDataFormat        (ICONA_BASE + 0x68) /*104*/

/* Values for ICONCTRLA_Set/GetImageDataFormat */
#define IDFMT_BITMAPPED     (0)  /* Bitmapped icon (planar, legacy) */
#define IDFMT_PALETTEMAPPED (1)  /* Palette mapped icon (chunky, V44+) */
#define IDFMT_DIRECTMAPPED  (2)  /* Direct mapped icon (truecolor 0xAARRGGBB, V51+) */

/* Defines for Get Data Functions - based on modifyicon */
#define GetUint32(a)  (((((unsigned char *)a)[0])<<24)|((((unsigned char *)a)[1])<<16)| \
                       ((((unsigned char *)a)[2])<< 8)|((((unsigned char *)a)[3])))
#define GetUint16(a)  (((((unsigned char *)a)[0])<< 8)|((((unsigned char *)a)[1]))) //scrwidth = filebuf[6] | filebuf[7] << 8;
//#define GetUint8(a) ((unsigned char*)a[0]) //doesn't work

/* Defines for Put Data Functions - based on modifyicon */
#define PutUint32(a, b) {unsigned long epu32 = (b); (a)[0] = (unsigned char) (epu32 >> 24); \
                         (a)[1] = (unsigned char) (epu32 >> 16); \
                         (a)[2] = (unsigned char) (epu32 >> 8); \
                         (a)[3] = (unsigned char) epu32;}
#define PutUint16(a, b) {unsigned short epu16 = (b); (a)[0] = (unsigned char) (epu16 >> 8); \
                         (a)[1] = (unsigned char) epu16;}
#define PutUint32Inc(a, b) {PutUint32(a,b); (a) += 4;}
#define PutUint16Inc(a, b) {PutUint16(a,b); (a) += 2;}

//#define GetUint16BE(a)  (((((unsigned char *)a)[0]))|((((unsigned char *)a)[1])<< 8))
//#define GetUint16LE(a)  (((((unsigned char *)a)[0])>> 8)|((((unsigned char *)a)[1])))

#define RGB8to32(RGB) (((ULONG)(RGB) << 24)|((ULONG)(RGB) << 16)|((ULONG)(RGB) << 8)|((ULONG)(RGB)))

int count = 0;
int action = 0;
WORD topoffset;
WORD leftoffset;
WORD deltasize;
WORD windowsize;
WORD buttonoffset;

BOOL open = FALSE;
BOOL help = FALSE;
BOOL verbose = TRUE;

//STRPTR srcename = NULL;
//STRPTR destname = NULL; 

//STRPTR iconfile = NULL;
//STRPTR srcefile = NULL;
//STRPTR destfile = NULL;
//STRPTR tempfile = NULL;
//STRPTR srcepath = NULL;

static int icontype = 3;

char *imagetype = NULL;
STRPTR filename = NULL;
STRPTR left_imagetype = NULL;
STRPTR right_imagetype = NULL;

STRPTR outfilename = NULL;
STRPTR infilename1 = NULL;
STRPTR infilename2 = NULL;
STRPTR tempfilename = NULL;

/* 'ticks' per dot, corresponding to ~72dpi */
//static ULONG tpdX = TPD_X(72), tpdY = TPD_Y(72);
static char *image1option;
static char *image2option;
static char *defaulttooloption;
static char *drawerdataoption;
static char **tooltypesoption;
static LONG typeoption = 3; /* WBTOOL */
//1 DISK                  a disk /* WBDISK */
//2 DRAWER                a directory   /* WBDRAWER */
//3 TOOL                  a program     /* WBTOOL */
//4 PROJECT               a project with defined start program /* WBPROJECT */
//5 GARBAGE               the trashcan, rare
//6 DEVICE                should never appear
//7 KICK                  a kickstart disk, rare
//8 APPICON               should never appear
static LONG iconleftoption = 0x80000000; /* NO_ICON_POSITION */
static LONG icontopoption = 0x80000000; /* NO_ICON_POSITION */
static LONG stackoption = 4096;
static LONG drawerleftoption = 50;
static LONG drawertopoption = 50;
static LONG drawerwidthoption = 400;
static LONG drawerheightoption = 100;
static LONG drawervleftoption = 0;
static LONG drawervtopoption = 0;
static LONG drawershowoption = 0;
static LONG drawershowasoption = 0;
static LONG transparentoption = -1;

/* Setup Variables for Load_ILBM */
static FILE 	     	    *file;
static struct ILBMImage     img1, img2;
static unsigned char	    *filebuffer;
//static unsigned char	    *filebuffer, *body;
static BOOL 	    	    have_bmhd, have_cmap, have_body;
//static BOOL 	    	    has_bmhd, has_cmap, has_body;

/* The List of Active Gadgets */
struct Gadget *glist = NULL, *butgad1, *butgad2, *butgad3;

/* Setup File Requesters */
struct FileRequester *SrcFileReq  = NULL, *DestFileReq = NULL;


/****************************************************************************************/

struct FileFaceChunk
{
    UBYTE Width;
    UBYTE Height;
    UBYTE Flags;
    UBYTE Aspect;
    UBYTE MaxPaletteBytes[2];
};

struct FileImageChunk
{
    UBYTE TransparentColor;
    UBYTE NumColors;
    UBYTE Flags;
    UBYTE ImageFormat;
    UBYTE PaletteFormat;
    UBYTE Depth;  
    UBYTE NumImageBytes[2];
    UBYTE NumPaletteBytes[2];
};

struct IconInfo 
{
	ULONG Width;
	ULONG  Height;
	ULONG  Format;
	ULONG  NumColors;
	const struct ColorRegister *CMap;
	UBYTE *Image;
	ULONG  Which;
	ULONG  NumImages;
};

struct Palette
{
    UWORD numentries;
    UBYTE rgb[][3];
};

//struct IconImage = struct ILBMImage

struct ILBMImage
{
    struct BitMapHeader bmh;
    struct BitMapHeader planarbmh;
    unsigned char   	*planarbuffer, *chunkybuffer;	
    LONG    	    	cmapentries, bpr, totdepth;
    UBYTE   	    	rgb[256][3];
    UBYTE   	    	remaptable[256];
    APTR                png;
    ULONG               png_size;
    APTR                argb;
    ULONG               argb_size;
};

/****************************************************************************************/

BOOL EXISTS(char *fname);
char *str_dupe(char *src);
char *get_ext(char* path);
char *GetPath(char *path);
LONG GetFileSize(char *imName);
LONG Get_IFFOffset(char *srcename);

char *RemoveFileExtension(char *path);
char* GetFileNameFromPath(char* path);
void SetFilesize(char *name, LONG newsize);
//char *Check_FileType(char *srcename);
//char *Check_Icon_Type(char *srcename);
//static LONG Write_ImagChunk(char *imagefile);
static LONG ReadLong(BPTR file, LONG offset);
static WORD ReadWord(BPTR file, LONG offset);
static void WriteWord(BPTR file, WORD data, LONG offset);
static void WriteLong(BPTR file, LONG data, LONG offset);
char *Check_Image_Signature(char *imagename);
//char *Check_Icon_Signature(char *srcename);
//BOOL Check_Image_Match(char *srcename, char *destname);
BOOL Check_Image_Sizes(char *srcename, char *destname);
char *Check_File_Signature(char *srcename, BOOL isIcon);
static void FreeIconImage(struct ILBMImage *img);
struct BitMapHeader *readbmhd(char *file_name);
char *Check_IFFType(char *srcename, LONG iffoffset);
const char *DetectBaseName_by_datatype(char *file_name);
void OutputMessage(STRPTR outputmsg, struct Window *win);
//STRPTR GetFilename(char * asl_title, struct Window *win);
//void Make_Icon(char *image1, char *image2, char *outfile);
BOOL inRectangle(int x,int y, int w, int h,int tx, int ty);

//void Copy_Icon(char *destname);
struct BitMapHeader *readbmhd_by_datatype(char *file_name);
STRPTR GetSourceFilename(char * asl_title, struct Window *win);
STRPTR GetDestinationFilename(char * asl_title, struct Window *win);
BOOL ReadPixels_by_datatype(UBYTE *pixels, char *file_name, int bpp);
//BOOL ReadPixels_by_datatype_lut8(struct IconImage *im, char *file_name);
//BOOL ReadPixels_by_datatype_alpha(struct IconImage *im, char *file_name);
static void Load_ILBM(char *imagename, struct ILBMImage *img);
static BOOL Save_ILBM(char *imagename, struct ILBMImage *img);
BOOL ReadImage (UBYTE *pixels, struct IconInfo info, int bpp);
static long Find_High_Index(unsigned char *colors, long length);
//BOOL ReadOS35_by_datatype(UBYTE *pixels, char *file_name, int bpp);
//BOOL ReadOS35_by_datatype(UBYTE *pixels, struct IconInfo info, int bpp);
//BOOL Save_Icon(struct DiskObject *oldicon, struct DiskObject *newicon, STRPTR name);
BOOL Save_Icon(char *infile1, char *infile2, char *outfile);
//void Write_DiskObject(struct BitMapHeader *bmhd, BPTR outfile);
BOOL Save_Classic_Icon(char *infile1, char *infile2, char *outfile, BOOL argb); //BOOL os41
//static void Write_OS35data(char *infile1, char *infile2, char *outfile);
static void Write_Image(BPTR outfile, struct ILBMImage *img, LONG offset);
static void Write_OS35data(char *infile1, char *infile2, char *outfile, LONG offset);
BOOL Save_Amiga_Icon(char *infile1, char *infile2, char *outfile, char *iconfile, int icontype);
BOOL SavePixels_by_datatype(ULONG *data, char *filename, int width, int height);
static unsigned char *UnpackByteRun(unsigned char *source, unsigned char *dest, LONG unpackedsize);
//static BOOL pack_byterun1(unsigned char *source, unsigned char *dest, LONG size, LONG check_size, LONG *packsize);
static void WriteData(char *new_filename, unsigned char *data, LONG datasize);
static void Write_DiskObject (unsigned char *buf, int width, int height);
void Copy_Data(char *srcename, char *destname, LONG filebytes);
void Copy_File(char *srcename, char *destname, LONG filebytes);
void Copy_Data_Append(char *srcename, char *destname, LONG offset, LONG filebytes);
//void Exchange_Icon(char *srcename, char *destname, WORD deltasize, struct Window *win);
//BOOL Display_Icon(char *srcename, char *position, WORD deltasize, struct Window *win);
static void Adjust_Alpha(UBYTE *Data, int width, int height);
BOOL Display_Blank(char *imagename, char *position, WORD deltasize, struct Window *win);
BOOL Display_Image(char *srcename, char *position, WORD deltasize, struct Window *win);
void Display_Rectangle(char *color, char *position, WORD deltasize, struct Window *win);
void Exchange_Icon(struct DiskObject *dobj1, struct DiskObject *dobj2, char *destname, WORD deltasize, struct Window *win);
static void MakePlanar(unsigned char *inbuffer, unsigned char *outbuffer, LONG width, LONG height, LONG depth);
static int ChunkyToBitplanes(UBYTE *chunkyBuffer, UBYTE *bitBuffer, LONG bitPlanes, LONG imagewidth);
static void BitplanesToChunky(unsigned char *source, unsigned char *dest, LONG width, LONG height,
    	    	LONG totplanes, LONG wantplanes, LONG chunkybpr);
static LONG MyMemStreamHandler(struct Hook *hook, struct IFFHandle * iff, struct IFFStreamCmd * cmd);
static char *createrle(unsigned long depth, unsigned char *dtype, LONG *dsize, unsigned long size, unsigned char *src);
static char *EncodeRLE(unsigned long depth, unsigned char *dtype, LONG *dsize, unsigned long len, unsigned char *srce);


/*********************************************************************************************/

BOOL EXISTS(char *fname)
{
    BPTR file;    
    file = Open(fname, MODE_OLDFILE);
	if (!file) 
	{
		Close(file);
        return TRUE;
	}
    return FALSE;
}

BOOL inRectangle(int x,int y, int w, int h,int tx, int ty)
{
  if (tx >=x  && tx <= x+w && ty >=y && ty <= y+h) return(TRUE); else return(FALSE);
}

static LONG MyMemStreamHandler(struct Hook *hook, struct IFFHandle * iff, struct IFFStreamCmd * cmd)
{
    LONG error = 0;
    APTR *buffp = (APTR *)iff->iff_Stream;
    APTR buff = *buffp;

    switch(cmd->sc_Command)
    {
    	case IFFCMD_INIT:
	case IFFCMD_CLEANUP:
	    error = 0;
	    break;
	    
	case IFFCMD_READ:
	    error = 0;
	    CopyMem(buff, cmd->sc_Buf, cmd->sc_NBytes);
	    (*buffp) += cmd->sc_NBytes;
	    break;
	    
	case IFFCMD_WRITE:
	    error = 0;
	    CopyMem(cmd->sc_Buf, buff, cmd->sc_NBytes);
	    (*buffp) += cmd->sc_NBytes;
	    break;
	    
	case IFFCMD_SEEK:
	    (*buffp) += cmd->sc_NBytes;
	    error = 0;
	    break;
    }
    
    return error;
}

/**************************************************************************************************/

static LONG ReadLong(BPTR file, LONG offset)
{
    //WORD ret; //LONG ??
    LONG ret;
    UBYTE Buffer32[4];
    
    Seek(file,offset,OFFSET_BEGINNING);
	Read(file,Buffer32, 4);	    		
	ret = GetUint32(Buffer32); 
    
    return ret;
}

/**************************************************************************************************/

static WORD ReadWord(BPTR file, LONG offset)
{
    WORD ret;
    UBYTE Buffer16[2];
    
    Seek(file,offset,OFFSET_BEGINNING);
	Read(file,Buffer16, 2);	    		
	ret = GetUint16(Buffer16); 
    
    return ret;
}

/**************************************************************************************************/

static void WriteWord(BPTR file, WORD data, LONG offset)
{    
	//FILE *outfile;
	//WORD data;
	//UBYTE wd[2];
	UBYTE Buffer16[2]; 

	//wd[0] = (data >> 8) & 0xFF;
    //wd[1] = data & 0xFF;   
 
    //if (fwrite(Buffer16, 1, 2, outfile) != 2)
	//if (fwrite(wd, 1, 2, outfile) != 2)
    //{
    	//OutputMessage("Error writing word value!"\n, win);
    //}

	Seek(file,offset,OFFSET_BEGINNING);
	PutUint16(Buffer16, data);
	Write(file,Buffer16,2);    
}

/**************************************************************************************************/

static void WriteLong(BPTR file, LONG data, LONG offset)
{    
	//FILE *outfile;
	//LONG data;
	//UBYTE lg[4];
	UBYTE Buffer32[4];

    //lg[0] = (l >> 24) & 0xFF;
    //lg[1] = (l >> 16) & 0xFF;
    //lg[2] = (l >> 8) & 0xFF;
    //lg[3] = l & 0xFF;
 
    //if (fwrite(lg, 1, 4, outfile) != 4)
    //{
    	//OutputMessage("Error writing long value!"\n, win);
    //}

	Seek(file,offset,OFFSET_BEGINNING);
	PutUint32(Buffer32, data);
	Write(file,Buffer32,4);    
}

/**************************************************************************************************/

static void WriteData(char *new_filename, unsigned char *data, LONG datasize)
{  
    BPTR fh = Open( new_filename, MODE_NEWFILE );
	if(fh)
	{        
	    Seek(fh,0,OFFSET_BEGINNING);
        Write(fh, data, datasize);
	}	
    Close( fh );
}

/**************************************************************************************************/

void OutputMessage(STRPTR msg, struct Window *win)
{
    struct EasyStruct es;    
   
    if (msg)
    {        
        es.es_StructSize   = sizeof(es);
        es.es_Flags        = 0;
        es.es_Title        = "Message";
        es.es_TextFormat   = msg;
        es.es_GadgetFormat = "OK";
           
        EasyRequestArgs(win, &es, NULL, NULL);        
    }
}

/*********************************************************************************************/

LONG Get_IFFOffset(char *srcename)
{	
	LONG offset;
	LONG i, j, k;	
	UWORD FileID;	
	UBYTE Buffer16[2];
	UBYTE Buffer32[4];	
	WORD width, height, depth;
	int imagesize, imageoffset;

	//Printf("\nGet_IFFOffset...\n");

	LONG firstimage, secondimage;	
	LONG tooltypes, deftool, drawerdata, toolwin;

	/* Open icon file to Read */
    BPTR fh = Open(srcename, MODE_OLDFILE);
    if (fh)
	{
		offset = 0;        		  

        Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer16, 2);	    		
		if( Buffer16[0] == 0xE3 && Buffer16[1] == 0x10 ) 
		//Printf ("Found Classic Icon File!\n");

		offset += 2;
		Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer16, 2);		
		FileID = GetUint16(Buffer16);
		if(FileID == 1)
		//Printf("Found Valid Version Number!\n");

		offset += 20; //22
		Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer32, 4);		
		//firstimage = (GetUint32(Buffer32)>0) ? 1: 0; //22
		firstimage = (GetUint32(Buffer32)==0) ? 0: 1; //22
		//Printf ("First Image: %ld, %ld\n", firstimage, offset);

		offset += 4; //26
		Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer32, 4);
		//secondimage = (GetUint32(Buffer32)>0) ? 1: 0; //26
		secondimage = (GetUint32(Buffer32)==0) ? 0: 1; //26		
		//Printf ("Second Image: %ld, %ld\n", secondimage, offset);

		offset += 24; //26+24=50
		Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer32, 4);
		//deftool = (GetUint32(Buffer32)>0) ? 1: 0; //50
		deftool = (GetUint32(Buffer32)==0) ? 0: 1; //50
		//Printf ("DiskObject - deftool: %ld, %ld\n", deftool, offset);
		offset += 4; //54
		Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer32, 4);
		//tooltypes = (GetUint32(Buffer32)>0) ? 1: 0; //54
		tooltypes = (GetUint32(Buffer32)==0) ? 0: 1; //54
        //Printf ("DiskObject - tooltypes: %ld, %ld\n", tooltypes, offset);
		offset += 12; //66
		Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer32, 4);
        //drawerdata = (GetUint32(Buffer32)>0) ? 1: 0; //66
		drawerdata = (GetUint32(Buffer32)==0) ? 0: 1; //66
		//Printf ("DiskObject - drawerdata: %ld, %ld\n", drawerdata, offset);        
		offset += 4; //70
		Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer32, 4);
		//toolwin = (GetUint32(Buffer32)>0) ? 1: 0; //70
		toolwin = (GetUint32(Buffer32)==0) ? 0: 1; //70
		//Printf ("DiskObject - toolwin: %ld, %ld\n", toolwin, offset);
		offset += 8; //64+14=78
		//Printf("Current Offset: %ld\n", offset); //78 ??

		if(drawerdata == 1)
        {		    
		    offset += 56; //78+56=134
		}
		if(firstimage == 1)
		{
			offset += 4;
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer16, 2);
            width = GetUint16(Buffer16); //offset + 4			           
			//Printf ("First Image Width: %ld, 0x%x%x, %ld\n", width, Buffer16[0],Buffer16[1], offset);

			offset += 2;
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer16, 2);
			height = GetUint16(Buffer16); //offset + 6			
			//Printf ("First Image Height: %ld, 0x%x%x, %ld\n", height, Buffer16[0],Buffer16[1], offset);

			offset += 2;
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer16, 2);
			depth = GetUint16(Buffer16); //offset + 8			
			//Printf ("First Image Depth: %ld, 0x%x%x, %ld\n", depth, Buffer16[0],Buffer16[1], offset);

			offset += 12; //offset + 20
			imagesize = (((width+15)&(~15)) * height);
			//imageoffset = (((((width+15)&(~15)) * height) * depth)>>3);
			imageoffset = ((imagesize * depth)>>3);

			//Printf ("First Image Size: %ld, %ld\n", imageoffset, offset);
			offset += imageoffset;
		}
		if(secondimage == 1)
		{
			offset += 4;
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer16, 2);
            width = GetUint16(Buffer16); //offset + 4			           
			//Printf ("Second Image Width: %ld, 0x%x%x, %ld\n", width, Buffer16[0],Buffer16[1], offset);

			offset += 2;
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer16, 2);
			height = GetUint16(Buffer16); //offset + 6			
			//Printf ("Second Image Height: %ld, 0x%x%x, %ld\n", height, Buffer16[0],Buffer16[1], offset);

			offset += 2;
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer16, 2);
			depth = GetUint16(Buffer16); //offset + 8			
			//Printf ("Second Image Depth: %ld, 0x%x%x, %ld\n", depth, Buffer16[0],Buffer16[1], offset);
			
			offset += 12; //offset + 20
			imagesize = (((width+15)&(~15)) * height);
			//imageoffset = (((((width+15)&(~15)) * height) * depth)>>3);
			imageoffset = ((imagesize * depth)>>3);						

			//Printf ("Second Image Size: %ld, %ld\n", imageoffset, offset);
			offset += imageoffset;
		}
		if(deftool == 1)
        {
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer32, 4);
			i = GetUint32(Buffer32);            
			offset += i+4;
			//Printf("Current Offset: %ld\n", offset);
		}
		if(tooltypes == 1)
		{
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer32, 4);
			i = GetUint32(Buffer32)/4-1;
			offset += 4;
			
			for(j = 0; j < i; ++j)
            {
				Seek(fh,offset,OFFSET_BEGINNING);
		        Read(fh,Buffer32, 4);
                k = GetUint32(Buffer32);
				offset += k+4;
            }
			//Printf("Current Offset: %ld\n", offset);
		}
		if(toolwin == 1)
        {
			Seek(fh,offset,OFFSET_BEGINNING);
		    Read(fh,Buffer32, 4);
            i = GetUint32(Buffer32);        
		    offset += i+4; 
			//Printf("Current Offset: %ld\n", offset);
        }
		/* Check for Beginning of OS3.5 IFF Data */
		Seek(fh,offset,OFFSET_BEGINNING);
		Read(fh,Buffer32, 4); //0x464F524D) = 'FORM'		
		if( Buffer32[0] == 0x46 && Buffer32[1] == 0x4F && Buffer32[2] == 0x52 && Buffer32[3] == 0x4D)
		{		
		    //Printf("Found Beginning of OS3.5 IFF Data: %ld\n", offset);
			return offset;
		}
		else
		{
			/* Check Again! */
			offset += 6; //Skip 6 zero bytes at end
			Seek(fh,offset,OFFSET_BEGINNING);
			Read(fh,Buffer32, 4);
			if( Buffer32[0] == 0x46 && Buffer32[1] == 0x4F && Buffer32[2] == 0x52 && Buffer32[3] == 0x4D)
		    {		
		        //Printf("Found Beginning of OS3.5 IFF Data: %ld\n", offset);
				return offset;
		    }
		}
	}
	Close(fh);	
    //Printf("Current Offset: %ld\n", offset);

	return offset;
}

char *Check_IFFType(char *srcename, LONG iffoffset)
{
    //Printf("\nCheck IFFType...\n");    
	
    //See: http://krashan.ppa.pl/articles/amigaicons/

	//78[diskobject]+(56)[drawerdata]=0?
    //20[imageheader]
    //2bpl 46~16=48x46=552[((w~16)*h)*bitplanes/8]
    //20[imageheader]
    //2bpl 46~16=48x46=552[((w~16)*h)*bitplanes/8]
	//[deftool]=0?
	//[tooltypes]=0?
    //FORM+4+ICONFACE

	//See: https://wiki.amigaos.net/wiki/IFFParse_Library

	//Remember to open the IFFParse.Library before using IFFParse code.
	//#define IFFParseBase	DemandOpenLibrary((struct Library **)&IconBase->ib_IFFParseBase, "iffparse.library", IFFParseBase_version)

	/* Code from AROS Icon Library */
	static const LONG stopchunks[] =
    {
        ID_ICON, ID_FACE,
        ID_ICON, ID_IMAG,        
        ID_ICON, ID_ARGB,
    };

	struct IFFHandle 	    *iff;
    struct Hook     	     iffhook;
    struct FileFaceChunk     fc;
    struct FileImageChunk    ic1, ic2;
    //UBYTE                   *argb1data = NULL, *argb2data = NULL;
    //UBYTE   	    	    *img1data = NULL, *img2data = NULL;
    //struct ColorRegister    *img1pal = NULL, *img2pal = NULL;
    BOOL  have_face = FALSE, have_imag1 = FALSE, have_imag2 = FALSE;
    BOOL                     have_argb1 = FALSE, have_argb2 = FALSE;
    //int have_png = 0;
    //LONG here, extrasize;
    APTR data;
	//UBYTE *dataBuffer;

	iffhook.h_Entry    = (HOOKFUNC)HookEntry;
    iffhook.h_SubEntry = (HOOKFUNC)MyMemStreamHandler;
    iffhook.h_Data     = (APTR)IconBase;

    
    //Printf("ReadIcon35 - Filename: %s\n", srcename);

	//if (IFFParseBase == NULL)
    	//return FALSE;

	//Printf ("ReadIcon35 - ReadIFF Working!\n");

	/* Get total icon file size */
	LONG imSize = GetFileSize(srcename);
	//Printf("Temp Icon FileSize: %ld\n", imSize);
	//unsigned char buffer[imSize];
	data = (UBYTE *)AllocVec(imSize-iffoffset, MEMF_ANY);    


    /* Open icon file to Read */
    BPTR fh = Open(srcename, MODE_OLDFILE);
    if (fh)
	{		
		//iffoffset = 1222;
		//**Seek Offset ??
		Seek(fh,iffoffset,OFFSET_BEGINNING); 
		Read(fh,data,imSize-iffoffset);

        //Using IFFParse_Library		
		if (iff = AllocIFF())
		{
			LONG error;
			//Printf("ReadIcon35. AllocIFF okay\n");			
    	    //iff->iff_Stream = (IPTR)fh; //Can be BPTR or pointer to data buffer
			iff->iff_Stream = (IPTR)&data;
			//iff->iff_Stream = Open(srcename, MODE_OLDFILE);

			//iff->iff_Stream = (ULONG) IDOS->Open ("filename", MODE_OLDFILE);
            //if(iff->iff_Stream) IIFFParse->InitIFFasDOS (iff);  /* use internal DOS stream manager */

			//Note: When using a custom handle, or a device other than the clipboard, you must provide your own flags and stream handler:
			//iff->iff_Stream = (ULONG) OpenMyCustomStream("foo");
            //IIFFParse->InitIFF (iff, IFFF_FSEEK | IFFF_RSEEK, &mystreamhook);
	
	        //Printf("ReadIcon35. InitIFF as iffHook\n");
	        InitIFF(iff, IFFF_RSEEK, &iffhook);
			//InitIFFasDOS(iff);			

            //After your stream is initialized, call OpenIFF():
            //error = IIFFParse->OpenIFF (iff, IFFF_READ);
			//Once you establish a read/write mode (by passing IFFF_READ or IFFF_WRITE), you remain in that mode until you call CloseIFF().

			//See: https://wiki.amigaos.net/wiki/Parsing_IFF

			//You can instruct the parser to stop when it encounters a specific IFF chunk by using the function StopChunk():
			//error = IIFFParse->StopChunk(iff, ID_ILBM, ID_BODY);

            //Printf("ReadIcon35. Try to OpenIFF\n");
			//if ((error = OpenIFF(handle, IFFF_READ)) == 0)			
			if (!OpenIFF(iff, IFFF_READ))			
			{				
                //Printf("ReadIcon35. OpenIFF okay\n"); 
				//if (!StopChunks(iff, stopchunks, 4))
				if (!StopChunks(iff, stopchunks, 3))
				//if (!StopChunk(iff, ID_ICON, ID_FACE))
				//if ((error = StopChunk(handle, ID_ICON, ID_FACE)) == 0)
				//if (!StopChunks(iff, stopchunks, 3))
				{							
	    	        //Printf("ReadIcon35. StopChunks okay\n");

                    //if ((error = OpenIFF(handle, IFFF_READ)) == 0)
					while(!(error = ParseIFF(iff, IFFPARSE_SCAN)))
					{
						struct ContextNode *cn;		    
		                cn = CurrentChunk(iff);

						//You can obtain a pointer to the current ContextNode through the function CurrentChunk():

						//struct ContextNode {
                        //struct MinNode  cn_Node;
                        //LONG            cn_ID;
                        //LONG            cn_Type;
                        //LONG            cn_Size;        /*  Size of this chunk             */
                        //LONG            cn_Scan;        /*  # of bytes read/written so far */
                        /*  There are private fields hiding here.  */
                        //};

    	    	        //Printf("inside ParseIFF loop\n"); //IFFParser stops at first Chunk: 'ID_FACE' 
						if ((cn->cn_ID == ID_FACE) && (cn->cn_Size >= sizeof(struct FileFaceChunk)))
		                {
							//When the parser encounters the requested chunk (stopchunk), parsing will stop, and ParseIFF() will return the value zero. 
							//The stream will be positioned ready to read the first data byte in the chunk. You may then call ReadChunkBytes() or 
							//ReadChunkRecords() to pull the data out of the current chunk.

							//You may call StopChunk() any number of times for any number of different chunk types. If you wish to identify the chunk on which you’ve stopped, 
							//you may call CurrentChunk() to get a pointer to the current ContextNode, and examine the cn_Type and cn_ID fields.
                            //Using StopChunk() for every chunk, you can parse IFF files in a manner very similar to the way you’re probably doing it now with manual methods.

		    	            //Printf("ReadIcon35. Found FACE chunk\n");
		    	            if (ReadChunkBytes(iff, &fc, sizeof(fc)) == sizeof(fc))
			                {
			                    have_face = TRUE;
			                }
						}
						else if (have_face && (cn->cn_ID == ID_IMAG) && (cn->cn_Size >= sizeof(struct FileImageChunk)))
						{
							//Printf("ReadIcon35. Found IMAG chunk\n");
							if (ReadChunkBytes(iff, (have_imag1 ? &ic2 : &ic1), sizeof(ic1)) == sizeof(ic1))
			                {
			                    if (have_imag1)
			                    {
			    	                //if (ReadImage35(&icon->ni_DiskObject, iff, &fc, &ic2, &img2data, &img2pal, IconBase))
				                    //{
			    	                    have_imag2 = TRUE;
				                    //}
			                    }
			                    else
			                    {
			    	                //if (ReadImage35(&icon->ni_DiskObject, iff, &fc, &ic1, &img1data, &img1pal, IconBase))
				                    //{
			    	                    have_imag1 = TRUE;
				                    //}
			                    }			    
			                }
						}
						else if (cn->cn_ID == ID_ARGB)
						{
							//Printf("ReadIcon35. Found ARGB chunk\n");
							if (have_argb1)
                            {
                                //if (ReadARGB35(&icon->ni_DiskObject, iff, &fc, cn->cn_Size, &argb2data, IconBase))
                                //{
                                    have_argb2 = TRUE;
                                //}
                            }
                            else
                            {
                                //if (ReadARGB35(&icon->ni_DiskObject, iff, &fc, cn->cn_Size, &argb1data, IconBase))
                                //{
                                have_argb1 = TRUE;
                                //}
			                }
						}
						//else if (cn->cn_ID == ID_png)
						//{
							//if (have_png < 2) 
							//{
                                    //LONG here = data - icon->ni_Extra.Data;
                                    //Printf("[%s] Found PNG image %d\n", __func__, have_png);
                                    //icon->ni_Extra.PNG[have_png].Offset = here;
                                    //icon->ni_Extra.PNG[have_png].Size = cn->cn_Size;
                            //} 
							//else 
							//{
                                //Printf("[%s] Ignoring PNG image %d\n", __func__, have_png);
                            //}
                            //have_png++;
						//}
						else 
						{
		                    //Printf("[%s] Unknown chunk ID 0x%08x\n", __func__, cn->cn_ID);
                        }			
		                //} // else if ((cn->cn_ID == ID_FACE)

					} /* while(!ParseIFF(iff, IFFPARSE_SCAN)) */

				} /* if (!StopChunks(iff, stopchunks, 4)) */				
				CloseIFF(iff);
			} /* if (!OpenIFF(iff, IFFF_READ)) */
			//if (iff->iff_Stream) Close (iff->iff_Stream);			
			FreeIFF(iff);			

		} /* if ((iff = AllocIFF())) */

		/* Only use the Width and Height fields of a FACE section
        * if it also contains image data. */
	}
	Close(fh);
	//FreeVec(data);

	/* Chech return types for icon images */
	//have_imag1 = TRUE; && have_imag2 = TRUE;
	//have_argb1 = TRUE; && have_argb2 = TRUE;
	if (have_argb1 == TRUE && have_argb2 == TRUE)
	{
		if (have_imag1 == TRUE && have_imag2 == TRUE)
		    return "argb";
		else if (have_imag1 == FALSE && have_imag2 == FALSE)
		    return "os41";
	}
	else if (have_argb1 == TRUE && have_argb2 == FALSE)
	{
        if (have_imag1 == TRUE) return "argb";
	}
	else if (have_imag1 == TRUE && have_imag2 == TRUE)
	{
        return "os35";
	}
	        

	return "none";
}

/*********************************************************************************************/

BOOL Check_Image_Sizes(char *srcename, char *destname)
{
	int width1, height1, depth1;
	int width2, height2, depth2;			
    struct BitMapHeader *bmhd1, *bmhd2;

	/* Find Width, Height & Depth of Image */
	bmhd1 = readbmhd_by_datatype(srcename);
	width1 = bmhd1->bmh_Width;
	height1 = bmhd1->bmh_Height;
	depth1 = bmhd1->bmh_Depth;

	/* Find Width, Height & Depth of Image */
	bmhd2 = readbmhd_by_datatype(destname);
	width2 = bmhd2->bmh_Width;
	height2 = bmhd2->bmh_Height;
	depth2 = bmhd2->bmh_Depth;

	if(width1 != width2) return FALSE;
    if(height1 != height2) return FALSE;
	if(depth1 != depth2) return FALSE;

	return TRUE;
}

/*********************************************************************************************/

void Display_Rectangle(char *color, char *position, WORD deltasize, struct Window *win)
{			
	LONG newpen;
	UWORD srce_x, srce_y;
	UWORD center_x, center_y;
    int width, height, depth;
	LONG x1, y1, x2, y2;
    struct RastPort *rastPort = win->RPort;	


	Printf ("Display Rectangle: %s\n\n", position);

	/* GET COLOR */

	if(stricmp(color,"red") == 0)
	{
        //newpen = ObtainBestPenA(win->WScreen->ViewPort.ColorMap, 0, 0, 0, 0); //black
		newpen = ObtainBestPenA(win->WScreen->ViewPort.ColorMap, 0xffffffff, 0x00000000, 0x00000000, 0); //red
		SetAPen(rastPort, newpen);
	}
	else if(stricmp(color,"green") == 0)
	{        
		newpen = ObtainBestPenA(win->WScreen->ViewPort.ColorMap, 0x41414141, 0xD7D7D7D7, 0x00000000, 0); //green
		SetAPen(rastPort, newpen);
	}

    /* DRAW COLORED RECTANGLE */

	/* Centered ? */
	center_x = ((80-width)/2);
	center_y = ((80-height)/2);

    /* Get coordinates */		
	if(stricmp(position,"left") == 0)
	{
		/* Top left of First Raised Panel */	    
		srce_x = 22 + center_x + deltasize;
	    srce_y = 25 + topoffset + center_y + deltasize;
		Printf ("Display Image %s srce_x= %d\n", position, srce_x);
		Printf ("Display Image %s srce_y= %d\n", position, srce_y);	

		/* Erase Display Area Left */ //EraseRect( rp, xmin, ymin, xmax, ymax) a1  d0:16 d1:16 d2:16 d3:16
		//EraseRect(win->RPort, leftoffset + 13 + 1 + deltasize, win->BorderTop + topoffset + 31 + deltasize,
        //win->Width - 109 - win->BorderRight - deltasize,
        //win->Height - 50 - win->BorderBottom - deltasize);

		//x1=srce_x;
		//y1=srce_y;
		y1 = topoffset + center_y + deltasize - 12; //25-37=-12
		//x2=x1+77;
		y2=y1+77;
		x1 = leftoffset + 13 + 1 + deltasize; //xmin
		//y1 = win->BorderTop + topoffset - 3 + deltasize; //ymin
		//y1 = win->BorderTop + topoffset + 1 + deltasize; //ymin
		x2 = win->Width - 109 - win->BorderRight - deltasize; //xmax
		//y2 = win->Height - 50 - win->BorderBottom - deltasize; //ymax
		Printf ("Display Image %s x1= %d\n", position, x1);
		Printf ("Display Image %s y1= %d\n", position, y1);				
		Printf ("Display Image %s x2= %d\n", position, x2); //x1+77 //100-34=+66
		Printf ("Display Image %s y2= %d\n", position, y2); //y1+77 //156-126=-30				

		/* Draw Left Rectangle */
		Move(rastPort, x1, y1); Draw(rastPort, x1+77, y1);
		Move(rastPort, x1+77, y1); Draw(rastPort, x1+77, y1+77);
		Move(rastPort, x1, y1); Draw(rastPort, x1, y1+77);
		Move(rastPort, x1, y1+77); Draw(rastPort, x1+77, y1+77);      
       
	}
	else if(stricmp(position,"right") == 0)
	{
		/* Top left of Second Raised Panel */
		srce_x = 111 + center_x + deltasize;
	    srce_y = 25 + topoffset + center_y + deltasize;
		Printf ("Display Image %s srce_x= %d\n", position, srce_x);
		Printf ("Display Image %s srce_y= %d\n", position, srce_y);

		/* Erase Display Area Right */
		//EraseRect(win->RPort, leftoffset + 102 + 1 + deltasize, win->BorderTop + topoffset + 31 + deltasize,
        //win->Width - 20 - win->BorderRight - deltasize,
        //win->Height - 50 - win->BorderBottom - deltasize);
        
                //x1=srce_x;
		//y1=srce_y;
		y1 = topoffset + center_y + deltasize - 12; //25-37=-12
	        //x2=x1+77;
		y2=y1+77;
                x1 = leftoffset + 102 + 1 + deltasize; //xmin
		//y1 = win->BorderTop + topoffset - 3 + deltasize; //ymin
		//y1 = win->BorderTop + topoffset + 1 + deltasize; //ymin
		x2 = win->Width - 20 - win->BorderRight - deltasize;
		//y2= win->Height - 50 - win->BorderBottom - deltasize;
		Printf ("Display Image %s x1= %d\n", position, x1);
		Printf ("Display Image %s y1= %d\n", position, y1);				
		Printf ("Display Image %s x2= %d\n", position, x2);
		Printf ("Display Image %s y2= %d\n", position, y2);				 

		/* Draw Right Rectangle */
		Move(rastPort, x1, y1); Draw(rastPort, x1+77, y1);
		Move(rastPort, x1+77, y1); Draw(rastPort, x1+77, y1+77);
		Move(rastPort, x1, y1); Draw(rastPort, x1, y1+77);
		Move(rastPort, x1, y1+77); Draw(rastPort, x1+77, y1+77);
	}
	ReleasePen(win->WScreen->ViewPort.ColorMap, newpen);	
    
	//return 0;
}

/**************************************************************************************************/

static void Adjust_Alpha(UBYTE *imageData, int width, int height)
{
    int i = 0;

	/* Get every pixel of 4 bytes then modify alpha byte */
    
    for (i = 0; i < (width * height) * 4; i += 4)
    {        
        imageData[i] = 255;  // alpha
        //data[i + 1] = 255; // red
        //data[i + 2] = 255; // green
        //data[i + 3] = 255; // blue
    }
}

/*********************************************************************************************/

BOOL Display_Blank(char *imagename, char *position, WORD deltasize, struct Window *win)
{	
	int bpp;
	UBYTE *pixelBuffer;
	BOOL retval = TRUE;	
	UWORD srce_x, srce_y;
	UWORD center_x, center_y;
    int width, height, depth;			
    struct BitMapHeader *bmhd;
	  

	Printf ("Display Blank Filename: %s\n", imagename);


	/* GET IMAGE TYPE */

	imagetype = Check_Image_Signature(imagename);
	//png = PNG image //iff = ILBM image

	Printf ("File Type: %s\n", imagetype);

	if(stricmp(imagetype, "png")!=0 && stricmp(imagetype, "iff")!=0) 
	{
		/* Show Output Message for Image Load Error */
		STRPTR outputmsg = "Loading Image Failed. Bad Filename.";
		OutputMessage(outputmsg, win);					    
	} 

	/* Find Width, Height & Depth of Image */
	bmhd = readbmhd_by_datatype(imagename);
	width = bmhd->bmh_Width;
	height = bmhd->bmh_Height;
	depth = bmhd->bmh_Depth;
	bpp = (depth/8);


    //Printf ("Dimensions: %d, %d\n", width, height);


    /* Allocate memory for pixel buffer */
    pixelBuffer = (UBYTE *)AllocVec(width * height * bpp, MEMF_ANY);

   	
	if(stricmp(imagetype, "png")==0)
	{
		/* Fill whole pixel array with 153 */ 
        memset(pixelBuffer, 153, width * height * bpp);
		
		/* Set the Alpha Values to 255 */        		
	    Adjust_Alpha(pixelBuffer, width, height);
	}
		

    /* SHOW ICON IMAGE */

	/* Centered ? */
	center_x = ((80-width)/2);
	center_y = ((80-height)/2);

    /* Get coordinates */		
	if(stricmp(position,"left") == 0)
	{
		/* Top left of First Raised Panel */
	    //srce_x = 26 + center_x + deltasize;
		srce_x = 22 + center_x + deltasize;
	    srce_y = 25 + topoffset + center_y + deltasize;
	    
	    /* Erase Image Left */
	    //y1 = win->BorderTop + topoffset - 3 + deltasize, y1 = topoffset + center_y + deltasize - 12
	    //y2 = win->Height - 50 - win->BorderBottom - deltasize, y2 = topoffset + center_y + deltasize + 65   
		//EraseRect(win->RPort, leftoffset + 13 + 1 + deltasize, win->BorderTop + topoffset - 3 + deltasize,
                //win->Width - 175 - win->BorderRight - deltasize,
                //win->Height - 50 - win->BorderBottom - deltasize);
	}
	else if(stricmp(position,"right") == 0)
	{
		/* Top left of Second Raised Panel */
	    //srce_x = 151 + center_x + deltasize;
		srce_x = 111 + center_x + deltasize;
	    srce_y = 25 + topoffset + center_y + deltasize;
	    
	    /* Erase Image Right */ //EraseRect( rp, xmin, ymin, xmax, ymax) a1  d0:16 d1:16 d2:16 d3:16
		//win->BorderTop + topoffset + 31 + deltasize //102 not 138
		//y1 = win->BorderTop + topoffset - 3 + deltasize, y1 = topoffset + center_y + deltasize - 12
	        //y2 = win->Height - 50 - win->BorderBottom - deltasize, y2 = topoffset + center_y + deltasize + 65
		//EraseRect(win->RPort, leftoffset + 102 + 1 + deltasize, win->BorderTop + topoffset - 3 + deltasize,
                //win->Width - 20 - win->BorderRight - deltasize,
                //win->Height - 50 - win->BorderBottom - deltasize);
	}	
	

	//if(retval == TRUE)    
	if(depth == 32)
	{
		/* Display 32bit icon image */			        
		WritePixelArrayAlpha(pixelBuffer, 0, 0, width * depth / 8, win->RPort, srce_x, srce_y, width, height, RECTFMT_ARGB);
	}
	else if(depth == 24)
	{
		/* Display 24bit icon image */        	        
		WritePixelArray(pixelBuffer, 0, 0, width * depth / 8, win->RPort, srce_x, srce_y, width, height, RECTFMT_RGB);
	}

	/* Check for icon image mismatch */
	if(left_imagetype != NULL && right_imagetype != NULL)
	{
	    if(stricmp(left_imagetype, right_imagetype)!=0)
	    {		    
			/* Show Output Message for Image Type Mismatch */
			STRPTR outputmsg = "Image type for both images must match.";
			OutputMessage(outputmsg, win);
	    }
		else
		{
			//Check height & width
		}
	}	
		
	//if (pixelBuffer) FreeVec(pixelBuffer);	

    return retval;	
}

/*********************************************************************************************/

//Draw_ImageFrame(color FrameColor); 
//Use pen color & move, draw a colored rectangle

//Draw_DropShadow(color BlurColor, int BlurOffset);
//int BlurOffset is shadow offset in pixels
//Use Gaussian Blur Effect to draw shadow
//struct color.r, .g, .b; //RGB Color
//color BlurColor = color FrameColor;
//color BlurColor = color GlowColor;
//Hue values of glow colors must be exact.

/*********************************************************************************************/

BOOL Display_Image(char *imagename, char *position, WORD deltasize, struct Window *win)
{	
	int bpp;		
	//char *type = NULL;		
	UBYTE *pixelBuffer;
	BOOL retval = TRUE;	
	UWORD srce_x, srce_y;
	UWORD center_x, center_y;
    int width, height, depth;			
    struct BitMapHeader *bmhd;
	//struct IconInfo info = {0};	
	//struct DiskObject *icon = NULL;    

	//Printf ("Display Image Filename: %s\n", imagename);


	/* GET IMAGE TYPE */

	imagetype = Check_Image_Signature(imagename);
	//png = PNG image //iff = ILBM image

	//Printf ("File Type: %s\n", imagetype);

	if(stricmp(imagetype, "png")!=0 && stricmp(imagetype, "iff")!=0) 
	{
		/* Show Output Message for Image Load Error */
		STRPTR outputmsg = "Loading Image Failed. Bad Filename.";
		OutputMessage(outputmsg, win);					    
	} 

	/* Find Width, Height & Depth of Image */
	bmhd = readbmhd_by_datatype(imagename);
	width = bmhd->bmh_Width;
	height = bmhd->bmh_Height;
	depth = bmhd->bmh_Depth;
	bpp = (depth/8);

    //Printf ("Dimensions: %d, %d\n", width, height);

    
    //pixelBuffer = (UBYTE *)AllocVec(width * height * bpp, MEMF_ANY);
		


    /* GET IMAGE LOCATION */

	if(stricmp(position, "left")==0)
	{			
		//Printf ("Display position %s\n", position);
		left_imagetype = imagetype;		
	}
	else if(stricmp(position, "right")==0)
	{
		//Printf ("Display position %s\n", position);
		right_imagetype = imagetype;		
	}

	/* GET IMAGE COUNT */ 

	if((left_imagetype == NULL) &&  (right_imagetype == NULL)) count++;
	if((left_imagetype != NULL) &&  (right_imagetype == NULL)) count++;
	if((left_imagetype == NULL) &&  (right_imagetype != NULL)) count++;
	//Printf ("Total image count %d\n", count);

	/* CHECK IMAGE DATA */

	//if(count)
	//{
	    //if(stricmp(left_imagetype, right_imagetype)!=0) SAVE = FALSE;
		//SAVE = Check_Image_Sizes(infilename1, infilename2); //TRUE or FALSE
	//}
	
	if(stricmp(imagetype, "png")==0)
	{
		/* Allocate memory for pixel buffer */
		pixelBuffer = (UBYTE *)AllocVec(width * height * bpp, MEMF_ANY);
		
		/* Load the Icon Image by Datatype */
        //Printf ("ReadPixels_by_datatype\n");		
	    retval = ReadPixels_by_datatype(pixelBuffer, imagename, bpp);
	}
	else if(stricmp(imagetype, "iff")==0)
	{
		/* Return ARGB pixel data from OS35 Hybrid or OS41 Icon */
		
		/* For displaying ILBM images */
		if(depth < 8)
		{
		    depth = 32;
		    bpp = 4; //32bit
		}
		
		/* Allocate memory for pixel buffer */
		pixelBuffer = (UBYTE *)AllocVec(width * height * bpp, MEMF_ANY);
		
		/* Load the Icon Image by Datatype */
            //Printf ("ReadPixels_by_datatype\n");		
	    retval = ReadPixels_by_datatype(pixelBuffer, imagename, bpp);

		ULONG *data1;
	    ULONG *data2;
		//BPTR file = NULL;	
		//Test Writing Hybrid Disk Object		
		//PutDiskObject("Ram Disk:TestIcon", icon);

		/* Find Beginning of OS3.5 IFF Data */
		//LONG iffoffset = Get_IFFOffset(srcename); //Find offset for OS3.5 IFF data		
		//Printf ("\nReturned IFF Offset: %ld\n", iffoffset);
		/* Check for OS3.5 & ARGB Data Chunks */
	    //char *ifftype = Check_IFFType(srcename, iffoffset); //OS35/OS41 or ARGB or NONE
		//Printf ("IFF Icon Type: %s\n", ifftype);
		char *ifftype = "os35";

		//Draw_Rectangle will draw colored rectangles to provide visual clues to what icon style is loaded.
        //Graphics Libary drawing routine is used after SetAPen to set the color. Color = Red or White.
		if(stricmp(position, "left")==0)
		{
			left_imagetype = ifftype;
			//Load_ILBM(imagename, 0); //index 0
		}
		else if(stricmp(position, "right")==0)
		{
			right_imagetype = ifftype;
			//Load_ILBM(imagename, 1); //index 1
		}


        /* Get Image Data from ARGB Data Chunks */		
        if(stricmp(ifftype, "argb")==0)
		{		    
	        //IconControl(icon,			    	    
		        //ICONCTRLA_GetARGBImageData1,      &data1,
		        //ICONCTRLA_GetARGBImageData2,      &data2,
		        //TAG_END);
			
			//pixelBuffer = (UBYTE *)data1; /* 1st Image For Icon Display */


			/* See: ReadMemPNG for method to convert ULONG array to UBYTE array */
			/* ReadMemPNG can be found in "diskobjPNGio.c" inside Icon Library */


			/* If position = "left" check icon type. 
	        If type = "argb" convert to png with messagebox.
            Save png icon as "Ram Disk:SourceIcon.info" */

			if(stricmp(position, "left")==0)
			{
				/* Write datatype object to file */
		        //if (file = Open("Ram Disk:Im1.png", MODE_NEWFILE))
		        //{
                    //Seek(file,0,OFFSET_BEGINNING);
                    //Write(file,pixelBuffer,(width * height * bpp));
				//}
                //if (file) Close (file);

			    /* Save Both Images as PNG */
			    char *image1 = "Ram Disk:Im1.png";
			    //SavePixels_by_datatype((UBYTE *)data1, image1, "png", 4);
			    //retval = SavePixels_by_datatype(data1, image1, width, height);				
			    char *image2 = "Ram Disk:Im2.png";
			    //SavePixels_by_datatype((UBYTE *)data2, image2, "png", 4);
			    //SavePixels_by_datatype((UBYTE *)data2, image2, info, bpp);
				//retval = SavePixels_by_datatype(data2, image2, width, height);

                /* Make PNG Icon from Image Data for Future Use */
			    //void Make_Icon(char *image1, char *image2, char *outfile);
			    //Make_Icon(image1, image2, "Ram Disk:SourceIcon.info");
			
			    /* Delete Images */
			    //DeleteFile(image1);
			    //DeleteFile(image2);
			}
			//Save_Icon(srcedobj, outputname); //Save_Icon(destdobj, outputname);						
		}
		else if(stricmp(ifftype, "os41")==0)
		{
            //IconControl(icon,			    	    
		        //ICONCTRLA_GetARGBImageData1,      &data1,
		        //ICONCTRLA_GetARGBImageData2,      &data2,
		        //TAG_END);
			
			//pixelBuffer = (UBYTE *)data1; /* 1st Image For Icon Display */

			if(stricmp(position, "left")==0)
			{				
			    /* Save Both Images as PNG */
			    char *image1 = "Ram Disk:Im1.png";
			    //SavePixels_by_datatype((UBYTE *)data1, image1, "png", 4);
			    //retval = SavePixels_by_datatype(data1, image1, width, height);				
			    char *image2 = "Ram Disk:Im2.png";
			    //SavePixels_by_datatype((UBYTE *)data2, image2, "png", 4);
			    //SavePixels_by_datatype((UBYTE *)data2, image2, info, bpp);
				//retval = SavePixels_by_datatype(data2, image2, width, height);

                /* Make PNG Icon from Image Data for Future Use */
			    //void Make_Icon(char *image1, char *image2, char *outfile);
			    //Make_Icon(image1, image2, "Ram Disk:SourceIcon.info");
			
			    /* Delete Images */
			    //DeleteFile(image1);
			    //DeleteFile(image2);
			}
		}
		else if(stricmp(ifftype, "os35")==0)
		{
			/* Get Image Data from OS3.5 IFF Data */
			//See INFO Datatype for Method for getting Image Data.
			/* Use switch (index) instead of assuming first image index */

			//IconControl(icon,			    
				//ICONCTRLA_GetWidth,			    &info.Width,
				//ICONCTRLA_GetHeight,			&info.Height,
				//ICONCTRLA_GetPaletteSize1,		&info.NumColors,
				//ICONCTRLA_GetPalette1,			&info.CMap,
				//ICONCTRLA_GetImageData1,		&info.Image,
				//TAG_END);

            //Printf ("ReadOS35_by_datatype\n");		
		    /* Return 32bit Pixel Data from OS35 Image Data */			
			//retval = ReadOS35_by_datatype(pixelBuffer, info, bpp);		
		    //retval = ReadOS35_by_datatype(pixelBuffer, info.Image, info.CMap, 4);
			//BOOL SaveAsDatatype_lut8(UBYTE *pixels, UBYTE *colorBytes, char *file_name, char *type);			

			/* Write image data to file */
		    //if (file = Open("Ram Disk:Im1.bin", MODE_NEWFILE))
		    //{
                //Seek(file,0,OFFSET_BEGINNING);
                //Write(file,info.Image,(info.Width * info.Height));
			//}
            //if (file) Close (file);

			/* New function for glow icon images */	        
			//retval = ReadImage (pixelBuffer, info, bpp);
		}				
	}
	

    /* SHOW ICON IMAGE */

	/* Centered ? */
	center_x = ((80-width)/2);
	center_y = ((80-height)/2);

    /* Get coordinates */		
	if(stricmp(position,"left") == 0)
	{
		/* Top left of First Raised Panel */
	    //srce_x = 26 + center_x + deltasize;
		srce_x = 22 + center_x + deltasize;
	    srce_y = 25 + topoffset + center_y + deltasize;
	    
	    /* Erase Image Left */
	    //y1 = win->BorderTop + topoffset - 3 + deltasize, y1 = topoffset + center_y + deltasize - 12
	    //y2 = win->Height - 50 - win->BorderBottom - deltasize, y2 = topoffset + center_y + deltasize + 65   
		//EraseRect(win->RPort, leftoffset + 13 + 1 + deltasize, topoffset + center_y + deltasize - 12,
                //win->Width - 175 - win->BorderRight - deltasize,
                //topoffset + center_y + deltasize + 65);
	}
	else if(stricmp(position,"right") == 0)
	{
		/* Top left of Second Raised Panel */
	    //srce_x = 151 + center_x + deltasize;
		srce_x = 111 + center_x + deltasize;
	    srce_y = 25 + topoffset + center_y + deltasize;
	    
	    /* Erase Image Right */ //EraseRect( rp, xmin, ymin, xmax, ymax) a1  d0:16 d1:16 d2:16 d3:16
		//win->BorderTop + topoffset + 31 + deltasize //102 not 138
		//y1 = win->BorderTop + topoffset - 3 + deltasize, y1 = topoffset + center_y + deltasize - 12
	        //y2 = win->Height - 50 - win->BorderBottom - deltasize, y2 = topoffset + center_y + deltasize + 65
		//EraseRect(win->RPort, leftoffset + 102 + 1 + deltasize, win->BorderTop + topoffset - 3 + deltasize,
                //win->Width - 20 - win->BorderRight - deltasize,
                //win->Height - 50 - win->BorderBottom - deltasize);
	}	
	

	//if(retval == TRUE)    
	if(depth == 32)
	{
		/* Display 32bit icon image */			        
		WritePixelArrayAlpha(pixelBuffer, 0, 0, width * depth / 8, win->RPort, srce_x, srce_y, width, height, RECTFMT_ARGB);
	}
	else if(depth == 24)
	{
		/* Display 24bit icon image */        	        
		WritePixelArray(pixelBuffer, 0, 0, width * depth / 8, win->RPort, srce_x, srce_y, width, height, RECTFMT_RGB);
	}

	/* Check for icon image mismatch */
	if(left_imagetype != NULL && right_imagetype != NULL)
	{
	    if(stricmp(left_imagetype, right_imagetype)!=0)
	    {		    
			/* Show Output Message for Image Type Mismatch */
			STRPTR outputmsg = "Image type for both images must match.";
			OutputMessage(outputmsg, win);
	    }
		else
		{
			//Check height & width
		}
	}	
		
	//if (pixelBuffer) FreeVec(pixelBuffer);
	//FreeDiskObject(icon);

    return retval;	
}

/*********************************************************************************************/

const char  *DetectBaseName_by_datatype(char *file_name)
{
    struct DataType *dt;
    const char *basename;    
	Object *DTImage = NULL;		

	DTImage = NewDTObject(	file_name, (DTA_SourceType), DTST_FILE, TAG_DONE);

	if (DTImage)
	{
        /* Get DataType BaseName */        
        GetDTAttrs(DTImage, DTA_DataType, (IPTR)&dt, TAG_DONE );        
        basename = dt->dtn_Header->dth_BaseName;
    }
    ReleaseDataType(dt);
    DisposeDTObject(DTImage);

    return basename;
}

/**************************************************************************************************/

struct BitMapHeader *readbmhd(char *file_name)
{
	Object *DTImage = NULL;
	struct BitMapHeader *bmh;

	DTImage = NewDTObject(	file_name,
							(DTA_SourceType),	DTST_FILE,
							(DTA_GroupID),		GID_PICTURE,
							(PDTA_Remap),		FALSE,
							(OBP_Precision),	PRECISION_EXACT,
							TAG_DONE);

	if (DTImage)
	{

		if (GetDTAttrs( DTImage,
					PDTA_BitMapHeader, &bmh,
					TAG_END ) == 1)

		{            
            return bmh;
        }
    }
    DisposeDTObject( DTImage );
    return bmh;
}

/**************************************************************************************************/

//BOOL readpixels_by_datatype(UBYTE *pixels, char *file_name, int bpp)
BOOL ReadPixels_by_datatype(UBYTE *pixels, char *file_name, int bpp)
{	
    Object *DTImage = NULL;
	struct BitMapHeader *bmh;
	struct pdtBlitPixelArray bpa;

	DTImage = NewDTObject(	file_name,
							(DTA_SourceType),	DTST_FILE,
							(DTA_GroupID),		GID_PICTURE,
							(PDTA_Remap),		FALSE,
							(OBP_Precision),	PRECISION_EXACT,
							TAG_DONE);

	if (DTImage)
	{

		if (GetDTAttrs( DTImage,
					PDTA_BitMapHeader, (IPTR)&bmh,
					TAG_END ) == 1)

		{					
			if (pixels)
			{
				bpa.MethodID = PDTM_READPIXELARRAY;
				bpa.pbpa_PixelData = (APTR)pixels;
				bpa.pbpa_PixelFormat = (bpp = 4) ? PBPAFMT_ARGB: PBPAFMT_RGB;
				bpa.pbpa_PixelArrayMod = bmh->bmh_Width * bpp;
				bpa.pbpa_Left = 0;
				bpa.pbpa_Top = 0;
				bpa.pbpa_Width = bmh->bmh_Width;
				bpa.pbpa_Height = bmh->bmh_Height;

				DoMethodA( DTImage, (Msg)&bpa );
				DisposeDTObject( DTImage );
				return TRUE;
			}
		}
		DisposeDTObject( DTImage );
	}
	return FALSE;
}

/**************************************************************************************************/

//BOOL ReadOS35_by_datatype(UBYTE *pixels, struct IconInfo info, int bpp)
BOOL ReadImage (UBYTE *pixels, struct IconInfo info, int bpp)
{
	int i;
	//int i, y;
	int numcolors;
	//BPTR file = NULL;
	//struct dtWrite dtw;
	Object *DTImage = NULL;	
	struct BitMapHeader *bmh = NULL;
	struct pdtBlitPixelArray bpa; 
    struct pdtBlitPixelArray dtb;
    struct ColorRegister *cmap;
    ULONG *cregs;	
    
    //D(bug("info.datatype/ConvertINFO(): Function: ReadImage.\n"));
	//Printf ("ReadOS35_by_datatype: ReadImage.\n");
	Printf ("Reading OS35 Image...\n");

	/* Set type for PNG */
    char *type = "png";

    DTImage = NewDTObject(	(APTR)NULL,
            DTA_SourceType, DTST_RAM,
            DTA_BaseName, (IPTR)type,
            PDTA_DestMode, PMODE_V43,
            TAG_DONE);
	
    GetDTAttrs(DTImage,
		PDTA_BitMapHeader,	&bmh,		
		TAG_END);
	
    
	bmh->bmh_Depth = 8;
	bmh->bmh_Width = info.Width;
	bmh->bmh_Height = info.Height;
	numcolors = info.NumColors;	
    

	//D(bug("info.datatype/ConvertINFO(): Picture Data: Width %d x Height %d\n", bmh->bmh_Width, bmh->bmh_Height));		
	//D(bug("info.datatype/ConvertINFO(): Picture Data: BitDepth: %d\n", bmh->bmh_Depth));	
	//D(bug("info.datatype/ConvertINFO(): Picture Data: NumColors: %d\n", info.NumColors));

	Printf ("Dimensions: (w,h,d) %d, %d, %d\n", bmh->bmh_Width, bmh->bmh_Height, bmh->bmh_Depth);

    	
	if (!info.CMap) 
    {        
		return FALSE;				
    }

	if(bmh->bmh_Depth == 8)
    {
        /* Get cregs and color palette */
        GetDTAttrs( DTImage,
            PDTA_CRegs,        (IPTR)&cregs,			
			PDTA_ColorRegisters,     (IPTR)&cmap,                                       
            TAG_END );        
    }

	/* Set colorregs & ColorRegister */

	for (i = 0; i < info.NumColors; i++) 
	{
		cmap->red = info.CMap->red;
		cmap->green = info.CMap->green;
		cmap->blue = info.CMap->blue;
		*cregs++ = RGB8to32(info.CMap->red);
		*cregs++ = RGB8to32(info.CMap->green);
		*cregs++ = RGB8to32(info.CMap->blue);
		cmap++; info.CMap++;
	}   
	
    
    if (!info.Image) 
    {        
		return FALSE;				
    }    

	/* Copy LUT8 Pixel Data to New DataType Object */
    if ( DTImage )
	{        
		dtb.MethodID = PDTM_WRITEPIXELARRAY;
		dtb.pbpa_PixelData = (APTR)info.Image;        
		dtb.pbpa_PixelFormat = PBPAFMT_LUT8;
		dtb.pbpa_PixelArrayMod = bmh->bmh_Width;
		dtb.pbpa_Left = 0;
		dtb.pbpa_Top = 0;
		dtb.pbpa_Width = bmh->bmh_Width;
		dtb.pbpa_Height = bmh->bmh_Height;
		
		DoMethodA(DTImage, (Msg) &dtb);
    }

	/* Set datatype attributes */	

	SetDTAttrs(DTImage, NULL, NULL,	    
		DTA_NominalHoriz,	bmh->bmh_Width,
		DTA_NominalVert,	bmh->bmh_Height,
		PDTA_NumColors,     numcolors,
		TAG_END);


	/* Save Image as PNG */
	//char *image1 = "Ram Disk:Im1.png";

	/* Write datatype object to file */
	//if (file = Open(image1, MODE_NEWFILE))
	//{
		//dtw.MethodID = DTM_WRITE;
		//dtw.dtw_GInfo = NULL;
		//dtw.dtw_FileHandle = file;
		//dtw.dtw_Mode = DTWM_RAW;
		//dtw.dtw_AttrList = NULL;

		//if (!DoMethodA(DTImage, (Msg) &dtw)) return FALSE;
	//}
	//if (file) Close (file);   
    
    
	/* Read ARGB Pixel Data from New DataType Object */
	if (pixels)
	{
		bpa.MethodID = PDTM_READPIXELARRAY;
		bpa.pbpa_PixelData = (APTR)pixels;
		bpa.pbpa_PixelFormat = (bpp = 4) ? PBPAFMT_ARGB: PBPAFMT_RGB;
		bpa.pbpa_PixelArrayMod = bmh->bmh_Width * bpp;
		bpa.pbpa_Left = 0;
		bpa.pbpa_Top = 0;
		bpa.pbpa_Width = bmh->bmh_Width;
		bpa.pbpa_Height = bmh->bmh_Height;

		DoMethodA( DTImage, (Msg)&bpa );		
	}

	DisposeDTObject( DTImage );

	
	return TRUE;
}

/**************************************************************************************************/

//BOOL SavePixels_by_datatype(UBYTE *pixels, char *filename, char *type, int bpp)
BOOL SavePixels_by_datatype(ULONG *data, char *filename, int width, int height)
{	
	UBYTE *pixels;
	BPTR file = NULL;
	BOOL retval = FALSE;
	Object *DTImage = NULL;
	struct dtWrite dtw;
	struct BitMapHeader *bmhd;	
	struct pdtBlitPixelArray dtb;

	//Printf("SavePixels_by_datatype: ARGB to PNG\n");

	char *type = "png";	
		
	DTImage = NewDTObject(	(APTR)NULL,
							DTA_SourceType, DTST_RAM,
							DTA_BaseName, (IPTR)type,
							PDTA_DestMode, PMODE_V43,
							TAG_DONE);
	if (!DTImage) return(FALSE);

	
	if (GetDTAttrs( DTImage,
					PDTA_BitMapHeader, (IPTR)&bmhd,
					TAG_END ) == 1)
	{
		bmhd->bmh_Width = width;
		bmhd->bmh_Height = height;
		bmhd->bmh_Left = 0;
        bmhd->bmh_Top = 0;
        bmhd->bmh_Depth = 32;
        bmhd->bmh_Masking = 0;
        bmhd->bmh_Compression = 0;
        bmhd->bmh_Pad = 0;
        bmhd->bmh_Transparent = 0;
        bmhd->bmh_XAspect = 10;
        bmhd->bmh_YAspect = 11;
        bmhd->bmh_PageWidth = width;
        bmhd->bmh_PageWidth = height;

		//Printf ("Icon Dimensions: %d, %d\n", bmhd->bmh_Width, bmhd->bmh_Height);

		pixels = (UBYTE *)AllocVec(width * height * 4, MEMF_ANY);
		pixels = (UBYTE *)data; /* ARGB Image Data to Convert */

		/* See: ReadMemPNG for method to convert ULONG array to UBYTE array */
		
		dtb.MethodID = PDTM_WRITEPIXELARRAY;
		dtb.pbpa_PixelData = (APTR)pixels;
		dtb.pbpa_PixelFormat = PBPAFMT_ARGB;
		dtb.pbpa_PixelArrayMod = bmhd->bmh_Width * 4;
		dtb.pbpa_Left = 0;
		dtb.pbpa_Top = 0;
		dtb.pbpa_Width = bmhd->bmh_Width;
		dtb.pbpa_Height = bmhd->bmh_Height;
		
		DoMethodA(DTImage, (Msg) &dtb);

		/* Write datatype object to file */
		if (file = Open(filename, MODE_NEWFILE))
		{
			dtw.MethodID = DTM_WRITE;
			dtw.dtw_GInfo = NULL;
			dtw.dtw_FileHandle = file;
			dtw.dtw_Mode = DTWM_RAW;
			dtw.dtw_AttrList = NULL;

			if (DoMethodA(DTImage, (Msg) &dtw)) retval = TRUE;
		}
	}

	if (file) Close (file);
	//if (pixels) FreeVec(pixels);
	if (DTImage) DisposeDTObject(DTImage);
	return retval;
}

/**************************************************************************************************/

struct BitMapHeader *readbmhd_by_datatype(char *file_name)
{
	Object *DTImage = NULL;
	struct BitMapHeader *bmh;

	DTImage = NewDTObject(	file_name,
							(DTA_SourceType),	DTST_FILE,
							(DTA_GroupID),		GID_PICTURE,
							(PDTA_Remap),		FALSE,
							(OBP_Precision),	PRECISION_EXACT,
							TAG_DONE);

	if (DTImage)
	{

		if (GetDTAttrs( DTImage,
					PDTA_BitMapHeader, &bmh,
					TAG_END ) == 1)

		{            
            return bmh;
        }
    }
    DisposeDTObject( DTImage );
    return bmh;
}

/**************************************************************************************************/

void Copy_Data_Append(char *srcename, char *destname, LONG offset, LONG filebytes)
{
    BPTR infile, outfile;
	UBYTE buffer[filebytes];    
    
    infile = Open(srcename, MODE_OLDFILE);
    if(infile)    
    {                
        Seek(infile,0,OFFSET_BEGINNING);
	    Read(infile,buffer,filebytes);
    }
    outfile = Open(destname, MODE_OLDFILE);
    if(outfile)
    {        
        Seek(outfile,offset,OFFSET_BEGINNING);
        Write(outfile,buffer,filebytes);               
    }	    

    if(infile) Close(infile);
    if(outfile) Close(outfile);
}

/**************************************************************************************************/

void Copy_Data(char *srcename, char *destname, LONG filebytes)
{
    BPTR infile, outfile;
	UBYTE buffer[filebytes];    
    
    infile = Open(srcename, MODE_OLDFILE);
    if(infile)    
    {                
        Seek(infile,0,OFFSET_BEGINNING);
	    Read(infile,buffer,filebytes);
    }
    outfile = Open(destname, MODE_OLDFILE);
    if(outfile)
    {        
        Seek(outfile,0,OFFSET_BEGINNING);
        Write(outfile,buffer,filebytes);               
    }	    

    if(infile) Close(infile);
    if(outfile) Close(outfile);
}

/**************************************************************************************************/

void Copy_File(char *srcename, char *destname, LONG filebytes)
{
    BPTR infile, outfile;
	UBYTE buffer[filebytes];    
    
    infile = Open(srcename, MODE_OLDFILE);
    if(infile)    
    {                
        Seek(infile,0,OFFSET_BEGINNING);
	    Read(infile,buffer,filebytes);
    }	
    outfile = Open(destname, MODE_NEWFILE);
    if(outfile)
    {        
        Seek(outfile,0,OFFSET_BEGINNING);
        Write(outfile,buffer,filebytes);               
    }	    

    if(infile) Close(infile);
    if(outfile) Close(outfile);
}

/**************************************************************************************************/

void SetFilesize(char *name, LONG newsize)
{    
	BPTR file = Open(name, MODE_OLDFILE);
	SetFileSize(file,newsize,OFFSET_BEGINNING);
	Close(file);
}

/**************************************************************************************************/

LONG GetFileSize(char *imName)
{
    BPTR file = Open(imName, MODE_OLDFILE);
    struct FileInfoBlock *fib = (struct FileInfoBlock*)AllocDosObject( DOS_FIB, NULL );
    LONG size = -1;
    
    if (fib)
    {
	if (ExamineFH( file, fib ))
	{
	    if ((fib->fib_DirEntryType > 0) && (fib->fib_DirEntryType != ST_SOFTLINK))
	    {
		size = 0;
	    }
	    size = fib->fib_Size;
	}
	FreeDosObject( DOS_FIB, fib );
    }
    Close(file);
    
    return size;
}

/**************************************************************************************************/

//char *Check_ImageType(char *srcename)
//{
	//Check_Image //if(Check_Icon(filename)) returns "image"
    //1. Determine the type of image (png or ilbm).
    //2. Check image signature - (1st 12 char)
    //3. If 'png & numimages = 1 then return "png image"
    //4. If FORM--ILBM then return "iff image"
    //5. Else return "none"
//}

/**************************************************************************************************/

char *Check_File_Signature(char *srcename, BOOL isIcon)
{	
	char *icontype;
	UBYTE fileID[5];    
	
	if(srcename) 
	{
        BPTR file = Open(srcename, MODE_OLDFILE);
        if(file)
		{
            Seek(file,0,OFFSET_BEGINNING);
	        Read(file,fileID, 4);
	        fileID[4] = ('\0');

			if( fileID[1] == 'P' && fileID[2] == 'N' && fileID[3] == 'G' )
            {
				Printf("Found PNG File!\n");
                icontype = "png";				           
            }
			else if( fileID[0] == 0xE3 && fileID[1] == 0x10 )
            {
                Printf("Found IFF File!\n");            
                icontype = "iff";                
            }
		}
		Close(file);
	}    

    return icontype;
}

/**************************************************************************************************/

char *Check_Image_Signature(char *srcename)
{	
	char *imgtype;
	UBYTE fileID[5];    
	
	if(srcename) 
	{
        BPTR file = Open(srcename, MODE_OLDFILE);
        if(file)
		{
            Seek(file,0,OFFSET_BEGINNING);
	        Read(file,fileID, 4);
	        fileID[4] = ('\0');

			if( fileID[1] == 'P' && fileID[2] == 'N' && fileID[3] == 'G' )
            {				
                imgtype = "png";				           
            }			
			else if( fileID[0] == 'F' && fileID[1] == 'O' && fileID[2] == 'R' && fileID[3] == 'M' )
            {                
				Seek(file,8,OFFSET_BEGINNING);
	            Read(file,fileID, 4);
	            fileID[4] = ('\0');

				if( fileID[0] == 'I' && fileID[1] == 'L' && fileID[2] == 'B' && fileID[3] == 'M' )
                    imgtype = "iff";
				else
				    imgtype = "none";             
            }
		}
		Close(file);
	}    

    return imgtype;
}

/**************************************************************************************************/

char *RemoveFileExtension(char *path) 
{
    /* This works */
    UBYTE *buffer;
    UBYTE *filePart;
    char *baseName;
    ULONG pathSize;
    ULONG partSize;
    ULONG position = 0;
    
	/* Remove ONLY ONE '.' segment in filename */
 
    pathSize = strlen(path) * sizeof(UBYTE *) + 1;
 
    /* Get the file part which we need to ignore. */
    filePart = FilePart(path);	
    partSize = strlen(filePart) * sizeof(UBYTE *) + 1;
 
    /* Allocate memory for the base name. */
    if((baseName = 
        AllocMem(
            strlen(path) * sizeof(UBYTE *) - strlen(filePart) + 1, 
            MEMF_ANY|MEMF_CLEAR
        )
    ) == NULL) return NULL;
 
    /* Allocate memory for path part. */
    if((buffer = AllocMem(pathSize, MEMF_ANY|MEMF_CLEAR)) == NULL) 
        return NULL;
 
    /* Strip pieces of the path and compose them. */
    do 
	{
        position = SplitName(path, '.', buffer, position, pathSize);
 
        switch(position == -1 ||
            strlen(buffer) == 0 || 
            strncmp(buffer, filePart, partSize) == 0) 
            {
 
            case TRUE: break; /* We have found the file part. */
            default: /* Otherwise, copy the buffer into the base path. */
                sprintf(baseName + strlen(baseName), "%s/", buffer);
                break;
            }
 
    } while(position != -1);
 
    switch(strlen(baseName)) 
    {
        case 0: /* Attempt to build a drive path. */
            if(SplitName(path, ':', buffer, 0, pathSize) != -1)
                sprintf(baseName + strlen(baseName), "%s:", buffer);
            break;
        default: /* Otherwise, strip trailing char. */
            baseName[strlen(baseName)-1] = '\0';
            break;
    }

	/* Restore dots in filename */	
	int i = 0;
	int count = 0;
    for(i = strlen(filePart) - 1;  i >= 0; i--)
	{
		if (filePart[i] == '.') 
		{
			count++;
			if(count > 1) 
		    {				
			    baseName[strlen(baseName)+1-i] = '.';
		    }
		}		
	}
 
    return baseName;
}

/**************************************************************************************************/

char *get_ext(char *filename)
{
	static char extension[32];
	int position=strlen((char *)filename)-1;

	strcpy(extension,"");

	while(position > -1 && filename[position] != '.') position--;

	if (position > -1)
	{
		strncpy(extension,&filename[position+1],32);
	}

	return extension;
}

/**************************************************************************************************/

char* GetFileNameFromPath(char* path)
{
    int i = 0;
    for(i = strlen(path) - 1;  i >= 0; i--)
    {
        //Includes Full Path with File Name
        if (path[i] == '/' && path[i] != ':')
        {
            return &path[i+1];
        }

        //Includes Volume + File Name Only!
        if (path[i] == ':' && path[i] != '/')
        {
            return &path[i+1];
        }
    }

   return path;
}

/**************************************************************************************************/

char *GetPath(char *path) 
{
    /* This works */
    UBYTE *buffer;
    UBYTE *filePart;
    char *baseName;
    ULONG pathSize;
    ULONG partSize;
    ULONG position = 0;

    /* Returns Path with ':' or '/' */
 
    pathSize = strlen(path) * sizeof(UBYTE *) + 1;
 
    /* Get the file part which we need to ignore. */
    filePart = FilePart(path);
    partSize = strlen(filePart) * sizeof(UBYTE *) + 1;
 
    /* Allocate memory for the base name. */
    if((baseName = 
        AllocMem(
            strlen(path) * sizeof(UBYTE *) - strlen(filePart) + 1, 
            MEMF_ANY|MEMF_CLEAR
        )
    ) == NULL) return NULL;
 
    /* Allocate memory for path part. */
    if((buffer = AllocMem(pathSize, MEMF_ANY|MEMF_CLEAR)) == NULL) 
        return NULL;
 
    /* Strip pieces of the path and compose them. */
    do {
        position = SplitName(path, '/', buffer, position, pathSize);
 
        switch(position == -1 ||
            strlen(buffer) == 0 || 
            strncmp(buffer, filePart, partSize) == 0) 
            {
 
            case TRUE: break; /* We have found the file part. */
            default: /* Otherwise, copy the buffer into the base path. */
                sprintf(baseName + strlen(baseName), "%s/", buffer);
                break;
        }
 
    } while(position != -1);
 
    switch(strlen(baseName)) 
    {
        case 0: /* Attempt to build a drive path. */
            if(SplitName(path, ':', buffer, 0, pathSize) != -1)
                sprintf(baseName + strlen(baseName), "%s:", buffer);
            break;
        default: /* Otherwise, strip trailing slash. */
            //baseName[strlen(baseName)-1] = '\0';
            break;
    }
 
    return baseName;
}

//static void Write_DiskObject (BPTR outfile, LONG offset)
static void Write_DiskObject (unsigned char *buf, int width, int height)
{    	
	//unsigned char *startbuf = buf;
    long i;	
	
	//static char *image1option;
    //static char *image2option;
    //static char *defaulttooloption;
    //static char *drawerdataoption;
    //static char **tooltypesoption;
	BOOL Image1Option = FALSE;
	BOOL Image2Option = FALSE;
	BOOL IconDrawerOption = FALSE;	
	BOOL DefaultToolOption = FALSE;
	BOOL ToolTypesOption = FALSE;

	int IconOSType = 1; //(1 for OS 2.x/3.x icons)
	int IconType = 3;	//3 or 4 (tool or project)
	int XPosition = 0, YPosition = 0;	
	int StackSize = 4096;
	
	//Image1Option = infilename1; //first image present
	if(infilename1) Image1Option = TRUE;
    //Image2Option = infilename2;	//second image present
	if(infilename2) Image2Option = TRUE;

    //uint16 magic number //0xE310
	//uint16 version number //usually set to 1
	//uint32 nextgadget //usually set to 0
	//uint16 leftedge //usually set to 0
	//uint16 topedge //usually set to 0
	//uint16 width //set to image width
    //uint16 height //set to image height
	//gadget flags //usually set to 5
	//activation //usually set to 3
	//gadget type //usually set to 1
	//uint32 gadget render //pointer for first image
	//uint32 gadget selectrender //pointer to second image
	//uint32 gadget text //usually set to 0
	//uint32 manual exclude //usually set to 0
	//uint32 special info //usually set to 0
	//gadget id //usually set to 0
	//uint32 user data //0 for OS 1.x icons. 1 for OS 2.x/3.x icons
	//uint8 gadget type //type of icon:
	    //1 disk or volume, 2 drawer (folder), 3 tool (exe), 4 project (data file),  
        //5 trashcan, 6 device, 7 Kickstart ROM image, 8 appicon (placed on the desktop).
	//uint8 padding //usually set to 0
	//uint32 default tool //pointer default tool string
	//uint32 tooltypes //pointer to tooltype strings
	//int32 currentx //horiz position of the icon in the drawer window
	//int32 currenty //vert position of the icon in the drawer window
	//uint32 drawer data //pointer to drawerdata structure
	//uint32 tool window //typically unused
	//uint32 stacksize //stack size for an application
	//total = 78 bytes (diskobject)


    PutUint16(buf, 0xE310);                     /* ic_Magic */
    PutUint16(buf, 1);                          /* ic_Version */
    PutUint32(buf, 0);                          /* ga_Next */
    PutUint16(buf, 0);                          /* ga_LeftEdge */
    PutUint16(buf, 0);                          /* ga_TopEdge */
    PutUint16(buf, width);                      /* ga_Width */
    PutUint16(buf, height);                     /* ga_Height */
    i = 4;            
    if(Image2Option) i |= 2;
	else i |= 1;
    PutUint16(buf, i);                              /* ga_Flags */
	//PutUint16(buf, 5);                          
    PutUint16(buf, 3); //0                          /* ga_Activation */
    PutUint16(buf, 1); //0                          /* ga_GadgetType */
    PutUint32(buf, 0x2A2A2A2A);                     /* ga_GadgetRender */	  
    PutUint32(buf, Image2Option ? 0x2A2A2A2A : 0);  /* ga_SelectRender */
    PutUint32(buf, 0);                              /* ga_GadgetText */
    PutUint32(buf, 0);                              /* ga_MutualExclude */
    PutUint32(buf, 0);                              /* ga_SpecialInfo */
    PutUint16(buf, 0);                              /* ga_GadgetID */                               
	PutUint32(buf, IconOSType);                     /* ga_UserData */
    *(buf++) = IconType;                            /* ic_Type */
    *(buf++) = 0;                                   /* ic_Pad */
    PutUint32(buf, DefaultToolOption ?
               0x2A2A2A2A : 0);                     /* ic_DefaultTool */
    PutUint32(buf, ToolTypesOption ?
               0x2A2A2A2A : 0);                     /* ic_ToolTypes */
    PutUint32(buf, XPosition);                      /* ic_CurrentX */
    PutUint32(buf, YPosition);                      /* ic_CurrentY */
    PutUint32(buf, IconDrawerOption ?
               0x2A2A2A2A : 0);                     /* ic_DrawerData */
    PutUint32(buf, 0);                              /* ic_ToolWindow */                 
	PutUint32(buf, StackSize);                      /* ic_StackSize */          
}

/**************************************************************************************************/

BOOL Save_Icon(char *image1, char *image2, char *outfile)
{	
	LONG offset = 0;
	LONG filebytes = 0;
	
	//Printf ("InfileName1: %s\n", image1);
	//Printf ("InfileName2: %s\n", image2);

	/* Save a PNG Icon */	
	if (image1)
	{
        filebytes = GetFileSize(image1);
	    Copy_File(image1, outfile, filebytes);
	    offset = filebytes;
	}
	if (image2)
	{
        filebytes = GetFileSize(image2);	
	    Copy_Data_Append(image2, outfile, offset, filebytes);
	}    
}

/**************************************************************************************************/

//BOOL Save_Classic_Icon(char *infile1, char *infile2, char *outfile, BOOL argb, BOOL os41)
BOOL Save_Classic_Icon(char *infile1, char *infile2, char *outfile, BOOL argb)
{
    BPTR file = BNULL;
	WORD width, height, depth;
	struct BitMapHeader *bmhd;

	LONG numimages = 0, numface = 0;
	LONG transparency, numcolors, icflags;  
	LONG imgfmt, palfmt, numimg, numpal, icSize;	
	
	unsigned char *buffer;

	/* Get total icon file size */
	icSize = 78; //diskobject size = 78 bytes
	//imSize = GetFileSize(srcename); //Calculate total icon size
	Printf ("Icon File Size: %ld\n", icSize);

    /* Get byte array of IFF data */
    //buffer = (unsigned char *)AllocVec(imSize-iffoffset, MEMF_ANY);
    buffer = AllocMem(icSize, MEMF_CLEAR);
    //FreeMem(buffer, icSize);
	
	/* Get width, height, depth of first image */
	bmhd = readbmhd_by_datatype(infile1);	
	width = bmhd->bmh_Width;
	height = bmhd->bmh_Height;
	//depth = bmhd->bmh_Depth;
	//Printf ("Dimensions: %d, %d\n", width, height);

    //Printf ("Save_Classic_Icon - Load Images\n");

    //PNGImage img1 = NULL;
	//PNGImage img2 = NULL;
    //ILBMImage ilbm1 = NULL;
	//ILBMImage ilbm2 = NULL;
    
	//img1 = LoadImage(infile1);
	//if (image2option) img2 = LoadImage(infile2);


	/* Save a Classic Amiga Icon with OS35 Extension */    

	/* Open Outputfile */
	file = Open(outfile, MODE_NEWFILE);
    if (file)
	{ 
		/* old write methods */
		//writediskobject();
        //writeolddrawerdata();
        //writeimage(&img1);    
        //if (image2option) writeimage(&img2);
    
        //if (defaulttooloption) writestring(defaulttooloption);
    
        //if (tooltypesoption)
        //{
    	    //char **strarray;	
    	    //LONG numtooltypes = 0;
	
	        //for(strarray = tooltypesoption; *strarray; strarray++, numtooltypes++);	
	        //writelong((numtooltypes + 1) * 4);

	        //for(strarray = tooltypesoption; *strarray; strarray++)
	        //{
	            //writestring(*strarray);
	        //}
	
        //}
    
        /* toolwindow would have to be saved in between here if there is any */
    
        //writenewdrawerdata();    
        //write35data();

		
		/* new write methods */

		Printf ("Save_Classic_Icon - Write_DiskObject");
		//fprintf(stderr, "Save_Classic_Icon - Write_DiskObject");
	    Write_DiskObject (buffer, width, height);

		//Write_DiskObject(buffer, file);
        
	    //Printf ("Save_Classic_Icon - Write_Old DrwerData");
	    //fprintf(stderr, "Save_Classic_Icon - Write_Old DrwerData");

		//Write_OldDrawerData(buffer, file);        

	    //Printf ("Save_Classic_Icon - Write_First Image");
	    //fprintf(stderr, "Save_Classic_Icon - Write_First Image");

		//Printf ("Save_Classic_Icon - Write_Second Image");
	    //fprintf(stderr, "Save_Classic_Icon - Write_Second Image");

		//if(infilename1) //Write_IconImage(buffer, infilename1, file);
		//if(infilename2) //Write_IconImage(buffer, infilename2, file);

		//Printf ("Save_Classic_Icon - Write_Default Tool");
	    //fprintf(stderr, "Save_Classic_Icon - Write_Default Tool");

		//if (defaulttooloption) write_string(defaulttooloption);

		//Printf ("Save_Classic_Icon - Write_ToolTypes");
	    //fprintf(stderr, "Save_Classic_Icon - Write_ToolTypes");

		//if (tooltypesoption) write_tooltypes();
		//[...]

		/* toolwindow data */
		//Write_NewDrawerData();

		//if ((stricmp(imagetype, "iff")==0)
		    //Write_os35(infilename1, infilename2, file);
		//else
		    //OutputMessage("Image Type must be ILBM to Save Classic Icon!\n", win);

		//For ARGB icons AROS style there are OS35 images.
		//For ARGB icons OS41 style there are no OS35 images.
		
		//if (argb == TRUE)
		//if ((stricmp(imagetype, "png")==0) && (argb == TRUE))
		    //Write_Argb(infilename1, infilename2, file);
		//else
		    //OutputMessage("Image Type must be PNG to Save ARGB Icon!\n", win);


		//Write Icon to Outputfile
		//if (fwrite(f, 1, 4, outfile) != 4)
        //{
    	    //OutputMessage("Error writing to output file!"\n);
			//return FALSE; //return 0; //return OK or NOT_OK
        //}
		
		/* write buffer to file */
		Write(file, buffer, icSize);
	}
	Close(file);	

	/* Free Resources */
	//FreeMem(buffer, icSize);

	return FALSE;
}

/**************************************************************************************************/

//BOOL Save_Amiga_Icon(struct DiskObject *oldicon, struct DiskObject *newicon, STRPTR name)
//static BOOL Save_Amiga_Icon (char *infile1, char *infile2, char *outputfile, BOOL argb)
BOOL Save_Amiga_Icon(char *infile1, char *infile2, char *outputfile, char *iconfile, int icontype)
//LONG Save_Classic_Icon(struct DiskObject *oldicon, struct DiskObject *newicon, unsigned char *buf)
{	
	char *error;
	LONG offset;
	LONG i, j, k;	
	ULONG FileLONG;
	UWORD FileWORD;	
	UBYTE Buffer16[2];
	UBYTE Buffer32[4];	
	int imagesize, imageoffset;
	struct BitMapHeader *bmhd;
	struct Window *win;	
	   

    Printf ("Save_Amiga_Icon - Assign Attributes\n");

    BOOL Image1Option = FALSE;
	BOOL Image2Option = FALSE;
	BOOL IconDrawerOption = FALSE;	
	BOOL DefaultToolOption = FALSE;
	BOOL ToolTypesOption = FALSE;
    BOOL ToolWindowOption = FALSE;

	int IconOSType = 1; //(1 for OS 2.x/3.x icons)
	int IconType = icontype;	//1, 2, 3 or 4 (disk, drawer, tool or project)
	int XPosition = 0, YPosition = 0;	
	int StackSize = 4096;

    if(IconType == 2) IconDrawerOption = TRUE;
	
	//first image present
	if(infile1) Image1Option = TRUE;
    //second image present
	if(infile2) Image2Option = TRUE; 

    //Printf ("Save_Amiga_Icon - Load Images\n");
    
    //PNGImage img1 = NULL;
	//PNGImage img2 = NULL;
    //ILBMImage ilbm1 = NULL;
	//ILBMImage ilbm2 = NULL;
    
	/* Load Icon Images*/
	//error = Load_ILBM(infile1, &img1); //index 0
	Load_ILBM(infile1, &img1); //index 0
	if(infile2)  Load_ILBM(infile2, &img2); //index 1

	#if (1) 
        /* Save Icon Images*/
		char *outfile1 = "Ram Disk:testimage1.iff";
		char *outfile2 = "Ram Disk:testimage2.iff";
		Save_ILBM(outfile1, &img1); //index 0	    
	    if(infile2)  Save_ILBM(outfile2, &img2); //index 1;
		//error = Save_ILBM(infile1, &img1); //index 0
		//if(error) OutputMessage(error, win); return OK; return NOT_OK;
		//if(error) return error;
		//error = NOT_OK;  goto exit;

		//place exit at end of main function
		//exit:
	    /* Free Resources */	
	    //if(buf) FreeMem(buf, bpr);
	    //return error;		   
	#endif

	/* Remap planar image data - remap indexes */
	//remapplanar requires findcolor & std4colpal & struct Palette & c2p
	
    //remapplanar(&img1, &std4colpal);
    //if (image2option) remapplanar(&img2, &std4colpal); 
	//if(infile2) remapplanar(&img2, &std4colpal);
	//RemapIconImage = remapplanar
	//RemapIconImage(&img1, &std4colpal);
	//if(infile2) RemapIconImage(&img2, &std4colpal);   
    

	
	//Printf ("Save_Amiga_Icon - Get Dimensions\n");

	WORD ic_width, ic_height, ic_depth;

    /* width, height, depth of first image */
	bmhd = readbmhd_by_datatype(infile1);	
	ic_width = bmhd->bmh_Width - 2;
	ic_height = bmhd->bmh_Height - 2;
	ic_depth = bmhd->bmh_Depth;    


	//**********************************************************//
	
    //uint16 magic number //0xE310
	//uint16 version number //usually set to 1
	//uint32 nextgadget //usually set to 0
	//uint16 leftedge //usually set to 0
	//uint16 topedge //usually set to 0
	//uint16 width //set to image width
    //uint16 height //set to image height
	//uint16 gadget flags //usually set to 5
	//uint16 activation //usually set to 1 or 0
	//uint16 gadget type //usually set to 1 or 0
	//uint32 gadget render //pointer for first image
	//uint32 gadget selectrender //pointer to second image
	//uint32 gadget text //usually set to 0
	//uint32 manual exclude //usually set to 0
	//uint32 special info //usually set to 0
	//uint16 gadget id //usually set to 0
	//uint32 user data //0 for OS 1.x icons. 1 for OS 2.x/3.x icons
	//uint8 gadget type //type of icon:
	    //1 disk or volume, 2 drawer (folder), 3 tool (exe), 4 project (data file),  
        //5 trashcan, 6 device, 7 Kickstart ROM image, 8 appicon (placed on the desktop).
	//uint8 padding //usually set to 0
	//uint32 default tool //pointer default tool string
	//uint32 tooltypes //pointer to tooltype strings
	//int32 currentx //horiz position of the icon in the drawer window
	//int32 currenty //vert position of the icon in the drawer window
	//uint32 drawer data //pointer to drawerdata structure
	//uint32 tool window //typically unused
	//uint32 stacksize //stack size for an application
	//total = 78 bytes (diskobject)


	//**********************************************************//

	/* NEW SAVE METHOD */

    BOOL OS41 = FALSE;
	LONG flag, bufSize;    
    LONG gadtype;
    LONG activation;
    LONG userdata;
	unsigned char *buf;
	unsigned char *buffer;
	unsigned char *drwdata;
	unsigned char *imgdata;
	unsigned char *os35data;
	unsigned char *imagedata1;
	unsigned char *imagedata2;

	LONG firstimage, secondimage;
	LONG tooltypes, deftool, drawerdata, toolwin;

	/* Read Source & Write Icon Data to File */
	

	//writediskobject();
    //writeolddrawerdata();
    //writeimage(&img1);
	//writeimage(&img2);
	//writedefaulttool;
	//writetooltypes;
	/* toolwindow */
	//writenewdrawerdata();    
    //write35data();

	offset = 0;

	/* WRITE ICON DATA TO FILE */	

	/* Open icon file to Write */ 
    BPTR outfile = Open(outputfile, MODE_NEWFILE);
	//BPTR outfile = Open("Ram Disk: TempIcon_Test.info", MODE_NEWFILE);
    if (outfile)
	{
        //Printf ("Save_Classic_Icon - Write_DiskObject");
		//fprintf(stderr, "Save_Classic_Icon - Write_DiskObject");
	    //offset = Write_DiskObject (outfile, offset);
		//offset += 78;
		//Printf ("Save_Classic_Icon - Write_OldIcon");
		//fprintf(stderr, "Save_Classic_Icon - Write_OldIcon");
		//offset = Write_OldIcon (outfile, offset);

        /* PREPARE ICON DATA BUFFER */	
	    
	    if(buf = AllocMem(MAXBUFFERSIZE, MEMF_ANY))
	    {   
		    unsigned char *startbuf = buf;

            if (outputfile != NULL)
	        {
			    flag = 6;
		        //offset = 0;	
                gadtype = 1;
                activation = 1;
                userdata = 1;			
				char *ifftype;
				UBYTE Buffer8;
				LONG numtt = 0;
				LONG pSize = 0;
				LONG imSize = 0;				
				UBYTE Buffer16[2];
	            UBYTE Buffer32[4];	
				LONG filebytes = 0;
				LONG iffoffset = 0;
				LONG imageoffset = 0;

				UWORD LeftEdge; //+0x00
                UWORD TopEdge; //+0x02
                UWORD Width; //+0x04
                UWORD Height; //+0x06
                UWORD Depth; //+0x08


				/* WRITE DISKOBJECT DATA */

                Printf ("Save_Amiga_Icon - Write DiskObject Data\n");
			    PutUint16(Buffer16, 0xE310);                               /* ic_Magic */
				Seek(outfile,offset,OFFSET_BEGINNING);
				Write(outfile, Buffer16, 2);
				offset += 2; //2
				PutUint16(Buffer16, 1);                                    /* ic_Version */
				Seek(outfile,offset,OFFSET_BEGINNING);
				Write(outfile, Buffer16, 2);
				offset += 2; //4
				PutUint32(Buffer32, 0);                                    /* ga_Next */
				Seek(outfile,offset,OFFSET_BEGINNING);
				Write(outfile, Buffer32, 4);                                
				offset += 4; //8
				PutUint16(Buffer16, 0);                                    /* ga_LeftEdge */
				Seek(outfile,offset,OFFSET_BEGINNING);
				Write(outfile, Buffer16, 2);
				offset += 2; //10
			    PutUint16(Buffer16, 0);                                    /* ga_TopEdge */
				Seek(outfile,offset,OFFSET_BEGINNING);
				Write(outfile, Buffer16, 2);
				offset += 2; //12
                PutUint16(Buffer16, ic_width);                                 /* ga_Width */
				Seek(outfile,offset,OFFSET_BEGINNING);
				Write(outfile, Buffer16, 2);
				offset += 2; //14
			    PutUint16(Buffer16, ic_height);                                 /* ga_Height */
                Seek(outfile,offset,OFFSET_BEGINNING);
				Write(outfile, Buffer16, 2);
				offset += 2; //16
			    PutUint16(Buffer16, flag);                                   /* ga_Flags */
				Seek(outfile,offset,OFFSET_BEGINNING);
				Write(outfile, Buffer16, 2);
				offset += 2; //18
				PutUint16(Buffer16, activation);                             /* ga_Activation */
				Seek(outfile,offset,OFFSET_BEGINNING);
				Write(outfile, Buffer16, 2);
				offset += 2; //20  
                PutUint16(Buffer16, gadtype);                                /* ga_GadgetType */
				Seek(outfile,offset,OFFSET_BEGINNING);
				Write(outfile, Buffer16, 2);
				offset += 2; //22
			    //If OS41 then use BOOL_NO
			    PutUint32(Buffer32, 0x2A2A2A2A);                              /* ga_GadgetRender */
				Seek(outfile,offset,OFFSET_BEGINNING);
				Write(outfile, Buffer32, 4);
				offset += 4; //26
                //If OS41 then use BOOL_NO
				PutUint32(Buffer32, 0x2A2A2A2A);                              /* ga_SelectRender */
				Seek(outfile,offset,OFFSET_BEGINNING);
				Write(outfile, Buffer32, 4);
				offset += 4; //30
                PutUint32(Buffer32, 0);                                       /* ga_GadgetText */
				Seek(outfile,offset,OFFSET_BEGINNING);
				Write(outfile, Buffer32, 4);
				offset += 4; //34
			    PutUint32(Buffer32, 0);                                       /* ga_MutualExclude */
				Seek(outfile,offset,OFFSET_BEGINNING);
				Write(outfile, Buffer32, 4);
				offset += 4; //38
				PutUint32(Buffer32, 0);                                       /* ga_SpecialInfo */
				Seek(outfile,offset,OFFSET_BEGINNING);
				Write(outfile, Buffer32, 4);
				offset += 4; //42
			    PutUint16(Buffer16, 0);                                       /* ga_GadgetID */
				Seek(outfile,offset,OFFSET_BEGINNING);
				Write(outfile, Buffer16, 2);
				offset += 2; //44
                PutUint32(Buffer32, userdata);                                       /* ga_UserData */
				Seek(outfile,offset,OFFSET_BEGINNING);
				Write(outfile, Buffer32, 4);
				offset += 4; //48
                
                Buffer8 = IconType;                                           /* ic_Type */
				//Seek(outfile,offset,OFFSET_BEGINNING);
				//Write(outfile, Buffer8, 1);
				FPutC(outfile,Buffer8);
				offset += 1; //49
			    Buffer8 = 0;                                                  /* ic_Pad */
				//Seek(outfile,offset,OFFSET_BEGINNING);
				//Write(outfile, Buffer8, 1);
				FPutC(outfile,Buffer8);				
				offset += 1; //50
			    
				//PutUint32(Buffer32, BOOL_NO);
			    PutUint32(Buffer32, DefaultToolOption ? BOOL_YES : 0);        /* ic_DefaultTool */
				Seek(outfile,offset,OFFSET_BEGINNING);
				Write(outfile, Buffer32, 4);
			    offset += 4; //54			    
				//PutUint32(Buffer32, BOOL_NO);
				PutUint32(Buffer32, ToolTypesOption ? BOOL_YES : 0);         /* ic_ToolTypes */
				Seek(outfile,offset,OFFSET_BEGINNING);
				Write(outfile, Buffer32, 4);
			    offset += 4; //58			    
			    PutUint32(Buffer32, XPosition);                               /* ic_CurrentX */
				Seek(outfile,offset,OFFSET_BEGINNING);
				Write(outfile, Buffer32, 4);
				offset += 4; //62           
			    PutUint32(Buffer32, YPosition);                               /* ic_CurrentY */
				Seek(outfile,offset,OFFSET_BEGINNING);
				Write(outfile, Buffer32, 4);
				offset += 4; //66			    
			    PutUint32(Buffer32, IconDrawerOption ? BOOL_YES : 0);         /* ic_DrawerData */
				Seek(outfile,offset,OFFSET_BEGINNING);
				Write(outfile, Buffer32, 4);
			    offset += 4; //70
			    //ToolWindow is Unused. Set to zero.
			    PutUint32(Buffer32, 0);                                        /* ic_ToolWindow */
				Seek(outfile,offset,OFFSET_BEGINNING);
				Write(outfile, Buffer32, 4);
				offset += 4; //74			    
			    PutUint32(Buffer32, StackSize);                                 /* ic_StackSize */
				Seek(outfile,offset,OFFSET_BEGINNING);
				Write(outfile, Buffer32, 4);
				offset += 4; //78

				//EndPutM16Inc(buf, oldicon->Drawer.LeftEdge);     /* nw_LeftEdge */
                //EndPutM16Inc(buf, oldicon->Drawer.TopEdge);      /* nw_TopEdge */
                //EndPutM16Inc(buf, oldicon->Drawer.Width);        /* nw_Width */
                //EndPutM16Inc(buf, oldicon->Drawer.Height);       /* nw_Height */
                //*(buf++) = 255;                              /* nw_DetailPen */
                //*(buf++) = 255;                              /* nw_BlockPen */
				//EndPutM32Inc(buf, 0);                        /* nw_IDCMPFlags */
				//EndPutM32Inc(buf, oldicon->Drawer.Flags);    /* nw_Flags */
				//EndPutM32Inc(buf, 0);                        /* nw_FirstGadget */
				//EndPutM32Inc(buf, 0);                        /* nw_CheckMark */
				//EndPutM32Inc(buf, 0);                        /* nw_Title */
				//EndPutM32Inc(buf, 0);                        /* nw_Screen */
				//EndPutM32Inc(buf, 0);                        /* nw_BitMap */
				//EndPutM16Inc(buf, 90);                  /* nw_MinWidth */
                //EndPutM16Inc(buf, 40);                  /* nw_MinHeight */
                //EndPutM16Inc(buf, 65535);               /* nw_MaxWidth */
                //EndPutM16Inc(buf, 65535);               /* nw_MaxHeight */
				//EndPutM16Inc(buf, 1);                            /* nw_Type */
				//EndPutM32Inc(buf, oldicon->Drawer.XPosition);    /* dd_CurrentX */
                //EndPutM32Inc(buf, oldicon->Drawer.YPosition);    /* dd_CurrentY */


				//BPTR oldfile = Open(destname, MODE_OLDFILE);
                //char *target = ("Ram Disk:Calculator.info");
				char *target = iconfile;

                Printf ("Save_Amiga_Icon - Srce Icon: %s\n", target); //inputfile	
				Printf ("Save_Amiga_Icon - Dest File: %s\n", outputfile);
				

			    /* Process Icon Drawer Data */ //ico = oldicon
			    if(IconDrawerOption)				
			    {
				    Printf ("Save_Amiga_Icon - Write OldDrawer Data\n");

					/* Open old icon to read the drawer data (56 bytes) at offset 78 */
					
					BPTR oldfile = Open(target, MODE_OLDFILE);
					if(oldfile)
					{
						Seek(oldfile,offset,OFFSET_BEGINNING);
						Read(oldfile, buf, 56);
					}
					Close(oldfile); 

					Seek(outfile,offset,OFFSET_BEGINNING);
				    Write(outfile, buf, 56);



                    //PutUint16(Buffer16, oldicon->Drawer.LeftEdge);      /* nw_LeftEdge */
				    //Seek(outfile,offset,OFFSET_BEGINNING);
				    //Write(outfile, Buffer16, 2);
				    //offset += 2; //80
					//PutUint16(Buffer16, oldicon->Drawer.TopEdge);      /* nw_TopEdge */
				    //Seek(outfile,offset,OFFSET_BEGINNING);
				    //Write(outfile, Buffer16, 2);
				    //offset += 2; //82
					//PutUint16(Buffer16, oldicon->Drawer.Width);      /* nw_Width */
				    //Seek(outfile,offset,OFFSET_BEGINNING);
				    //Write(outfile, Buffer16, 2);
				    //offset += 2; //84
					//PutUint16(Buffer16, oldicon->Drawer.Height);      /* nw_Height */
				    //Seek(outfile,offset,OFFSET_BEGINNING);
				    //Write(outfile, Buffer16, 2);
				    //offset += 2; //86

					//Buffer8 = 255;                                      /* nw_DetailPen */
					//FPutC(outfile,Buffer8);
				    //offset += 1; //87
					//Buffer8 = 255;                                      /* nw_BlockPen */
					//FPutC(outfile,Buffer8);
				    //offset += 1; //88

					//PutUint32(Buffer32, 0);                             /* nw_IDCMPFlags */
				    //Seek(outfile,offset,OFFSET_BEGINNING);
				    //Write(outfile, Buffer32, 4);
				    //offset += 4; //92
					//PutUint32(Buffer32, oldicon->Drawer.Flags);          /* nw_Flags */
				    //Seek(outfile,offset,OFFSET_BEGINNING);
				    //Write(outfile, Buffer32, 4);
				    //offset += 4; //96
					//PutUint32(Buffer32, 0);                             /* nw_FirstGadget */
				    //Seek(outfile,offset,OFFSET_BEGINNING);
				    //Write(outfile, Buffer32, 4);
				    //offset += 4; //100
					//PutUint32(Buffer32, 0);                             /* nw_CheckMark */
				    //Seek(outfile,offset,OFFSET_BEGINNING);
				    //Write(outfile, Buffer32, 4);
				    //offset += 4; //104
					//PutUint32(Buffer32, 0);                             /* nw_Title */
				    //Seek(outfile,offset,OFFSET_BEGINNING);
				    //Write(outfile, Buffer32, 4);
				    //offset += 4; //108
					//PutUint32(Buffer32, 0);                             /* nw_Screen */
				    //Seek(outfile,offset,OFFSET_BEGINNING);
				    //Write(outfile, Buffer32, 4);
				    //offset += 4; //112
					//PutUint32(Buffer32, 0);                             /* nw_BitMap */
				    //Seek(outfile,offset,OFFSET_BEGINNING);
				    //Write(outfile, Buffer32, 4);
				    //offset += 4; //116 
                    
                    //PutUint16(Buffer16, 90);                             /* nw_MinWidth */
				    //Seek(outfile,offset,OFFSET_BEGINNING);
				    //Write(outfile, Buffer16, 2);
				    //offset += 2; //118
					//PutUint16(Buffer16, 40);                             /* nw_MinHeight */
				    //Seek(outfile,offset,OFFSET_BEGINNING);
				    //Write(outfile, Buffer16, 2);
				    //offset += 2; //120
					//PutUint16(Buffer16, 65535);                          /* nw_MaxWidth */
				    //Seek(outfile,offset,OFFSET_BEGINNING);
				    //Write(outfile, Buffer16, 2);
				    //offset += 2; //122
					//PutUint16(Buffer16, 65535);                          /* nw_MaxHeight */
				    //Seek(outfile,offset,OFFSET_BEGINNING);
				    //Write(outfile, Buffer16, 2);
				    //offset += 2; //124
					//PutUint16(Buffer16, 1);                              /* nw_Type */
				    //Seek(outfile,offset,OFFSET_BEGINNING);
				    //Write(outfile, Buffer16, 2);
				    //offset += 2; //126

					//PutUint32(Buffer32, oldicon->Drawer.XPosition);      /* dd_CurrentX */
				    //Seek(outfile,offset,OFFSET_BEGINNING);
				    //Write(outfile, Buffer32, 4);
				    //offset += 4; //130
					//PutUint32(Buffer32, oldicon->Drawer.YPosition);       /* dd_CurrentY */
				    //Seek(outfile,offset,OFFSET_BEGINNING);
				    //Write(outfile, Buffer32, 4);
				    //offset += 4; //134
					offset += 56; //134

					/* Total Size = 56 bytes */ /* 78 + 56 = 134 bytes */				    
			    }


				//char *target = ("Ram Disk:icons/Screen.info");

				//char *ifftype = NULL;
				//if BOOL argb == FALSE && BOOL os41 == FALSE then ifftype = os35
				//if BOOL argb == TRUE && BOOL os41 == FALSE then ifftype = argb
				//if BOOL argb == FALSE && BOOL os41 == TRUE then ifftype = os41				

                //** Note: Get IFFType to decide if we need to copy IFF Data **//
				iffoffset = Get_IFFOffset(target);				
				Printf ("Save_Amiga_Icon - Returned IFF Offset: %ld\n", iffoffset);
		        /* Check for OS3.5 & ARGB Data Chunks */
	            ifftype = Check_IFFType(target, iffoffset); //0s35/os41 or argb or none						        
		        Printf ("Save_Amiga_Icon - IFF Icon Type: %s\n", ifftype);
				imSize = GetFileSize(target);

				//Printf ("Save_Classic_Icon - Source Icon: %s\n", target); //inputfile	
				//Printf ("Save_Classic_Icon - Destination: %s\n", outputfile);


				/* COPY NEW ICON DATA HERE */

			    
			    /* Get Planar Image Data from newicon */
				/* Open srcename for Reading */ //infile
				//BPTR infile = Open(srcename, MODE_OLDFILE);
				BPTR infile = Open(target, MODE_OLDFILE);
				{
				    //Use writeimage with imageoffset for a sample from ilbmtoicon
					//static void writeimage(struct ILBMImage *img)
					//Use the planar image data loaded previously by Load_ILBM function
					//Write headers & planar images for gadget_render & gadget_selectrender 
					
					/* Process First Image */
				    if(Image1Option)
				    {
						//uint8 gadget type //type of icon:
	                    //1 disk or volume, 2 drawer (folder), 3 tool (exe), 4 project (data file),  
                        //5 trashcan, 6 device, 7 Kickstart ROM image, 8 appicon (placed on the desktop).

					    //Open srcename to read the image data (20 bytes) at offset 78 or 134 
					    //Then read the needed values from the image data byte buffer
					    
					    /* Get target icon type */
					    unsigned char type;
					    Seek(infile,48,OFFSET_BEGINNING);
					    Read(infile, buf, 10);
					    type = buf[0];			    

						/* Offset to Image Data */
						//if(IconDrawerOption)
						//if((IconType == 1) || (IconType == 2)) //or TrashCan Icon = 5
						if((type == 1) || (type == 2)) //or TrashCan Icon = 5
					    {
						    imageoffset = 134;
					    }
					    else
					    {
						    imageoffset = 78;
					    }

                        Printf ("Save_Amiga_Icon - Read First Image: %ld\n", imageoffset);
					    Seek(infile,imageoffset,OFFSET_BEGINNING);
						Read(infile, buf, 20);                        
                        
					    /* Try to Get 1st Image Size */
						LeftEdge = GetUint16(buf+0x00);
                        TopEdge = GetUint16(buf+0x02);
					    Width = GetUint16(buf+0x04);
                        Height = GetUint16(buf+0x06);
					    Depth = GetUint16(buf+0x08);

						Printf ("Save_Amiga_Icon - Image Dims: %d, %d, %d\n", Width, Height, Depth);                        

					    /* Filesize of image data. Both images are the same. */
					    filebytes = (((((Width+15)&(~15)) * Height) * Depth)>>3);
						Printf ("Save_Amiga_Icon - Image Size: %d\n", filebytes);

					    /* For First Image Copy the Image Data & Write to File */
				    
					    Printf ("Save_Amiga_Icon - Write First Image\n");
					    //offset += 20; //Offset from beginning of outfile //offset = bufSize + 20 = 154					 					
					    //if((imagedata1 = AllocMem(filebytes, MEMF_ANY)))
					    //{						    
							Seek(infile,imageoffset,OFFSET_BEGINNING);
						    Read(infile, buf, filebytes + 20);							
						    Seek(outfile,offset,OFFSET_BEGINNING);
						    Write(outfile, buf, filebytes + 20);						    					
					    //}
					    //FreeMem(imagedata1, filebytes);
						//Write_Image(outfile, &img1, offset);

						/* The new offsets may be different */
						offset = offset + filebytes + 20;
						imageoffset = imageoffset + filebytes + 20;
				    }
				    /* Process Second Image */
				    if(Image2Option)
				    {
					    /* For Second Image Copy the Image Data & Write to File */
				    
					    Printf ("Save_Amiga_Icon - Write Second Image\n");					
					    //offset += 20; //Offset from beginning of outfile //offset = offset + filebytes + 20;					
					    //if((imagedata2 = AllocMem(filebytes, MEMF_ANY)))
					    //{						    
                            Seek(infile,imageoffset,OFFSET_BEGINNING);
						    Read(infile, buf, filebytes + 20);
						    Seek(outfile,offset,OFFSET_BEGINNING);
						    Write(outfile, buf, filebytes + 20);
					    //}
					    //offset = offset + filebytes; 
					    //offset += filebytes; //offset = bufSize + 20 + filebytes + 20 + filebytes
					    //FreeMem(imagedata2, filebytes);
						//Write_Image(outfile, &img2, offset);

						/* The new offsets may be different */
						offset = offset + filebytes + 20;
						imageoffset = imageoffset + filebytes + 20;
				    }
					//Close(oldfile);				
                

				    /* COPY OLD ICON DATA HERE */


                    //Setup new buffer to write to outfile
				    /* Process DefaultTool & ToolTypes Array */
			        if(DefaultToolOption)
			        {
				        Printf ("Save_Amiga_Icon - Write DefaultTool\n");						

						Seek(infile,imageoffset,OFFSET_BEGINNING);
                        Read(infile,Buffer32, 4); /* Get Size of DefaultTool String */
                        pSize = GetUint32(Buffer32);

						//Printf ("DefaultTool: %s, Size:%ld/n", oldicon->do_DefaultTool, pSize);

						Read(infile, buf, pSize);
						Seek(outfile,offset,OFFSET_BEGINNING);
						Write(outfile, buf, pSize);
						offset += pSize;
						imageoffset += pSize; //infileoffset

                        //for(i = 0; ico->DefaultTool[i]; ++i)
                        //buf[4+i] = ico->DefaultTool[i];
                        //buf[4+(i++)] = 0;
                        //EndPutM32Inc(buf, i);
                        //buf += i;
			        }  
                    if(ToolTypesOption)
			        {
				        Printf ("Save_Amiga_Icon - Write ToolTypes\n");
				        
                        //LONG numtt = 0; //i = EndGetM32(buffer)/4-1;
						Seek(infile,imageoffset,OFFSET_BEGINNING);
                        Read(infile,Buffer32, 4);
						numtt = GetUint32(Buffer32)/4-1;
						Printf ("Number of ToolTypes: %ld\n", numtt);

                        //buf += 4;
                        //while(oldicon->do_ToolTypes[numtt])
                        //{
                            //for(i = 0; oldicon->do_ToolTypes[numtt][i]; ++i)
					        //{
                                //buf[4+i] = oldicon->ToolTypes[numtt][i];
								//Printf ("ToolType: %s/n", oldicon->do_ToolTypes[numtt][i]);
					        //} 
							//See: ToolTypes Sample Values.txt
							//Calculate size of ToolTypes in bytes.
							//Read oldicon ToolTypes data & write to output file.

                            //buf[4+(i++)] = 0;
                            //EndPutM32Inc(buf, i);
                            //buf += i;
                            //++numtt;
                        //}
				        //EndPutM32(buffer, (numtt+1)*4);

						//offset += pSize;
						//imageoffset += pSize; //infileoffset
			        }

			        /* toolwindow would be saved here if there is any */
					if(ToolWindowOption)
			        {
						/* Unused. Should always be zero. */
                        Printf ("Save_Amiga_Icon - Write ToolWindow Data\n");
						Seek(infile,imageoffset,OFFSET_BEGINNING);
		                Read(infile,buf, 4);
                        Seek(outfile,offset,OFFSET_BEGINNING);
						Write(outfile, buf, 4);     
		                
						offset += 4;
						imageoffset += 4; //infileoffset
					}


                    /* WRITE NEW DRAWER DATA */

			        
				    Printf ("Save_Amiga_Icon - Write NewDrawer Data\n");

					//uint8 gadget type //type of icon:
	                //1 disk or volume, 2 drawer (folder), 3 tool (exe), 4 project (data file),  
                    //5 trashcan, 6 device, 7 Kickstart ROM image, 8 appicon (placed on the desktop).

					//if(userdata == 1) //If UserData = 1 then OSType = "os31"
					//if((IconType == 1) || (IconType == 2) || (IconType == 5) && (userdata == 1))
					if((IconType == 1) || (IconType == 2) && (userdata == 1))
					{ 
						/* Write NewDrawerData. Should be: 0001 01 */
					    PutUint32(Buffer32, 1);                                     /* dd_Flags */
				        Seek(outfile,offset,OFFSET_BEGINNING);
				        Write(outfile, Buffer32, 4);
				        offset += 4;
					    PutUint16(Buffer16, 1); //01 ?? 10                          /* dd_ViewModes */
				        Seek(outfile,offset,OFFSET_BEGINNING);
				        Write(outfile, Buffer16, 2);
				        offset += 2;
					}


				    /* COPY NEW ICON OS35 DATA HERE */

				    			
				    /* Calculate total filebytes for os35 */
					filebytes = (imSize - iffoffset);
					//offset = ?? See NewDrawerData Last Offset 

					//static void Write_OS35data(void)
					//{
					    //LONG value = 0;
						//LONG formsize = 0;
						//formsize += Calc_FaceChunk(); //6
                        //formsize += Calc_ImagChunk(infilename1); //10+chunksize
                        //if (image2option) formsize += Calc_ImagChunk(infilename2); //10+chunksize

						//writelong(ID_FORM); //4                        
                        //writelong(formsize); //4
                        //writelong(ID_ICON); //4

						//value += Write_FaceChunk(); //6
                        //value += Write_ImagChunk(infilename1); //10+chunksize
						//if (image2option) value += Write_ImagChunk(infilename2); //10+chunksize

						//formsizeseek = ftell(outfile); //Compare actual file size
	                //}					
					
				    Close(outfile);
				    /* Use BOOL argb & BOOL os41 to determine which icon style to save */
					if((stricmp(ifftype, "os41")==0) || (stricmp(ifftype, "argb")==0) || (stricmp(ifftype, "os35")==0))
				    { 
				        //Setup os35data buffer to write to file
                        //if((os35data = AllocMem(filebytes, MEMF_ANY)))
					    //{
			                /* Write OS35 IFF Data & ARGB Data */
				            Printf ("Save_Amiga_Icon - Write OS35 IFF Data\n");
							//LONG fileoffset = GetFileSize(outputfile);
							Write_OS35data(infile1, infile2, outputfile, offset);
							//Write_OS35data(infile1, infile2, "Ram Disk:OS35.bin");				            		

				            //Seek(infile,iffoffset,OFFSET_BEGINNING);
				            //Read(infile, buf, filebytes);
                            //Seek(outfile,offset,OFFSET_BEGINNING);
				            //Write(outfile, buf, filebytes);
					    //}
					    //FreeMem(os35data, filebytes);
				    }
				} //End infile
				Close(infile);
				//Close srcename //infile //Close source file								

	        } //End if(name)
		    else
		    {
			    Printf("Destination filename missing!\n");
			    return FALSE;
		    }
            /* Calculate Buffer Size for Icon Data */
	        //bufSize = buf-startbuf;

	    } //End if(buf)
	
	} //End if(outfile)
	else
    {
        Printf("failed opening destination icon.\n");
        FreeMem(buf, MAXBUFFERSIZE);
		return FALSE;
    }
	//Close(outfile);
	FreeMem(buf, MAXBUFFERSIZE); 
	FreeIconImage(&img1);
    FreeIconImage(&img2);   

    
	return TRUE;		   
}

/**************************************************************************************************/

static char *EncodeRLE(unsigned long depth, unsigned char *dtype, LONG *dsize, unsigned long len, unsigned char *srce)
{
    int i, j;
    unsigned char val;
    long res, cnt;    
    long idx, pdx, prv;    
    unsigned char *dest;
    unsigned char c1, c2;         
    long out, count, equal;
    unsigned long bitbuf, numbits;     
    
    out = 0;
    count = 0;
    equal = 1;
    numbits = 0;
    
    dest = malloc(len * 2);
    if (!dest) return NULL;
    
    
    /* FOR TESTING */
    c1 = 0;
    depth = 5;
    equal = 128;
    bitbuf = (bitbuf<<8) | (256 - (equal - 1));
    val = (bitbuf>>numbits);
    Printf ("EncodeRLE - RLE test whole value: %d\n", val);
    if(depth < 8)
    {
        numbits += depth;
        bitbuf = (bitbuf<<depth) | c1;
        //val = bitbuf<<(8 - numbits);
        val = (bitbuf>>numbits);
        Printf ("EncodeRLE - RLE test encoded value: %d\n", val);
    }
    depth = 8;
    equal = 1;
    
    
    Printf ("EncodeRLE - RLE total length: %ld\n", len);    

    c1 = srce[0]; /* previous byte */    
    for (idx = 1; idx < len+1; idx++)
    {
        c2 = srce[idx]; /* next byte */        
        if ((idx < len) && (c1 == c2)) /* There is a byte run - value is same */
	    {  
            equal++; /* increment equal */
            /* Force 'if (runCount == 128)' because we start
			at runCount = 1 so that we have all 127 bytes */
		    if (equal == 128) //if (runCount == 128)
		    {				
				/* Don't exceed 127 max bytes */		
				equal = 128;          		          
			    dest[out] = 256 - (equal - 1);		                			              
                dest[out+1] = c1;		               		                
		        cnt = 2; //2-byte pair 
			    /* reset run */
                out += cnt;                 
                equal = 1;
                prv = (idx+equal)-1;                
	        }	    
	        if (count > 1) //AAACBBB not AAABBB count must be >1 for true transition
	        {	        
                /* write out the number of single bytes */                                
                dest[out] = count - 2; //Don't include 1st NOT equal		                			              
                dest[out+1] = c1;                
                /* Copy single bytes after RLE count */
                //memcpy(dest+out+1,srce+idx-count,count-1);
                j = idx-count-1;
                for (i=1; i < count; i++)
                {
                    dest[out+i] = srce[j+i];
                }
                                
		cnt = count; //RLE count + number of single bytes
                pdx = out; //store current count location 
                /* reset run */
                out += cnt;
                count = 0;
                prv = (idx+count)-1;
            }
            if (count == 1) count = 0;
            //Printf ("EncodeRLE - RLE previous index value: %ld\n", prv);            
        }        
        else /* There is No byte run - value is different */
        {
            //Printf ("EncodeRLE - RLE previous index value: %ld\n", prv);
            //if((count + prv) < len)
            //if((count + prv) < 51);  
                count++; /* increment count */
            //Printf ("EncodeRLE - RLE count value: %ld\n", count);
            if (equal == 2 && dest[pdx] <= 128)
            {
                count += equal;
                equal = 1;
                count += dest[pdx]+1; //Previous count will be overwritten                                
                out = pdx; //pdx is previous out location
            }
            if ((equal > 1) && (equal <= 128))
            {
                /* write out the number of equal bytes */
                /* rle byte must be encoded as (256 - (equal - 1)) */                                                            
                dest[out] = 256 - (equal - 1); 
                dest[out+1] = c1; //unencoded byte                
		        cnt = 2; //2-byte pair 
                /* reset run */
                out += cnt;
                equal = 1;
                prv = (idx+equal)-1;
            }
            //Printf ("EncodeRLE - RLE previous index value: %ld\n", prv);                       
        }
        c1 = c2;                	
    }

    /* Finish up last byte */
    if ((count > 1) || (equal > 1))	
    {
        if (count > 1)
        {    
            //Printf ("EncodeRLE - RLE total length: %ld\n", len);                    
            //count = (len - pdx);
            //if((count + prv) < 51);
            //Printf ("EncodeRLE - RLE size value: %ld\n", out);
            //Printf ("EncodeRLE - RLE index value: %ld\n", (idx-count)+1);            
            /* write out the number of single bytes */
            dest[out] = count - 2; //Don't include 1st NOT equal
            /* Copy single bytes after RLE count */
            //memcpy(dest+out+1,srce+idx-count,count-1);
            j = idx-count-1;
            for (i=1; i < count; i++)
            {
                dest[out+i] = srce[j+i];
            }
                        
	    cnt = count; //RLE count + number of single bytes	        
	    //res = out;                                        
            out += cnt;          
                        
        }
        if (equal > 1)
        {
            //Printf ("EncodeRLE - RLE size value: %ld\n", out);
            //Printf ("EncodeRLE - RLE index value: %ld\n", (idx-count)+1);
            /* write out the number of equal bytes */
            /* rle byte must be encoded as (256 - (equal - 1)) */                              
            dest[out] = 256 - (equal - 1);		                			              
            dest[out+1] = c1;		               		                
            cnt = 2; //2-byte pair            
            /* reset run */
            //res = out;            
            out += cnt;          
                     
        }
        //Printf ("EncodeRLE - RLE loop index value: %ld\n", (idx-count)+1);
        //Printf ("EncodeRLE - RLE loop size value: %ld\n", out);        
    }
    Printf ("EncodeRLE - RLE size value: %ld\n", out); 

    /* Check Compression */
    if(out > len) /* no RLE */
    {
        out = len;
        *dtype = 0;
        for(i = 0; i < len; ++i)
	    {            
            dest[i]= srce[i];
	    }
    }
    else
    *dtype = 1;       
    *dsize = out;
    //*dsize = 43;
    
         
    return dest;  
}

/**************************************************************************************************/

//static char *EncodeNew(unsigned long depth, unsigned char *dtype, LONG *dsize, unsigned long size,
    	    	        //unsigned char *src)
static char * createrle(unsigned long depth, unsigned char *dtype, LONG *dsize, unsigned long size,
    	    	        unsigned char *src)
{
	/**************************************/
    /*    FUNCTION BASED ON MODIFYICON    */
    /**************************************/
    
	int i, j, k;    
    unsigned char *buf;
	unsigned char byte;
	unsigned char loop;
	unsigned char loopbytes;
	unsigned char singlebytes;
	unsigned long bitbuf, numbits;
    long ressize, numcopy, numequal;
	//entries = size

    buf = malloc(size * 2);
    if (!buf) return NULL;
  
    numcopy = 0;
    numequal = 1;
    bitbuf = 0;
    numbits = 0;
    ressize = 0;
    k = 0; /* the real output pointer */


    for(i = 1; numequal || numcopy;)
    {
        if(i < size && numequal && (src[i-1] == src[i]))
        {
            ++numequal; ++i; //loop byte
        }
        else if(i < size && numequal*depth <= 16) //2 not equal is 2 (or less)*8 <=16
        {
            /* counts the single bytes till the next equal byte */
			//Runs of two bytes adjacent to non-runs are typically written as literal data.
			numcopy += numequal; //increment numcopy
			numequal = 1; ++i; //reset numequal
        }
        else
        {
            /* care for end case, where it maybe better to join the two */
			//if(i < size && numcopy + numequal <= 128 && (numequal-1)*depth <= 8) ??
            if(i == size && numcopy + numequal <= 128 && (numequal-1)*depth <= 8)
            {
                //last entry?? //counts the single bytes??
				numcopy += numequal; //capture numequal
				numequal = 0; //reset numequal
            }

            if(numcopy) //singlebytes
            {
                if((j = numcopy) > 128) j = 128;
                bitbuf = (bitbuf<<8) | (j-1);
                numcopy -= j; //reset numcopy
            }
            else //numequal, loopbytes
            {
                if((j = numequal) > 128) j = 128;
                bitbuf = (bitbuf<<8) | (256-(j-1));
                numequal -= j; //reset numequal = 0
                k += j-1;
                j = 1;
            }            
	        byte = (bitbuf >> numbits); //numbits = 0
                //Printf ("createrle - RLE actual value: %d\n", byte);
	        if(byte >= 128) loop = (256-byte) + 1; //loop byte
	        else if(byte < 128) loop = byte + 1; //single byte
		//if(loop) loopbytes += 2; //control byte + byte value
	        //Printf ("createrle - RLE loop value: %d\n", loop);
		buf[ressize++] = (bitbuf >> numbits);

            while(j--) //process one time for each j
            {
                numbits += depth; //start encode by depth
                bitbuf = (bitbuf<<depth) | src[k++]; //load the byte

                if(numbits >= 8) //for pal numbits = 8
                {
                    numbits -= 8; //reset numbits = 0
                    buf[ressize++] = (bitbuf >> numbits);
		            byte = (bitbuf >> numbits); //numbits = 0
                    //Printf ("createrle - RLE whole byte: %d\n", byte);
                }
            }

            if(i < size && !numcopy && !numequal)
            {
                numequal = 1; ++i; //reset numequal = 1
            }
        }//end else /* care for end case */
    }//end for(k = 0...)

    if(numbits) //finish encode by depth
	{
        buf[ressize++] = bitbuf << (8-numbits);
        byte = bitbuf << (8-numbits);
        //Printf ("createrle - encoded byte: %d\n", byte);
	}

    if(ressize > size) /* no RLE */
    {
        ressize = size;
        *dtype = 0;
        for(i = 0; i < size; ++i)
	    {
            buf[i]= src[i];
	        byte = buf[i];
            Printf ("createrle - no RLE byte value: %d\n", byte);
	    }
    }
    else
    *dtype = 1;    
    *dsize = ressize;

	Printf ("createrle - total entries: %ld\n", size);
	Printf ("createrle - RLE loopbytes: %d\n", loopbytes);
  
    return buf;
}

/**************************************************************************************************/

static void MakePlanar(unsigned char *inbuffer, unsigned char *outbuffer, LONG width, LONG height, LONG depth)
{
    int y; //Y-Loop iterator
    LONG kOffset, pOffset; //chunky & planar scanline offsets   
    LONG alignedwidth = (width + 15) & ~15; //Chunky scanline length //Modulus 16
    LONG bytesPerRow = (alignedwidth / 8); //Bitplane length
    LONG linelength = (bytesPerRow * depth); //Planar scanline length

    //Printf("MakePlanar");

    /* Set up line buffer to receive chunky data */
    UBYTE chunkyBuffer[alignedwidth];

    /* Set up line buffer to receive planar data */
    UBYTE bitBuffer[linelength];

    /* Set up Y-Loop to process all scanlines */
    for(y = 0; y < height; y++)
    {
        kOffset = (width * y); /* chunky offset */
        memcpy(chunkyBuffer, inbuffer+kOffset, width); /* copy chunky data */       

        /* Convert Chunky ScanLine to Planar ScanLine */	      
        ChunkyToBitplanes(chunkyBuffer, bitBuffer, depth, alignedwidth);

        pOffset = (linelength * y); /* planar offset */
        memcpy(outbuffer+pOffset, bitBuffer, linelength); /* copy planar data */        
    }
}

/**************************************************************************************************/

//static void c2p(unsigned char *source, unsigned char *dest, LONG width, LONG height, LONG planes)
//{
    /**************************************/
    /*    FUNCTION BASED ON ILBMTOICON    */
    /**************************************/

	//LONG alignedwidth, x, y, p, bpr, bpl;
    
    //alignedwidth = (width + 15) & ~15;
    //bpr = alignedwidth / 8;
    //bpl = bpr * planes;
    
    //for(y = 0; y < height; y++)
    //{
	//for(x = 0; x < width; x++)
	//{
	    //LONG mask   = 0x80 >> (x & 7);
	    //LONG offset = x / 8;
	    //unsigned char chunkypix = source[x];

	    //for(p = 0; p < planes; p++)
	    //{
		//if (chunkypix & (1 << p))
	    	    //dest[p * bpr + offset] |= mask;
		//else
		    //dest[p * bpr + offset] &= ~mask;
	    //}
	//}

    	//source += width;
    	//dest += bpl;
    //}
 
//}

/**************************************************************************************************/

static int ChunkyToBitplanes(UBYTE *chunkyBuffer, UBYTE *bitBuffer, LONG bitPlanes, LONG imagewidth) 
{    
    /*************************************/
    /*    FUNCTION BASED ON ICONSPLIT    */
    /*************************************/

	int i, j;
    int hOffset = 0; //scanline offsets
    int bitplaneIndex = 0; //Bitplane index
    //int alignedwidth = (imagewidth + 15) & ~15; //Modulus 16   
    int bytesPerLine = (imagewidth / 8); //Bitplane length
    int rowSize = (bytesPerLine * bitPlanes); //Planar scanline length

    /* Converts one scanline from chunky to planar */

    //Printf("ChunkyBytesToBitplanes");
    
    //Block of 8 bytes    
    UBYTE colorBytes[8];   
    
        
    while (hOffset < bytesPerLine)
    {
        //Copy 8 bytes to new buffer. Chunky Buffer is Modulus 16 & therefore Modulus 8.
        memcpy(colorBytes, chunkyBuffer + (hOffset * 8), 8);       
        for (j = 0; j < bitPlanes; j++) //Fill bitplane byte locations (Up to bytesPerLine)
        {
            bitplaneIndex = (j * bytesPerLine);
            //UBYTE bitmask = (UBYTE)(1 << bit);
            
            for (i = 0; i < 8; i++) //Set corresponding bits in bitplane buffers.
            {
                /* Check if the current bit of the index value is set */
                if ((colorBytes[i] & (1 << j)) != 0)
                {                    
                    /* (Bit is Set (1)) Modify the current bit in the bitplane byte to be 1 and leave the others untouched */
                    bitBuffer[bitplaneIndex + hOffset] |= (UBYTE)(1 << 7 - i);
                    //Set bit = number |= 1 << x; //return value |= (UBYTE)(1 << pos);
                }
                else
                {                    
                    /* (Bit is Not Set (0)) Modify the current bit in the bitplane byte to be 0 and leave the others untouched */
                    bitBuffer[bitplaneIndex + hOffset] &= (UBYTE)(~(1 << 7 - i));
                    //Clear bit = number &= ~(1 << x); //return value &= (UBYTE)(~(1 << pos));
                }
            }            
        }
        hOffset++;
	
    } //End while loop

    return 0;        
}

/**************************************************************************************************/

static void BitplanesToChunky(unsigned char *source, unsigned char *dest, LONG width, LONG height,
    	    	LONG totplanes, LONG wantplanes, LONG chunkybpr)
{
	/**************************************/
    /*    FUNCTION BASED ON ILBMTOICON    */
    /**************************************/

    LONG alignedwidth, x, y, p, bpr, bpl;
    
    alignedwidth = (width + 15) & ~15;
    bpr = alignedwidth / 8;
    bpl = bpr * totplanes;
    
    for(y = 0; y < height; y++)
    {
    	for(x = 0; x < width; x++)
	{
	    LONG mask   = 0x80 >> (x & 7);
	    LONG offset = x / 8;
	    unsigned char chunkypix = 0;
	    
	    for(p = 0; p < wantplanes; p++)
	    {
	    	if (source[p * bpr + offset] & mask) chunkypix |= (1 << p);
	    }
	    dest[x] = chunkypix;
	}
	
	source += bpl;
	dest += chunkybpr;
    }
}

/**************************************************************************************************/

static unsigned char *UnpackByteRun(unsigned char *source, unsigned char *dest, LONG unpackedsize)
{	
    unsigned char r;
    signed char c;
    
    for(;;)
    {
	c = (signed char)(*source++);
	if (c >= 0)
	{
    	    while(c-- >= 0)
	    {
		*dest++ = *source++;
		if (--unpackedsize <= 0) return source;
	    }
	}
	else if (c != -128)
	{
    	    c = -c;
	    r = *source++;

	    while(c-- >= 0)
	    {
		*dest++ = r;
		if (--unpackedsize <= 0) return source;
	    }
	}
    }
    
}

/**************************************************************************************************/

//static unsigned char *PackByteRun(unsigned char *source, unsigned char *dest, LONG packedsize)

/**************************************************************************************************/

static void FreeIconImage(struct ILBMImage *img)
{ 
	/**************************************/
    /*    FUNCTION BASED ON ILBMTOICON    */
    /**************************************/

    if (img->chunkybuffer)
    {
    	free(img->chunkybuffer);
	    img->chunkybuffer = NULL;
    }
    
    if (img->planarbuffer)
    {
    	free(img->planarbuffer);
	    img->planarbuffer = NULL;
    }

    if (img->argb)
    {
        free(img->argb);
        img->argb = NULL;
    }
    
    if (filebuffer)
    {
    	free(filebuffer);
	    filebuffer = NULL;
    }
    
    if (file)
    {
    	fclose(file);
	    file = NULL;
    }    
}

/**************************************************************************************************/

//static void Clear_Images(void)
static void cleanup(char *msg)
{
    if (msg) fprintf(stderr, "iconpress error: %s\n", msg);
    
    FreeIconImage(&img1);
    FreeIconImage(&img2);
}

/**************************************************************************************************/

//static void Clear_Image(char *imagetype, int index)
//{
    /* Free images by type & index - ilbm1,ilbm2,img1,img2 */
    //Sample:
    //if(stricmp(imagetype, "iff")==0)
    //if(image1option) Clear_Image("iff", 0)
    //if(image2option) Clear_Image("iff", 1)
    //Sample:
    //if(stricmp(imagetype, "png")==0)
    //if(image1option) Clear_Image("png", 0)
    //if(image2option) Clear_Image("png", 1)

    //if(ilbm1) ilbm->planarbuffer = NULL;
    //...

//}

//static char *Load_PNG(char *imagename, struct ILBMImage *img)
//static void Load_PNG(char *imagename, int index)
//{
    //struct PNGImage
    //{
    //struct BitMapHeader bmh; 
    //struct BitMapHeader planarbmh;
    //ULONG               planar_size;
    //unsigned char   	  *planarbuffer;   
    //unsigned char   	  *chunkybuffer;
    //unsigned char       *planarcmap;
    //ULONG               cmap_size;
    //LONG    	    	  cmapentries, 
    //LONG                rowsize; 
    //LONG                totdepth; 
    //UBYTE   	    	  cmap[256][3];   
    //UBYTE   	    	  remaptable[256];
    //APTR                png;
    //ULONG               png_size;
    //APTR                argbbuffer;
    //ULONG               argb_size;
    //};

    /* Load PNGImage by Datatype */
    //Printf ("ReadPixels_by_datatype 32bit\n");
	//if(index == 0) 
	//{
        //retval = ReadPixels_by_datatype_alpha(png1, imagename);
	//}
	//if(index == 1) 
	//{
        //retval = ReadPixels_by_datatype_alpha(png2, imagename);
	//}   

    /* Fill remaining data for PNGImage */
    //img->planarbuffer, img->chunkybuffer, img->png, img->pngsize, img->argb, img->argbsize
//}

/**************************************************************************************************/

//static char *Save_ILBM(char *name, struct IconImage *im)
static BOOL Save_ILBM(char *imagename, struct ILBMImage *img)
{     
    /*************************************/
    /*    FUNCTION BASED ON ICONSPLIT    */
    /*************************************/	

    int i;
    LONG transp;
    LONG numpal = 0;    
    LONG ilbmsize = 0;
	LONG formsize = 0;
    LONG numcolors = 0;
    LONG width, height, depth;

    unsigned char *imdata;
    unsigned char *imgbits;    

    width = img->bmh.bmh_Width; 
    height = img->bmh.bmh_Height; 
    depth = img->bmh.bmh_Depth;  
    transp = img->bmh.bmh_Transparent; 
    numcolors = 1<<depth; //No. colors by depth
	numpal = numcolors*3; //Colormap size in bytes     
	     

    /* Colors by depth not numentries */
	UBYTE rgb[numcolors*3]; //if depth = 5, numcolors = 32
	for(i = 0; i < img->cmapentries; i++)    
    {
        //Convert Palette to CMAP Byte Array        
		rgb[i * 3] = img->rgb[i][0];     //red
        rgb[(i * 3)+1] = img->rgb[i][1]; //green
        rgb[(i * 3)+2] = img->rgb[i][2]; //blue
    }
    
    /* Add padding bytes if needed for each scanline */
	LONG alignedwidth = (img->bmh.bmh_Width + 15) & ~15; //Modulus 16
    LONG bytesPerRow = (alignedwidth / 8); 
    LONG linelength = (bytesPerRow * depth); //Planar scanline length
    LONG planarsize = (linelength * img->bmh.bmh_Height);

    Printf ("depth: %ld\n", depth);
    Printf ("alignedwidth: %ld\n", alignedwidth);
	Printf ("bytesPerRow: %ld\n", bytesPerRow);
	Printf ("linelength: %ld\n", linelength);
	Printf ("planarsize: %ld\n", planarsize);
        
    imgbits = AllocMem( planarsize, MEMF_ANY );    

    //unsigned char *inbuffer, *outbuffer
	MakePlanar(img->chunkybuffer, imgbits, width, height, depth);	        
    
    ilbmsize = ( 40 + (numpal+8) + (planarsize+8) );
    formsize = (ilbmsize - 8);

    Printf ("ILBM Planar File Size: %ld\n", ( 40 + (numpal+8) + (planarsize+8) ));
	Printf ("ILBM Planar Form Size: %ld\n", ( 40 + (numpal+8) + (planarsize) ));
    
    imdata = AllocMem( ilbmsize, MEMF_ANY );

    /* Build new byte array for bmhd */
    unsigned char bmhd[20];
    bmhd[0] =  (width >> 8); //Width
    bmhd[1] =  (width & 0xFF);
    bmhd[2] =  (height >> 8); //Height
    bmhd[3] =  (height & 0xFF);
	bmhd[4] =  bmhd[5] = 0; //Left Offset
    bmhd[6] =  bmhd[7] = 0; //Top Offset
    bmhd[8] =  depth;
	bmhd[9] =  0; //Masking
    bmhd[10] = 0; //Compression //imgfmt
    bmhd[11] = 0; //Padding
    bmhd[12] = (transp>> 8); 
    bmhd[13] = (transp & 0xFF);
    bmhd[14] = 10; //XAspect;
    bmhd[15] = 11; //YAspect;
    bmhd[16] = (width >> 8); //PageWidth
    bmhd[17] = (width & 0xFF);
    bmhd[18] = (height >> 8); //PageHeight
    bmhd[19] = (height & 0xFF);

    /* Build new byte array with ilbm data */
    PutUint32(imdata, 0x464F524D); //ID_FORM 0x464F524D					
	PutUint32(imdata+4, formsize); 
	PutUint32(imdata+8, 0x494C424D); //ID_ILBM 0x494C424D
	PutUint32(imdata+12, 0x424D4844); //ID_BMHD 0x424D4844
	PutUint32(imdata+16, 20);
	memcpy(imdata+20, bmhd, 20);
    PutUint32(imdata+20+20, 0x434D4150); //ID_CMAP 0x434D4150
	PutUint32(imdata+24+20, numpal);
    memcpy(imdata+28+20, rgb, numpal);
    PutUint32(imdata+48+numpal, 0x424F4459); //ID_BODY 0x424F4459     
    PutUint32(imdata+52+numpal, planarsize);  
    memcpy(imdata+56+numpal, imgbits, planarsize);
    
    /* Write ILBM */
    WriteData(imagename, imdata, ilbmsize);

    /* Free Resources */
    if (imdata) FreeMem( imdata, ilbmsize );
    if (imgbits) FreeMem( imgbits, planarsize );    
    
    return TRUE;
}

/**************************************************************************************************/

//static char *Load_ILBM(char *imagename, struct ILBMImage *img)
static void Load_ILBM(char *imagename, struct ILBMImage *img)
{
	/*************************************/
    /*    FUNCTION BASED ON ICONSPLIT    */
    /*************************************/
	
	int i;
	LONG filesize;
	LONG formsize;	
	ULONG chunkid;
	ULONG chunksize;	
	LONG cmapoffset;
	LONG numcolors;			

	have_bmhd = 0; //has_bmhd //
    have_cmap = 0; //has_cmap //
    have_body = 0; //has_cmap //


	FreeIconImage(img);	

    /* Get total icon image size */
	filesize = GetFileSize(imagename);	
	
	file = fopen(imagename, "rb");
    if (!file) cleanup("Can't open file!");

    //fseek(file, 0, SEEK_END);
    //filesize = ftell(file);

    fprintf(stderr, "Filesize is %ld\n", filesize);    
    if (filesize < 12) cleanup("Bad file size!");
    
    filebuffer = malloc(filesize + 10);
    if (!filebuffer) cleanup("Memory allocation for file buffer failed!");

	fseek(file, 0, SEEK_SET);    
    if (fread(filebuffer, 1, filesize, file) != filesize)
    	cleanup("Error reading file!");
    
    fclose(file); 
	file = NULL;	

	
	//Verify the ILBM file signature which is also done in Display_Image
	if(GetUint32(filebuffer) != 0x464F524D || GetUint32(filebuffer+8) != 0x494C424D) //FORM & ILBM
	    cleanup("Reading ILBM file failed!"); //return error;
	else
	    fprintf(stderr, "Success reading ILBM file!\n");

	formsize = GetUint32(filebuffer+4); //formsize
	fprintf(stderr, "Formsize is %ld\n", formsize);

	//fileoffset += 12;
	filebuffer += 12;

	
	/* Load Image */
	while(formsize > 0)
	{
		chunkid  = GetUint32(filebuffer);
	    chunksize = GetUint32(filebuffer+4);	
		filebuffer += 8;	

		switch(chunkid)
		{
			case 0x424D4844: //ID_BMHD
			    fprintf(stderr, "Found chunkid BMHD in ILBM file\n");
			    if (chunksize != 20) cleanup("Bad BMHD chunk size!");	    

	            /* Read BMHD data from ILBM */				
	            //GetUint8(filebuffer+20) //GetUint16(filebuffer+20) GetUint32(filebuffer+16)
	            img->bmh.bmh_Width 	     = GetUint16(filebuffer); //+fileoffset+8=20 
	            img->bmh.bmh_Height      = GetUint16(filebuffer+2);
	            img->bmh.bmh_Left 	     = (WORD)GetUint16(filebuffer+4);
	            img->bmh.bmh_Top 	     = (WORD)GetUint16(filebuffer+6);
	            img->bmh.bmh_Depth 	     = filebuffer[8]; 
	            img->bmh.bmh_Masking     = filebuffer[9]; 
	            img->bmh.bmh_Compression = filebuffer[10]; 
	            img->bmh.bmh_Pad 	     = filebuffer[11]; 
	            img->bmh.bmh_Transparent = GetUint16(filebuffer+12);
	            img->bmh.bmh_XAspect     = filebuffer[14]; 
	            img->bmh.bmh_YAspect     = filebuffer[15]; 
	            img->bmh.bmh_PageWidth   = (WORD)GetUint16(filebuffer+16);
	            img->bmh.bmh_PageHeight  = (WORD)GetUint16(filebuffer+18); //offset = 38

	            if (img->bmh.bmh_Depth > 8) cleanup("ILBM file has too many colors!");
	            if ((img->bmh.bmh_Compression != 0) && (img->bmh.bmh_Compression != 1)) cleanup("Compression method unsupported!");

	            /* Verify */
				have_bmhd = 1;

	            img->totdepth = img->bmh.bmh_Depth + ((img->bmh.bmh_Masking == 1) ? 1 : 0);
	            img->bpr = ((img->bmh.bmh_Width + 15) & ~15) / 8;
	            img->planarbmh = img->bmh;

	            fprintf(stderr, "BMHD Dimensions: %d x %d x %d (%d)\n", img->bmh.bmh_Width, img->bmh.bmh_Height, img->bmh.bmh_Depth, img->totdepth);
				break;
			case 0X434D4150: //ID_CMAP
			    fprintf(stderr, "Found chunkid CMAP in ILBM file\n");
				if (!have_bmhd) cleanup("CMAP chunk before BMHD chunk (or no BMHD chunk at all!");				

				numcolors = chunksize / 3;
				if (chunksize & 1) chunksize++;
				fprintf(stderr, "Number of colors: %ld\n", numcolors);
				
								
				/* Load CMAP data from ILBM */				
				cmapoffset = 0;
				for(i = 0; i < numcolors; i++)
		        {
		            img->rgb[i][0] = filebuffer[cmapoffset]; 
		            img->rgb[i][1] = filebuffer[cmapoffset+1]; 
		            img->rgb[i][2] = filebuffer[cmapoffset+2]; 	            
					cmapoffset += 3;					
		        }
                /* Verify */
		        have_cmap = 1;

		        break;
			case 0x424F4459: //ID_BODY
			    fprintf(stderr, "Found chunkid BODY in ILBM file\n");
			    if (!have_bmhd) cleanup("BODY chunk before BMHD chunk (or no BMHD chunk at all!");						        

				if (img->bmh.bmh_Compression == 0) //CMP_NONE
		        {
		            LONG bodysize = img->totdepth * img->bpr * img->bmh.bmh_Height;
		            if (chunksize != bodysize) cleanup("BODY chunk size seems to be wrong!");
		        }
				
	            LONG unpackedsize = img->bpr * img->bmh.bmh_Height * img->totdepth;

				img->planarbuffer = malloc(unpackedsize);
	            if (!img->planarbuffer) cleanup("Memory allocation for planar buffer failed!");

				/* Load BODY data from ILBM */
				if (img->bmh.bmh_Compression == 0) //CMP_NONE
                {
    	            memcpy(img->planarbuffer, filebuffer, unpackedsize);
                }
                else
                {					
					/* Uncompress BODY */
					UnpackByteRun(filebuffer, img->planarbuffer, unpackedsize);    	            
                }
                /* Allocate memory for chunky buffer */
				img->chunkybuffer = malloc(img->bmh.bmh_Width * img->bmh.bmh_Height);
                if (!img->chunkybuffer) cleanup("Memory allocation for chunky buffer failed!");

				/* Conver planar to chunky */
				BitplanesToChunky(img->planarbuffer, img->chunkybuffer, img->bmh.bmh_Width, img->bmh.bmh_Height, img->totdepth,	img->bmh.bmh_Depth, img->bmh.bmh_Width);
				//img->cmapentries = chunksize / 3;
				//if (chunksize & 1) chunksize++;
				
				img->cmapentries = Find_High_Index(img->chunkybuffer, img->bmh.bmh_Width*img->bmh.bmh_Height); 
                                if ((img->cmapentries < 2) || (img->cmapentries > 256)) cleanup("CMAP chunk has bad number of entries!");
				fprintf(stderr, "Number of used colors: %ld\n", img->cmapentries);
				
				
			    /* Verify */
				have_body = 1;		        
		
	        default:	    	    
		    break;
	    }
		if (chunksize & 1) chunksize++;
		filebuffer += chunksize;
		formsize -= 8+chunksize;		
	    if (have_bmhd && have_body && have_cmap) break;		
    }
	/* Cleanup */
	if(formsize < 0) cleanup("Bad ILBM filesize!");
	if (!have_bmhd) cleanup("BMHD chunk missing!");
    if (!have_body) cleanup("BODY chunk missing!");    

	//return 0;     
}

/**************************************************************************************************/

static long Find_High_Index(unsigned char *colors, long length)
{
    int i;    
    unsigned char c1, c2;
    
    c1 = colors[0]; /* previous index */
    for(i=1; i < length; i++)
    {
        c2 = colors[i]; /* next index */
        if(c2 > c1) 
            c1 = c2;                  
    }
    return c1+1;
}

/**************************************************************************************************/

//static void Write_ARGBdata(char *infile1, char *infile2, char *argbfile, LONG pngoffset)
//static void Write_ARGBdata(char *infile1, char *infile2, char *argbfile)
//{
    //Write OS 3.5 IFF Data to Test File.
    //Use Create_RLE to Calculate IMAG Chunk Sizes
    //Calculate formsize based on (IMAG Chunk Sizes + 26) - 8

	//LONG value = 0;
	//LONG offset = 0;
	//LONG formsize = 0;
	//UBYTE Buffer32[4];
	//unsigned char *imagdata; 
	//img1->rledata, img1->rlesize

	//PNGImage img1 = NULL;
	//PNGImage img2 = NULL;
    //ILBMImage ilbm1 = NULL;
	//ILBMImage ilbm2 = NULL;

	//img1 = LoadImage(infile1);
	//if (image2option) img2 = LoadImage(infile2);

	//Use PNGImage to contain APTR png; ULONG png_size; APTR argb; ULONG argb_size;
//}

/**************************************************************************************************/

static void Write_Image(BPTR outfile, struct ILBMImage *img, LONG offset)
{
    /* Calculated size of the planar image data */
	//filebytes = (((((Width+15)&(~15)) * Height) * Depth)>>3);
	LONG filebytes = (((((img1.bmh.bmh_Width+15)&(~15)) * img1.bmh.bmh_Height) * img1.bmh.bmh_Depth)>>3);
	Printf ("Write_Image - Planar Image Size: %d\n", filebytes);

	LONG x, y;
	unsigned char planardata[20];
	//planardata = AllocMem( filebytes+20, MEMF_ANY );
	
	//EndPutM16Inc(buf, 0);                          /* im_LeftEdge */
    //EndPutM16Inc(buf, 0);                          /* im_TopEdge */
    //EndPutM16Inc(buf, newicon->do_Gadget.Width);   /* im_Width */ //newicon->Image.Width
    //EndPutM16Inc(buf, newicon->do_Gadget.Height);  /* im_Height */ //newicon->Image.Height
    //EndPutM16Inc(buf, newicon->do_Gadget.Depth);   /* im_Depth */ //newicon->Image.Depth
    //EndPutM32Inc(buf, 0x2A2A2A2A);                 /* im_ImageData */
    //*(buf++) = 0;                                  /* im_PlanePick */
    //*(buf++) = 0;                                  /* im_PlaneOnOff */
    //EndPutM32Inc(buf, 0);                          /* im_Next */

    /* Build new byte array for planar data */
	PutUint16(planardata, 0);                        /* im_LeftEdge */
	PutUint16(planardata+2, 0);                      /* im_TopEdge */
	PutUint16(planardata+4, img->bmh.bmh_Width);    /* im_Width */
	PutUint16(planardata+6, img->bmh.bmh_Height);   /* im_Height */
	PutUint16(planardata+8, img->bmh.bmh_Depth);    /* im_Depth */
	PutUint32(planardata+10, 0x2A2A2A2A);            /* im_ImageData */
	planardata[14] = 0;                              /* im_PlanePick */
	planardata[15] = 0;                              /* im_PlaneOnOff */
	PutUint32(planardata+16, 0);                     /* im_Next */
	/* Total Size = 20 bytes */

	/* Copy image planar buffer to char array */
	//memcpy(planardata+20, img->planarbuffer, filebytes);

	/* Find imageoffset for output file */
	Seek(outfile,offset,OFFSET_BEGINNING);
	/* Write the byte buffer to output file */
	//Write(outfile, planardata, filebytes + 20);

	//if( Write( outfile, planardata, 20) != 20 )
	if( FWrite( outfile, planardata, 20, 1 ) != 20 )
	{
     	cleanup("Error writing image header to outfile!");
	}
	//offset += 20;

	/* Write planar image data to file */
	for(x = 0; x < img->planarbmh.bmh_Depth; x++)
    {
    	UBYTE *dat = img->planarbuffer + img->bpr * x;	
	    for(y = 0; y < img->planarbmh.bmh_Height; y++)
	    {
	        
			//Write(outfile, buf, filebytes + 20);
			//FWrite( file, linebuf, bpr, 1 )
			//if( FWrite( outfile, dat, img->bpr, 1 ) != img->bpr )
			//LONG FWrite( BPTR fh, CONST APTR block, ULONG blocklen, ULONG number ); 
			//if(fwrite(dat, 1, img->bpr, outfile) != img->bpr)
			if( FWrite( outfile, dat, img->bpr, 1 ) != img->bpr )
	        {
     	        cleanup("Error writing image data to outfile!");
	        }
	        dat += (img->planarbmh.bmh_Depth * img->bpr);
			//offset += (img->planarbmh.bmh_Depth * img->bpr);
	    }
    }	
	/* Free Resources */
    //if (planardata) FreeMem( planardata, filebytes+20 );
}

static LONG writeimagchunk(struct ILBMImage *img, char *outfilename)
{
    //struct imagchunk ic;  
    LONG imagsize;
    UBYTE equalpalette = 0;
    UBYTE *pal, *gfx;
    LONG palsize, gfxsize;
    UBYTE palpacked, gfxpacked;
    
    WORD i;
    
    imagsize = 10; //sizeof(struct imagchunk);

	/* if this is second image check whether palette is identical to
       the one of first image */

	if (img == &img2)
	{
        if (img1.cmapentries == img2.cmapentries)
		{
            for (i = 0; i < img1.cmapentries; i++)
			{
				if (img1.rgb[i][0] != img2.rgb[i][0]) break;
	    	    if (img1.rgb[i][1] != img2.rgb[i][1]) break;
	    	    if (img1.rgb[i][2] != img2.rgb[i][2]) break;
			}
			if (i == img1.cmapentries) equalpalette = 1;
		}		
	}

	if (!equalpalette)
	{
    	pal = createrle(8,
	    	    	&palpacked,
			&palsize,
			img->cmapentries * 3,
			(unsigned char *)img->rgb);
			
    	imagsize += palsize;
    }

	gfx = createrle(img->bmh.bmh_Depth,
    	    	    &gfxpacked,
		    &gfxsize,
		    img->bmh.bmh_Width * img->bmh.bmh_Height,
		    img->chunkybuffer);

	imagsize += gfxsize;

	
	fprintf(stderr, "Writing to icon file: %s\n");
	FILE *outfile = fopen(outfilename, "wb");

	if (fwrite(gfx, 1, gfxsize, outfile) != gfxsize)
    {
    	cleanup("Error write gfx data in imag chunk!");
    }

	if (!equalpalette)
    {
	    if (fwrite(pal, 1, palsize, outfile) != palsize)
	    {
    	    cleanup("Error write palette data in imag chunk!");
	    }    	
    }
	fclose(outfile);
}

/**************************************************************************************************/

//static void Write_OS41data(char *infile1, char *infile2, char *os41file)
//static LONG Write_OS35data(char *infile1, char *infile2, char *os35file, LONG iffoffset, BOOL os41)
static void Write_OS35data(char *infile1, char *infile2, char *os35file, LONG iffoffset)
//static void Write_OS35data(char *infile1, char *infile2, char *os35file)
{
    //Write OS 3.5 IFF Data to Test File.
    //Use Create_RLE to Calculate IMAG Chunk Sizes
    //Calculate formsize based on (IMAG Chunk Sizes + 8 + 8 + 26) - 8
	//IMAG chunk contains IMAG Header & image data followed by palette data
	//return end offset of os35 data

	int i;
	LONG value = 0;
	LONG offset = 0;
	LONG os35size = 0;
	LONG formsize = 0;
	LONG imagsize = 0;	
	LONG imagesize = 0;		
	LONG maxpalsize = 0;	
	UBYTE Buffer32[4];

	LONG imag1, imag2;
	LONG equalpalette = 0;
	LONG planaroffset = 0;
	LONG palsize, imgsize;
    UBYTE palpacked, imgpacked;	
	unsigned char *os35data;
	struct BitMapHeader *bmhd;
	    

	/* Values to calculate imag sizes */
	LONG ilbm1_cmapentries = 0; //numcolors
	LONG ilbm1_planarsize = 0; //imagesize
	LONG ilbm1_cmapsize = 0; //palsize	
	//LONG ilbm1_planarsize = 654; //imagesize
	//LONG ilbm1_cmapsize = 43; //palsize
	LONG ilbm2_cmapentries = 0; //numcolors
	LONG ilbm2_planarsize = 0; //imagesize
	LONG ilbm2_cmapsize = 0; //palsize
	//LONG ilbm2_planarsize = 903; //imagesize
	//LONG ilbm2_cmapsize = 43; //palsize

	/* Buffers for encoded img data & pal data */
	/* Buffers are allocated by the encoder & need to be freed in freeimage */
    unsigned char *ilbm1_planardata, *ilbm1_planarcmap;
	unsigned char *ilbm2_planardata, *ilbm2_planarcmap;


	/* Check equal palette for img1 & img2 */
	/* No palette for IMAG chunks for os41 */
	if ((infile1) && (infile2))
	{
        if (img1.cmapentries == img2.cmapentries)
		{
            for (i = 0; i < img1.cmapentries; i++)
			{
				if (img1.rgb[i][0] != img2.rgb[i][0]) break;
	    	    if (img1.rgb[i][1] != img2.rgb[i][1]) break;
	    	    if (img1.rgb[i][2] != img2.rgb[i][2]) break;
			}
			if (i == img1.cmapentries) equalpalette = 1;
		}		
	}


	//if(os41 == FALSE)
	//{
    /* Encode chunky img data & pal data */
	if (infile1) //index == 0
	{     		
	    //ilbm1_gfxdata = EncodeNew(...)
		Printf ("Write_OS35data - Encode image1 & palette1 data\n");
        ilbm1_planardata = createrle(img1.bmh.bmh_Depth, &imgpacked, &imgsize,
		    img1.bmh.bmh_Width * img1.bmh.bmh_Height, img1.chunkybuffer);
	    ilbm1_planarsize = imgsize; //ilbm1_gfxsize
		Printf("First Encoded IMAG image size: %ld\n", imgsize);
		
		//ilbm1_cmapdata = EncodeNew(...)
		//ilbm1_planarcmap = createrle(8, &palpacked, &palsize, img1.cmapentries * 3,
			//(unsigned char *)img1.rgb);
		//ilbm1_planarcmap = createrle(8, &palpacked, &palsize, img1.cmapentries * 3,	cmap);
		ilbm1_planarcmap = EncodeRLE(8, &palpacked, &palsize, img1.cmapentries * 3,
			(unsigned char *)img1.rgb);
	    ilbm1_cmapsize = palsize;
		Printf("First Encoded IMAG cmap size: %ld\n", palsize);
		if (ilbm1_planarsize % 2 != 0) ilbm1_cmapsize += 1; //pad cmap		
	}
	if (infile2) //index == 1
	{
		//ilbm2_gfxdata = EncodeNew(...)
		Printf ("Write_OS35data - Encode image2 & palette2 data\n");
		ilbm2_planardata = createrle(img2.bmh.bmh_Depth, &imgpacked, &imgsize,
		    img2.bmh.bmh_Width * img2.bmh.bmh_Height, img2.chunkybuffer);
		ilbm2_planarsize = imgsize; //ilbm2_gfxsize
		Printf("Second Encoded IMAG image size: %ld\n", imgsize);

        if (!equalpalette)
		{
			//ilbm2_cmapdata = EncodeNew(...)
		    //ilbm2_planarcmap = createrle(8, &palpacked, &palsize, img2.cmapentries * 3,
			//(unsigned char *)img2.rgb);
			ilbm2_planarcmap = EncodeRLE(8, &palpacked, &palsize, img2.cmapentries * 3,
			(unsigned char *)img2.rgb);
		    ilbm2_cmapsize = palsize;
			Printf("Second Encoded IMAG cmap size: %ld\n", palsize);
			if (ilbm2_planarsize % 2 != 0)ilbm2_cmapsize += 1; //pad cmap			
		}
	}
	//}
	

	/* Calculate MaxPaletteBytes */
	/* No palette for IMAG chunks for os41 */	
	//maxpalsize = (ilbm1.cmapentries > ilbm2.cmapentries) ? ilbm1.cmapentries : ilbm2.cmapentries;	
	maxpalsize = (img1.cmapentries > img2.cmapentries) ? img1.cmapentries : img2.cmapentries;	 
	maxpalsize = maxpalsize * 3;


	/* Fill header information for OS3.5 IFF data */
    //If BOOL os41 == TRUE,  only write the IFF Header & the Face Chunk.
	//OS41 argb icons don't have either of the IMAG chunks.
	//ARGB chunks will be appended to the IFF data later.

	/* Setup face chunk */   /* For both argb & os41 icons */
	unsigned char facechunk[6];
	facechunk[0] = img1.bmh.bmh_Width - 1; //img1->bmh.bmh_Width - 1
	facechunk[1] = img1.bmh.bmh_Height - 1; //img1->bmh.bmh_Height - 1	
	facechunk[2] = 0; //Flags
	facechunk[3] = 17; // 0x11); //Aspect
	facechunk[4] = ((maxpalsize - 1) >> 8); //MaxPaletteBytes
    facechunk[5] = ((maxpalsize - 1) & 0xFF);	
	
	//if(os41 == FALSE)
	//{
	/* Setup IMAG chunk1 */   /* If BOOL os41 == FALSE */
	unsigned char imagchunk1[10];
	imagchunk1[0] = 0; //First color in colormap
	imagchunk1[1] = (img1.cmapentries - 1); //ilbm1->NumColors - 1; //0x10
	imagchunk1[2] = 3; /* 3 or 2, 3 = HasTranparent, 2 = HasPalette */
	imagchunk1[3] = 1; /* cmp = 1 */
	imagchunk1[4] = 1; /* cmp = 1 */
	imagchunk1[5] = img1.bmh.bmh_Depth;
	imagchunk1[6] = ((ilbm1_planarsize - 1) >> 8); //imagesize - 1
	imagchunk1[7] = ((ilbm1_planarsize - 1) & 0xFF);
	imagchunk1[8] = ((ilbm1_cmapsize - 1) >> 8); //palsize - 1
	imagchunk1[9] = ((ilbm1_cmapsize - 1) & 0xFF);
    
	/* Setup IMAG chunk2 */   /* If BOOL os41 == FALSE */
	unsigned char imagchunk2[10];
	imagchunk2[0] = 0; //First color in colormap
	imagchunk2[1] = (img2.cmapentries - 1); //0x10 //ilbm2->NumColors
	imagchunk2[2] = 3; /* 3 or 2, 3 = HasTranparent, 2 = HasPalette */
	imagchunk2[3] = 1; /* cmp = 1 */
	imagchunk2[4] = 1; /* cmp = 1 */
	imagchunk2[5] = img2.bmh.bmh_Depth;
	imagchunk2[6] = ((ilbm2_planarsize - 1) >> 8); //imagesize - 1
	imagchunk2[7] = ((ilbm2_planarsize - 1) & 0xFF);
	imagchunk2[8] = ((ilbm2_cmapsize - 1) >> 8); //palsize - 1
	imagchunk2[9] = ((ilbm2_cmapsize - 1) & 0xFF);
	//}

	
    /* Calculate IMAG Sizes */
	imag1 = ilbm1_planarsize + ilbm1_cmapsize + 10;
	
	if (!equalpalette) 
	    imag2 = ilbm2_planarsize + ilbm2_cmapsize + 10;
	else
	    imag2 = ilbm2_planarsize + 10;

	//if ! imag1%2 add a padding byte
	if (imag1 % 2 != 0) 
	{
	    imag1 += 1;
	    ilbm1_cmapsize += 1;
	}	
	//int mod = imag1 % 2; //0 if no remainder, or non-zero
	//int is_divisible = (mod == 0) ? 0 : 1; //'y' : 'n'
	//if(is_divisible == 1) imag1 += 1; //add 1 byte

	//if ! imag2%2 add a padding byte
	if (imag2 % 2 != 0)
	{
	    imag2 += 1;
	    ilbm2_cmapsize += 1;
	}	
	//mod = imag2 % 2; //0 if no remainder, or non-zero
	//is_divisible = (mod == 0) ? 0 : 1; //'y' : 'n'
	//if(is_divisible == 1) imag2 += 1; //add 1 byte		
	
	Printf("First IMAG chunk size: %ld\n", imag1);
	Printf("Second IMAG chunk size: %ld\n", imag2);

	//FILE *file = fopen(os35file, "rb");
	//fseek(file, 0, SEEK_END);
    //long fileoffset = ftell(file);
	//fclose(file); file = NULL;
	//Printf("File IFF Offset: %ld\n", fileoffset); 


    //Write IFF Header (FORM + formsize + ICON) 12 bytes & FACE chunk (id + facesize + data) 8+6 bytes = 26    
    //Write both IMAG chunks (id + imagsize + header + image data + pal data) +8 bytes each

	/* Calculate Form Size */
	/* No IMAG chunks for os41 */	
	formsize = ((26+imag1+8+imag2+8)-8);
	os35size = formsize + 8;

	os35data = AllocMem( os35size, MEMF_ANY );	
    	
	/* Write to os35data buffer */
	//BPTR fh = Open( os35file, MODE_NEWFILE );	
	BPTR fh = Open( os35file, MODE_OLDFILE );
	if(fh)
	{		 		
		/* Build new byte array with os35 data */
        PutUint32(os35data, 0x464F524D); //ID_FORM 0x464F524D
		PutUint32(os35data+4, formsize);
		PutUint32(os35data+8, 0X49434F4E); //ID_ICON 0X49434F4E
		/* Write Face Chunk */
	    PutUint32(os35data+12, 0X46414345); //ID_FACE 0X46414345
		PutUint32(os35data+16, 6);		
		memcpy(os35data+20, facechunk, 6);
		
        /* Write IMAG Header */
		//if(os41 == FALSE) else iffoffset += 26;
		PutUint32(os35data+26, 0x494D4147); //ID_IMAG 0x494D4147
		imagsize = ilbm1_planarsize + ilbm1_cmapsize + 10;		
		PutUint32(os35data+30, imagsize);
		/* Write IMAG Chunk */
		memcpy(os35data+34, imagchunk1, 10);
		/* Write Image Data */
		planaroffset += 44;	//iffoffset	
		memcpy(os35data+planaroffset, ilbm1_planardata, ilbm1_planarsize);		
		planaroffset += ilbm1_planarsize;
		/* Write Palette Data */
		memcpy(os35data+planaroffset, ilbm1_planarcmap, ilbm1_cmapsize);
		planaroffset += ilbm1_cmapsize;
		/* Write IMAG Header */
		PutUint32(os35data+planaroffset, 0x494D4147); //ID_IMAG 0x494D4147
		imagsize = ilbm2_planarsize + ilbm2_cmapsize + 10;		
		PutUint32(os35data+planaroffset+4, imagsize);
		planaroffset += 8;
		/* Write IMAG Chunk */		
		memcpy(os35data+planaroffset, imagchunk2, 10);
		planaroffset += 10;
		/* Write Image Data */
		memcpy(os35data+planaroffset, ilbm2_planardata, ilbm2_planarsize);
		planaroffset += ilbm2_planarsize;
		/* Write Palette Data */
		memcpy(os35data+planaroffset, ilbm2_planarcmap, ilbm2_cmapsize);
		planaroffset += ilbm2_cmapsize;		

		/* Write to output file */
		//LONG iffoffset = 0; //for testing only!!
		Seek(fh,iffoffset,OFFSET_BEGINNING);
		//LONG os35minsize = 26;
		//if(os41 == FALSE) 		
        Write(fh, os35data, os35size);
		//else Write(fh, os35data, os35minsize);
	}	
    Close( fh );
	//imagsize = writeimagchunk(&img1, os35file);
	//Printf("First Real IMAG chunk size: %ld\n", imagsize);
	//if (infile2) imagsize = writeimagchunk(&img2);
	//Printf("Second Real IMAG chunk size: %ld\n", imagsize);

	/* Free Resources */
    if (os35data) FreeMem( os35data, os35size );

	//return iffoffset;
}

/**************************************************************************************************/

void Exchange_Icon(struct DiskObject *dobj1, struct DiskObject *dobj2, char *destname, WORD deltasize, struct Window *win)
{	    
	LONG imSize = 0;	
    BOOL retval = FALSE;	

	tempfilename = "tempicon.info";
	//tempfile = RemoveFileExtension(tempname);	

	/* Copy old icon attributes to new icon */    
	//retval = Save_Icon(dobj1, dobj2, tempfilename);	
    
    if(retval == TRUE)
	{
		/* Get temp icon file size */
	    imSize = GetFileSize(tempfilename);		

		/* Erase Window RasterPort "right" */	
	    EraseRect(win->RPort, leftoffset + 138 + 1 + deltasize, win->BorderTop + topoffset + 1 + deltasize,
            win->Width - 20 - win->BorderRight - deltasize,
            win->Height - 20 - win->BorderBottom - deltasize);		
	    
	    SetFilesize(destname,0); //SetFileSize(0)	    
		Copy_Data(tempfilename,destname,imSize);	 
	    		
		DeleteFile(tempfilename);	
	    Display_Image(destname, "right", deltasize, win);
	}
}

/**************************************************************************************************/

STRPTR GetDestinationFilename(char * asl_title, struct Window *win)
{
    STRPTR dstname = NULL;	
	if( DestFileReq == NULL )
    {
        DestFileReq = (struct FileRequester *)AllocAslRequestTags( ASL_FileRequest,
                                            ASLFR_Screen,          (win -> WScreen),                                            
											ASLFR_TitleText,       (IPTR)asl_title,                                            
                                            ASLFR_InitialLeftEdge, ((win -> LeftEdge) + (win -> BorderLeft) + 1UL),
                                            ASLFR_InitialTopEdge,  ((win -> TopEdge)  + (win -> BorderTop) + 1UL),
                                            ASLFR_DoPatterns,      TRUE,                                            
                                            TAG_DONE );
    }

    if( DestFileReq )
    {
        if( AslRequest( DestFileReq, NULL ) )
        {
            ULONG  buffsize;
            char *filebuffer = NULL;

            buffsize = strlen( (DestFileReq -> fr_Drawer) ) + strlen( (DestFileReq -> fr_File) ) + 10UL;

             if( filebuffer = (STRPTR)AllocVec( buffsize, MEMF_PUBLIC ) )
            {
                strcpy( filebuffer, (DestFileReq -> fr_Drawer) );
                AddPart( filebuffer, (DestFileReq -> fr_File), buffsize );
				dstname = filebuffer;                
            }
        }
    }
	else
	{
		/* User Cancelled */
		dstname = NULL;
	}
	return dstname;
}

/**************************************************************************************************/

STRPTR GetSourceFilename(char * asl_title, struct Window *win)
{	
	STRPTR source = NULL;
	if( SrcFileReq == NULL )
    {
        SrcFileReq = (struct FileRequester *)AllocAslRequestTags( ASL_FileRequest,
                                            ASLFR_Screen,          (win -> WScreen),                                            
											ASLFR_TitleText,       (IPTR)asl_title,                                            
                                            ASLFR_InitialLeftEdge, ((win -> LeftEdge) + (win -> BorderLeft) + 1UL),
                                            ASLFR_InitialTopEdge,  ((win -> TopEdge)  + (win -> BorderTop) + 1UL),
                                            ASLFR_DoPatterns,      TRUE,
                                            TAG_DONE );
    }

    if( SrcFileReq )
    {
        if( AslRequest( SrcFileReq, NULL ) )
        {
            ULONG  buffsize;
            char *filebuffer = NULL;

            buffsize = strlen( (SrcFileReq -> fr_Drawer) ) + strlen( (SrcFileReq -> fr_File) ) + 10UL;

             if( filebuffer = (STRPTR)AllocVec( buffsize, MEMF_PUBLIC ) )
            {
                strcpy( filebuffer, (SrcFileReq -> fr_Drawer) );
                AddPart( filebuffer, (SrcFileReq -> fr_File), buffsize );
				source = filebuffer;                
            }
        }
    }
	else
	{
		/* User Cancelled */
		source = NULL;
	}

    return source;
}

/**************************************************************************************************/

int getarguments(int argc, char **argv)
{
    int retvalue = 0;
	//BOOL help = FALSE;
	//BOOL open = FALSE;

	for (; argc > 1 && argv[1][0] == '-'; argc--, argv++) 
	{
        if (strcmp(argv[1],"--h") == 0) 
		{
			help = TRUE;            
            continue;
        }
        if (strcmp(argv[1],"--o") == 0) 
		{
			open = TRUE;            
            continue;
        }        
    }	
	
	if (help == TRUE)
	{
		/* Print help message */
        //Printf("Usage: iconpress [--h] [--o] [srce image] [dest image]\n");
		//Printf("Switch h: [iconpress --h] = show help\n");
		//Printf("Switch o: [iconpress --o] = open gui\n");
		retvalue = 0;
	}

	if (argc == 0)    
    {    	
		//Arguments = 0 //argv[1] = --o		
		retvalue = 1; //no srce //use gui
                //Printf ("Arguments: %d\n", argc);			    
    }	
    if ((argc == 1) && (open == TRUE))    
    {    	
		//Arguments = 1 //argv[1] = --o		
		retvalue = 1; //no srce //use gui
		//Printf ("Arguments: %d\n", argc);	    
    }	
	if ((argc == 2) && (open == TRUE))    
    {    	
		//Arguments = 2 //argv[1] = --o
		retvalue = 2; //srce present, use gui
		//Printf ("Arguments: %d\n", argc);	    
    }
	if ((argc == 3) && (open == TRUE))
    {
		//Arguments = 3 //argv[1] = --o    	
		retvalue = 4; //srce & dest present, use gui
		//Printf ("Arguments: %d\n", argc);	    
    }

	return retvalue;
}

/**************************************************************************************************/

char *str_dupe(char *src)
{
  int i;
  size_t len = strlen(src) + 1;
  char *dst = malloc(len);
  if(dst == NULL) return NULL;
  for(i=0; i=len; i++)
  {
    dst[i] = src[i];
  }
  //dst[len] = '/0';
  
  return dst;  
}

/**************************************************************************************************/

int main(int argc, char **argv)
{
	/* CLI Arguments & Switches */
	open = TRUE; //Open GUI
	action = getarguments(argc, argv);
	//Printf ("Action Value: %d\n", action);

	/* Action Values */
	//action = 0;
	//action = 1;
	//action = 2;
	//action = 3;
	//action = 4;	

	/* CLI switches */
	//--o (open gui)
	//--h (show help)	

	/* Possible return values */	
	//"iconclone" returns 0  //Wrong # of arguments 
    //"iconclone --h" returns 0  //Wrong # of arguments //Show help
    //"iconclone --o" returns 1  //Open empty GUI
    //"iconclone more.info" returns 0  //Wrong # of arguments
    //"iconclone --h more.info" returns 0  //Wrong # of arguments //Show help
    //"iconclone --o more.info" returns 2  //Open GUI & display srce icon
	//"iconclone more.info editor.info" returns 3  //Don't open GUI just Exchange icons
    //"iconclone --h more.info editor.info" returns 0  //Show help   
    //"iconclone --o more.info editor.info" returns 4  //Open GUI & display srce icon & dest icon
	//Open from Workbench returns 1  //Arguments = 0  //Open empty GUI

	/* Valid return values */	
	//"iconclone --o" returns 1
	//"iconclone --o more.info" returns 2
	//"iconclone more.info editor.info" returns 3
	//"iconclone --o more.info editor.info" returns 4
	/* Note: remove quotes when using CLI */


	//char *infilename1 = ""; //srce file				
    //char *infilename2 = ""; //dest file


	//Use Switch & Case:
    //switch (action)
	//{
	//case 1:
        //Arguments = 2 //No srce present, use gui
		//open = TRUE; //argv[1] = --o       														
		//break;
	//case 2:

	if (action == 1) //Arguments = 2 //No srce present, use gui
    {
    	open = TRUE; //argv[1] = --o
		//No srce icon //Open gui
    }
	if (action == 2) //Arguments = 3 //srce present, use gui
    {
		open = TRUE; //argv[1] = --o
    	//infilename1 = argv[2]; //srce icon
		infilename1 = argv[2]; //srce icon
		//Printf ("Srce Name: %s\n", infilename1);		
		/* Check for .info file extension */
    }	
	if (action == 4) //Arguments = 4 //srce & dest present, use gui
    {
		open = TRUE; //argv[1] = --o
    	//srcame = argv[2]; //srce icon				
    	//destname = argv[3]; //dest icon
		infilename1 = argv[2]; //srce icon				
    	infilename2 = argv[3]; //dest icon
		//Printf ("Srce Name: %s\n", infilename1);
		//Printf ("Dest Name: %s\n", infilename2);		
		/* Check for .info file extensions */	    
    }	

	
	/* User Interface based on several GadTools samples */
    struct Gadget *gad;		/* The Actual Gadget */
    struct NewGadget ng;	/* Various Information about new Gadgets */
    struct Screen *scr;
    struct Window *win;
    struct MsgPort *app_port;
    struct AppWindow *app_window;
    struct AppMessage   *amsg;
    struct IntuiMessage *imsg;
    struct DrawInfo *dri;
    BOOL reverse = FALSE;
    UBYTE filebuffer[300];
    UBYTE namebuffer[300];
    ULONG TOPIMAGE;
    ULONG LEFTIMAGE_X;
    ULONG RIGHTIMAGE_X;
    ULONG winmask;
    ULONG msgmask;
    ULONG sigs;
    BOOL cont;
    char *location;
    long winw,winh;
    
    char *pictureMessage;
	
    /* Allocate Memory for IconImages */
	//ilbm1 = (struct IconImage *)AllocMem(sizeof(struct IconImage), MEMF_CLEAR);
	//ilbm2 = (struct IconImage *)AllocMem(sizeof(struct IconImage), MEMF_CLEAR);


    if (scr = LockPubScreen (NULL))
	{
	    glist = NULL;
	    gad = CreateContext (&glist);	/* Here we initialize the Gadget List */

        if ((dri = GetScreenDrawInfo(scr)))
        {
    	    topoffset = dri->dri_Font->tf_YSize + scr->WBorTop - 10 + 5;			
	    }

	    /* We set these values for all Gadgets to use */
	    ng.ng_VisualInfo = GetVisualInfo (scr,TAG_END);
	    ng.ng_TextAttr   = scr->Font;
	    ng.ng_Flags      = 0;

	    leftoffset = scr->WBorLeft + 4;

	    /* The following block (including CreateGadget) is repeated for each new gadget. 
        Only the values ​​that change are adapted.
	    The previous gadget is passed to CreateGadget so that the 
        Concatenation can be established.
	    It is not necessary to check every time whether a result has been achieved.    
        If the previous gadget was NULL, all further gadgets will also be NULL, so  
        you only have to check once at the end. */

	    //struct NewGadget
        //{
        //WORD ng_LeftEdge, ng_TopEdge;  /* gadget position */
        //WORD ng_Width, ng_Height;      /* gadget size */
        //UBYTE *ng_GadgetText;          /* gadget label */
        //struct TextAttr *ng_TextAttr;  /* desired font for gadget label */
        //UWORD ng_GadgetID;             /* gadget ID */
        //ULONG ng_Flags;                /* see below */
        //APTR ng_VisualInfo;            /* Set to retval of GetVisualInfo() */
        //APTR ng_UserData;              /* gadget UserData */
        //};

	    /* Note: Use struct TextAttr *ng_TextAttr to specify the gadget font */
	    /* Default is to use the screen font & size which may be too large */	

	    ng.ng_LeftEdge   = scr->WBorLeft + 4 + 8 * scr->RastPort.TxWidth;
	    ng.ng_TopEdge    = scr->WBorTop + scr->RastPort.TxHeight + 5;
	    ng.ng_Width      = 20 * scr->RastPort.TxWidth + 80; //20
	    ng.ng_Height     = scr->RastPort.TxHeight + 6;

	    if(ng.ng_Height > 19)
	    {
	        buttonoffset     = ng.ng_Height - 19;
	        windowsize       = buttonoffset * 4;
	        deltasize        = windowsize / 2;
	    }
	    else
	    {
		    buttonoffset     = 0;
	        windowsize       = 0;
	        deltasize        = 0;
	    }
    

	    /* Center buttons inside outer groupbox */
	    //Outer groupbox = 228 + windowsize, 105 + windowsize

	//ng.ng_LeftEdge   = scr->WBorLeft + 4 + 10 * scr->RastPort.TxWidth;
	ng.ng_TopEdge   += ng.ng_Height + 4;
	ng.ng_Width      = 20 * scr->RastPort.TxWidth + 60;	

	//ng.ng_LeftEdge = 20 * scr->RastPort.TxWidth + scr->WBorLeft + 4;
	//ng.ng_LeftEdge = leftoffset + 105 + deltasize;
	//ng.ng_TopEdge   += ng.ng_Height + 4;
	//ng.ng_Width      = 20;
	//ng.ng_GadgetText = "?";
	//ng.ng_TextAttr = &topaz11;
	//ng.ng_TextAttr = &arial20;
	//ng.ng_GadgetID   = GID_REQ1;
	//butgad1 = gad = CreateGadget (BUTTON_KIND,gad,&ng,TAG_END);

	//ng.ng_LeftEdge = 20 * scr->RastPort.TxWidth + scr->WBorLeft + 0;
	//ng.ng_LeftEdge = leftoffset + 100 + deltasize;
	//ng.ng_TopEdge   += ng.ng_Height + 4;
	//ng.ng_Width      = 30;
	//ng.ng_GadgetText = "<->";
	//ng.ng_TextAttr = &topaz11;
	//ng.ng_TextAttr = &arial20;
	//ng.ng_GadgetID   = GID_EXCH;
	//butgad2 = gad = CreateGadget (BUTTON_KIND,gad,&ng,TAG_END);

	ng.ng_LeftEdge   = scr->WBorLeft + 74;
	ng.ng_TopEdge   += ng.ng_Height + 4;
	ng.ng_Width      = 20 * scr->RastPort.TxWidth + 60;

	//ng.ng_LeftEdge = 20 * scr->RastPort.TxWidth + scr->WBorLeft + 4;
	//ng.ng_LeftEdge = leftoffset + 105 + deltasize;
	//ng.ng_TopEdge   += ng.ng_Height + 4;	
	//ng.ng_Width      = 20;
	//ng.ng_GadgetText = "?";
	//ng.ng_TextAttr = &topaz11;
	//ng.ng_TextAttr = &arial20;
	//ng.ng_GadgetID   = GID_REQ2;
	//butgad3 = gad = CreateGadget (BUTTON_KIND,gad,&ng,TAG_END);

	//ng.ng_LeftEdge   = scr->WBorLeft + 4 + 10 * scr->RastPort.TxWidth;
	ng.ng_LeftEdge   = leftoffset + 2;
	//ng.ng_TopEdge    = scr->WBorTop + scr->RastPort.TxHeight + 5;
	ng.ng_TopEdge    = topoffset + 125 + windowsize; //12 + 113 = 125
	//ng.ng_TopEdge    = scr->WBorTop + 12 + 124;
	//ng.ng_Width      = 20 * scr->RastPort.TxWidth; //20
	ng.ng_Width      = 11 + deltasize + 80;	
	ng.ng_Height     = scr->RastPort.TxHeight + 6;
	ng.ng_GadgetText = "Icon Info"; //"Icon Type"
	ng.ng_GadgetID   = GID_SAV1;
	butgad1 = gad = CreateGadget (BUTTON_KIND,gad,&ng,TAG_END);

	//ng.ng_LeftEdge   = scr->WBorLeft + 4;
	//ng.ng_TopEdge   += ng.ng_Height + 4;
	//ng.ng_Width      = 15 * scr->RastPort.TxWidth + 8;
	
	ng.ng_LeftEdge   = leftoffset + 2 + ng.ng_Width + 10;	
	ng.ng_Width      = 11 + deltasize + 80;	
	ng.ng_Height     = scr->RastPort.TxHeight + 6;
	ng.ng_GadgetText = "Save Icon";
	ng.ng_GadgetID   = GID_SAV2;
	butgad2 = gad = CreateGadget (BUTTON_KIND,gad,&ng,TAG_END);

	ng.ng_LeftEdge   = scr->WBorLeft + 4;
	ng.ng_TopEdge   += ng.ng_Height + 4;
	ng.ng_Width      = 15 * scr->RastPort.TxWidth + 8;	

	ng.ng_LeftEdge  += ng.ng_Width + 64;
	
	
	    //winw = leftoffset + 13 + 228 + windowsize; //use 192 not 228 (-36)
	    winw = leftoffset + 13 + 192 + windowsize; //192 or 193?
	    winh = topoffset + 23 + 105 + windowsize + 30 + buttonoffset; //Add length to allow buttons
		//winh = topoffset + 23 + 105 + windowsize;	



	    	    
	    /* Open the GUI */
	    if (open == TRUE) //if(open)
	    {
	        if (gad)	/* Check whether all gadgets have been created */
	        {
		        //Create the AppMsg_Port
		        app_port = CreateMsgPort();		

		        /* When opening the window, the IDCMP flags for Gadtools gadgets must be specified. */
                /* The constants <typ>IDCMP are used for this. */
		
		        if (win = OpenWindowTags (NULL,
				WA_Width,winw,
				WA_Height,winh,
				WA_Left,(scr->Width - winw) / 2,	
				WA_Top,(scr->Height - winh) / 2,
				WA_PubScreen,scr,
				WA_Title,"Icon Drop v1.02",
				//WA_BlockPen,
				//WA_DetailPen,
				WA_Flags,WFLG_CLOSEGADGET | WFLG_DRAGBAR | WFLG_DEPTHGADGET | WFLG_ACTIVATE,
				WA_IDCMP,IDCMP_CLOSEWINDOW | IDCMP_VANILLAKEY | IDCMP_REFRESHWINDOW | BUTTONIDCMP | CYCLEIDCMP | STRINGIDCMP,
				WA_Gadgets,glist,
				TAG_END))
			    {
			        GT_RefreshWindow (win,NULL);	/* Redraw all Gadtools gadgets. */
											/* This has to be done once at the beginning. */

			        UnlockPubScreen (NULL,scr);		/* the window blocks the screen anyway, */
			        scr = NULL;						/* therefore the lock is no longer required here. */

                    /* DRAW FOUR BEVEL BOXES */
					//The two inner Bevel Boxes are raised panels.
	                //The two outer Bevel Boxes are the group boxes.
					//DrawBevelBox(rport, left, top, width, height, ...)	                	        

			        /* First Raised Panel */ //26,151,15-3 //leftoffset + 13,leftoffset + 138,leftoffset + 2		
                    DrawBevelBox(win->RPort, leftoffset + 13 + deltasize, 25 + topoffset + deltasize, 80, 80,
                         GT_VisualInfo, (IPTR) ng.ng_VisualInfo, TAG_DONE);
	                /* Second Raised Panel */ //-9 now 129 was 138
                    DrawBevelBox(win->RPort, leftoffset + 102 + deltasize, 25 + topoffset + deltasize, 80, 80,
                         GT_VisualInfo, (IPTR) ng.ng_VisualInfo, TAG_DONE);    				 
	                /* First Outer Group Box - Left */ //138-36=102
                    DrawBevelBox(win->RPort, leftoffset + 2, 12 + topoffset, 192 + windowsize, 105 + windowsize,
                         GTBB_FrameType, BBFT_RIDGE, GTBB_Recessed, TRUE,
                         GT_VisualInfo, (IPTR) ng.ng_VisualInfo, TAG_DONE); //228-36=192
					/* Second Outer Group Box - Right*/
                    //DrawBevelBox(win->RPort, leftoffset + 2, 12 + topoffset, 228 + windowsize, 105 + windowsize,
                         //GTBB_FrameType, BBFT_RIDGE, GTBB_Recessed, TRUE,
                         //GT_VisualInfo, (IPTR) ng.ng_VisualInfo, TAG_DONE);

                    /* Offsets used for Rectangle Coords */
			        TOPIMAGE = 25 + topoffset + deltasize;
			        LEFTIMAGE_X = leftoffset + 13 + deltasize; //leftoffset + 13 + deltasize
			        RIGHTIMAGE_X = leftoffset + 102 + deltasize; //leftoffset + 102 + deltasize		

            

			        /* Open GUI & Display Icons */
	                //if ((strlen(infilename1) > 0) && (open == TRUE)) //action = 2,4
			        //if ((strlen(srcename) > 0) && (open == TRUE))
					if ((action == 2) || (action == 4))
                    {								
                        /* Srce Icon */																	    
		                reverse = FALSE;
		                location = "left";
		                /* Srce Icon File has .info Extension */
		                //srcefile = RemoveFileExtension(infilename1);
				        //if(srcename) srcefile = RemoveFileExtension(srcename);
				        if(infilename1) //Printf ("Srce File: %s\n", infilename1);		
		                //srcedobj = GetDiskObjectNew(srcefile);

		                /* Display Srce Icon Image */		
		                Display_Image(infilename1, location, deltasize, win);				    
                    }	                
	                //if ((strlen(infilename2) > 0) && (open == TRUE)) //action = 4
			        //if ((strlen(destname) > 0) && (open == TRUE))
					if (action == 4)
                    {
		                /* Dest Icon */
		                reverse = FALSE;
		                location = "right";
		                /* Dest Icon File has .info Extension */
		                //destfile = RemoveFileExtension(infilename2);
				        //if(destname) destfile = RemoveFileExtension(destname);
				        if(infilename2) //Printf ("Dest File: %s\n", infilename2);
		                //destdobj = GetDiskObject(destfile);

		                /* Display Dest Icon Image */							
                        Display_Image(infilename2, location, deltasize, win);				    
                    }
			
	        

                    /* Setup an AppWindow using MsgPort app_port */
			        if(!(app_window = AddAppWindow(0, 0, win, app_port, NULL)))
                    {
                        /* Can't Add Appwindow! */
				        Printf ("Can't Add Appwindow!\n");
			        }
                    else
			        {				
				        /* AppWindow added... Drag Icons into AppWindow */
				        //Printf("AppWindow added... Drag Icons into AppWindow\n");               
				        /* Setup our signals */
			            winmask = 1L << win->UserPort->mp_SigBit;
			            msgmask = 1L << app_port->mp_SigBit;			


			            /* This is followed by a normal input query for a window. */
                        /* Gadtools functions GT_GetIMsg & GT_ReplyIMsg are used instead of GetMsg & ReplyMsg */
			            cont = TRUE;
			            do	
			            {
				            /* Wait for IDCMP & AppMessages */				
				            sigs = Wait(winmask | msgmask);
                                               				
				            if (sigs & msgmask) /* Got an AppMessage */
				            { 
					            /* User dropped an Icon! Get the message from the app_port */					
				                while ( (app_port) && (amsg = (struct AppMessage *) GetMsg(app_port)) )
                                {  
							        // Now we know that a file was dropped onto the app window.
                                    // So first lets determine if the user dropped an icon image in the left or right
							        // image display area so that we know where to display the image that was dropped.
									
									//namebuffer[0] = 0;
									//filebuffer[0] = 0;

									//Printf ("File dropped: mouseX %d, mouseY %d\n", amsg->am_MouseX, amsg->am_MouseY);
									                            
                                    if (inRectangle(LEFTIMAGE_X, TOPIMAGE, MAX_IMAGE_WIDTH, MAX_IMAGE_HEIGHT, amsg->am_MouseX, amsg->am_MouseY))
                                    {
                                        /* Icon dropped in Left box */
								        //Printf ("File dropped in Left box\n");													    
								        location = "left";								

								        /* Erase Image Left */ 
								        EraseRect(win->RPort, leftoffset + 13 + 1 + deltasize, win->BorderTop + topoffset + 1 + deltasize,
                                        win->Width - 109 - win->BorderRight - deltasize,
                                        win->Height - 50 - win->BorderBottom - deltasize);

										/* Assign first image */
										if (amsg->am_NumArgs >= 1)							            
                                        {
                                            NameFromLock(amsg->am_ArgList->wa_Lock, namebuffer, 299);
                                            AddPart(namebuffer, amsg->am_ArgList->wa_Name, 299);
                                            //infilename1 = str_dupe(namebuffer);
                                            infilename1 = namebuffer; //Get unique filename using double STRPTR method                                             
                                            filename = infilename1;								            
											//Printf ("Drag-n-Drop Srce File: %s\n", infilename1);                                
                                        }

								        /* Assign first image */
								        //infilename1 = (filename);
										//copy the dropped filename into first input filename
                                        //strncpy(infilename1, filename, strlen(filename));
										//last char position is for the null terminator
                                        //infilename1[strlen(filename)] = '\0'; // place the null terminator
                                        //Printf ("Drag-n-Drop Srce File: %s\n", infilename1);			                                 
                                    }
                                    else if (inRectangle(RIGHTIMAGE_X, TOPIMAGE, MAX_IMAGE_WIDTH, MAX_IMAGE_HEIGHT, amsg->am_MouseX, amsg->am_MouseY))
                                    {
                                       /* Icon dropped in Right box */
								        //Printf ("File dropped in Right box\n");							    
							            location = "right";								

								        /* Erase Image Right */ //EraseRect( rp, xmin, ymin, xmax, ymax) a1  d0:16 d1:16 d2:16 d3:16
										//win->BorderTop + topoffset + 31 + deltasize //102 not 138
								        EraseRect(win->RPort, leftoffset + 102 + 1 + deltasize, win->BorderTop + topoffset + 1 + deltasize,
                                        win->Width - 20 - win->BorderRight - deltasize,
                                        win->Height - 50 - win->BorderBottom - deltasize);

										/* Assign second image */
										if (amsg->am_NumArgs >= 1)							            
                                        {
                                            NameFromLock(amsg->am_ArgList->wa_Lock, filebuffer, 299);
                                            AddPart(filebuffer, amsg->am_ArgList->wa_Name, 299);
                                            infilename2 = filebuffer;
                                            //infilename2 = str_dupe(namebuffer);											
                                            filename = infilename2;
											//Printf ("Drag-n-Drop Dest File: %s\n", infilename2);                                
                                        }

								        /* Assign second image */
								        //infilename2 = (filename); 
										//copy the dropped filename into second input filename
                                        //strncpy(infilename2, filename, strlen(filename));
                                        //memcpy(infilename2, filename, strlen(filename));
										//last char position is for the null terminator
                                        //infilename2[strlen(filename)] = '\0'; // place the null terminator
                                        //Printf ("Drag-n-Drop Dest File: %s\n", infilename1);                 
                                    }
                                    if (filename)
                                    {								
                                        //Display_Blank(filename, location, deltasize, win);
					Display_Image(filename, location, deltasize, win);
					//Display_Rectangle("red", location, deltasize, win);
					//Display_Rectangle("green", location, deltasize, win);
					//Display_Rectangle("blue", location, deltasize, win);
                                    }
                        

                                    ReplyMsg ((struct Message *) amsg);						
                                    ActivateWindow(win);                        
                                } /* while ((amsg = (struct AppMessage *) GetMsg(app_port))) */
				            } //Sigs & msgmask


				            // User clicked a gadget or something				      
                            if (sigs & winmask) /* Got an IDCMP message */
                            {
				                while (imsg = GT_GetIMsg (win->UserPort))
				                {
					                switch (imsg->Class)
						            {
					                case IDCMP_GADGETUP:
						                gad = (struct Gadget *) imsg->IAddress;
						                switch (gad->GadgetID)
						                {												
						                case GID_SAV1:							                
											/* This is same as TYPE or INFO */
											if(infilename1)
											{
                                                /* Display Icon Image Information */
                                                pictureMessage = malloc(260 * sizeof(char));
											    char *pictureType = NULL;
											    if(stricmp(imagetype, "png")==0)
                                                {
                                                    pictureType = "PNG";
                                                }
											    /* Find Width, Height & Depth of First Image */
											    struct BitMapHeader *bmhd;
                                                bmhd = readbmhd_by_datatype(infilename1);
	                                            int wd = bmhd->bmh_Width;
	                                            int ht = bmhd->bmh_Height;
	                                            int de = bmhd->bmh_Depth;
											    /* Show version string with date */                                    
                                                char *codeVersion = ("Icon Drop v1.0.1 by Mike R. Build: 12-18-23");
											    /* Show Width x Height, Depth of First Image */
											    sprintf(pictureMessage, "\nIcon Type: %s\n\n%s\n\nIcon Data: Width %d x Height %d , Depth: %d\n", pictureType, codeVersion, wd, ht, de);
                                                //EasyRequest(NULL, &ResultReq, NULL, pictureMessage);
											    OutputMessage(pictureMessage, win);
                                                free(pictureMessage);
	                                            pictureMessage = NULL;
											}
											else
											{
												OutputMessage("Can't Find First Icon Image", win);
											}							                
							                break;
						                case GID_SAV2:
							                /* Get Output Filename */
							                outfilename = GetDestinationFilename("File To Save...", win);
							                //Printf ("Output Filename: '%s'\n", outfilename);
							                if(outfilename != NULL && strlen(outfilename) > 0)							
							                {		
												//If destination .info file exists prompt
												//"Destination icon file exists. Would you like to exchange?" YES/NO
												//"Save Icon"					                    
												if(stricmp(imagetype, "png")==0)
												{
													/* Save a PNG Icon */ //Copy_Data, Copy_Data_Append
													//int iconcount = 0;
													//if(infilename1) iconcount++;
													//if(infilename2) iconcount++;
													//if(iconcount == 2)
													Save_Icon(infilename1, infilename2, outfilename);
													//else if(iconcount == 1)
													//Save_Single_Icon(infilename1, outfilename);
												}
												else if(stricmp(imagetype, "iff")==0)
												{
													/* Save a Classic Amiga Icon with OS35 Extension */
													//char *iconfilename = GetSourceFilename("Icon To Open...", win);
													//Save_Classic_Icon(infilename1, infilename2, outfilename, FALSE);
													//Printf ("Not Implemented. Can't save filename: '%s'\n", outfilename);
													//retval = Save_Amiga_Icon(dobj1, dobj2, "Sys:MyWorkspace/Icons/Test/tempicon.info");
													//Save_Amiga_Icon(infilename1, infilename2, outfilename, iconfilename, icontype);
												}																				
							                }
							                else
							                {								
								                //Printf ("\nNothing Selected!\n");
                                                outfilename = NULL;
							                }														
							                break;						                						
						                }
						            break;
					                case IDCMP_VANILLAKEY:
						                if (imsg->Code == 0x1b) /* Esc */
							                cont = FALSE;
						                break;
					                case IDCMP_CLOSEWINDOW:
						                cont = FALSE;
						                break;
					                case IDCMP_REFRESHWINDOW:
						                GT_BeginRefresh(win);
						                GT_EndRefresh(win, TRUE);
						                break;
						            }
					                GT_ReplyIMsg (imsg);
				                }
				            } //Sigs & winmask
				
			            } //Do-While Loop
			            while (cont); /* done */
			            //while ((cont) && (open)); /* done */
			            RemoveAppWindow(app_window);
			        } //AppWindow added

			    CloseWindow (win);
			    } //if (win)
			    /* Make sure there are no more outstanding messages */
                while(amsg = (struct AppMessage *)GetMsg(app_port))
                  ReplyMsg((struct Message *)amsg);                  
                DeleteMsgPort(app_port);			
		    } //if (gad)
        } //Open the GUI
    
	FreeGadgets (glist);

	FreeVisualInfo (ng.ng_VisualInfo);	

	if (scr) UnlockPubScreen (NULL,scr);
	}
	//FreeDiskObject(srcedobj);
	//FreeDiskObject(destdobj);

	//FreeMem( ilbm1, sizeof(struct IconImage) );
	//FreeMem( ilbm2, sizeof(struct IconImage) );

return (0);
}